package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModel.java
//    System: VModel
//   Version: 0.4
//    Author: Leo C. Ureel II
//   Created: June 21, 2001 12:10:52
//   Purpose: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Thursday, June 21, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.lang.*;
import javax.swing.*;
import java.applet.*;
import java.awt.*;
import java.net.*;
import java.util.*;
import java.io.*;

// -----------------------------------------------------------------------------
// NOTES: 
// * VModelSystem (the superclass) contains its own main function for testing.
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Class: VModel
// Description: VModel executable class.  This is the primary class used to
//              organize and control access to the modules of the VModel system.  
// -----------------------------------------------------------------------------
public class VModel extends VModelSystem implements VModelConstants {
    
	// Properties and Accessors
	// --------------------------------------------------------------------------
	protected boolean forceLogin = true;
	public boolean forceLogin ( ) { return(forceLogin); }
	public boolean forceLogin ( boolean f ) { return(forceLogin = f); }

	protected VModelCoach coach = null;
	public VModelCoach coach ( ) { return(coach); }
	public VModelCoach coach ( VModelCoach c ) { return(coach = c); }
		
	protected VModelGUI gui = null;
	public VModelGUI gui ( ) { return(gui); }
	public VModelGUI gui ( VModelGUI g ) { return(gui = g); }
		
	// Methods
	// -----------------------------------------------------------------------
	public void updatePreferences ( ) {
//		gui().updatePreferences();
	} // method updatePreferences

	public void save ( ) {
		super.save();
		//preferences().localPreferences().save();
	} // method save

	public void quit ( int rtn ) {
		super.quit(rtn);
	} // method quit
	
	public boolean superUser ( ) { return(SUPERUSER || preferences().getBoolean("STUDENT", "SUPERUSER") ); }

	// Initialization
	// -----------------------------------------------------------------------
	public void vmodelInit() {

		QRGutils.debug(TESTRUN, "VModel.dbg", 500000);
		if (QRGutils.isApplet()) {
			URL foo = null;
			String bar = "";
			try { 
				foo = new URL(getCodeBase().toString()); 
				bar = foo.toString();
			} catch (Exception e) { bar = getCodeBase().toString(); }
			QRGutils.QRGroot(bar);
		} else {
			String userdir =  System.getProperty("user.dir");
			URL fu = getClass().getResource("vmodel.ini");
			String fubar = null;
			if (fu != null) {
				QRGutils.debug("VModelSystem","vmodelInit fu=" + fu);
				fubar = fu.getFile();
				if (fubar != null) fubar = (new File(fubar)).getParent() + "/";
			} else {
				//String userdir = System.getProperty("user.data");
				QRGutils.debug("VModelSystem","vmodelInit -->userdir=" + userdir);
				File userdirpath = new File(userdir);
				File userinifile = new File(userdirpath, "vmodel.ini");
				while((userinifile != null) && (!userinifile.exists())) {
					userdir = userdirpath.getParent(); 
					QRGutils.debug("VModelSystem","vmodelInit userdir=" + userdir);
					if (userdir != null) {
						userdirpath = new File(userdir);
						userinifile = new File(userdirpath, "vmodel.ini");
					} else userinifile = null;
				} // end while
				QRGutils.debug("VModelSystem","vmodelInit userinifile=" + userinifile);
				if (userinifile != null) fubar = userinifile.getParent() + "/";
			} // end if fu
			QRGutils.debug("VModelSystem","vmodelInit fubar=" + fubar);
			if (fubar != null) QRGutils.QRGroot(fubar);
			else QRGutils.QRGroot(VMODELROOT);
		} // end if is applet
		preferences(new VModelPreferences(this));
		preferences().loadDataPrefs();
		//ontology(new VModelOntology(this));
		System.gc();
			gui(new VModelGUI(this));
	} // method VModel

	// Applet Methods
	// -----------------------------------------------------------------------
	public void init () {
		setBackground(Color.blue);
		repaint();
		QRGutils.isApplet(this);
		super.init();
	} // method init

	public void start () {
		super.start();
	} // method start

	public void stop () {
		super.stop();
	} // method stop

	public void destroy () {
		super.destroy();
	} // method destroy

	public void paint (Graphics g) {
		super.paint(g);
	} // method paint

	// Application Methods
	// -----------------------------------------------------------------------
	public static void main (String[] args) {
		VModel system = null;
QRGutils.debug("VModel", "main");
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) { e.printStackTrace(); }

		system = new VModel();
		QRGutils.isApplet(false);
		system.vmodelInit();
	} // method main

	// Constructor
	// --------------------------------------------------------------------------
	public VModel() { super(); } // default constructor

} // class VModel

// -----------------------------------------------------------------------------
// End of File: VModel.java                     (c) 2001 Northwestern University
