package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelFileFilter.java
//    System: VModel
//   Version: 4.0
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: The VModel main window.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Thursday, June 21, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.awt.*;
import javax.swing.*;
import java.util.*;
import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.UIManager.*;
import java.io.*;
import javax.swing.filechooser.*;

// -----------------------------------------------------------------------------
// Class: VModelFileFilter
// Description: VModel GUI manager. 
// -----------------------------------------------------------------------------
public class VModelFileFilter extends FileFilter {

    private String TYPE_UNKNOWN = "Type Unknown";
    private String HIDDEN_FILE = "Hidden File";

    private Hashtable filters = null;
    private String description = null;
    private String fullDescription = null;
    private boolean useExtensionsInDescription = true;

    public VModelFileFilter() { this.filters = new Hashtable(); }

    public VModelFileFilter(String extension) { this(extension,null); }

    public VModelFileFilter(String extension, String description) {
		this();
		if(extension!=null) addExtension(extension);
	 	if(description!=null) setDescription(description);
    }

    public VModelFileFilter(String[] filters) { this(filters, null); }

    public VModelFileFilter(String[] filters, String description) {
		this();
		for (int i = 0; i < filters.length; i++) {
		    // add filters one by one
		    addExtension(filters[i]);
		}
	 	if(description!=null) setDescription(description);
    }

    public boolean accept(File f) {
		if(f != null) {
		    if(f.isDirectory()) {
				return true;
		    }
		    String extension = getExtension(f);
		    if(extension != null && filters.get(getExtension(f)) != null) {
			return true;
		    };
		}
		return false;
    }

     public String getExtension(File f) {
	if(f != null) {
	    String filename = f.getName();
	    int i = filename.lastIndexOf('.');
	    if(i>0 && i<filename.length()-1) {
		return filename.substring(i+1).toLowerCase();
	    };
	}
	return null;
    }

    public void addExtension(String extension) {
	if(filters == null) {
	    filters = new Hashtable(5);
	}
	filters.put(extension.toLowerCase(), this);
	fullDescription = null;
    }

    public String getDescription() {
	if(fullDescription == null) {
	    if(description == null || isExtensionListInDescription()) {
 		fullDescription = description==null ? "(" : description + " (";
		// build the description from the extension list
		Enumeration extensions = filters.keys();
		if(extensions != null) {
		    fullDescription += "." + (String) extensions.nextElement();
		    while (extensions.hasMoreElements()) {
			fullDescription += ", " + (String) extensions.nextElement();
		    }
		}
		fullDescription += ")";
	    } else {
		fullDescription = description;
	    }
	}
	return fullDescription;
    }

    public void setDescription(String description) {
	this.description = description;
	fullDescription = null;
    }

    public void setExtensionListInDescription(boolean b) {
	useExtensionsInDescription = b;
	fullDescription = null;
    }

    public boolean isExtensionListInDescription() {
	return useExtensionsInDescription;
    }
} // innerclass

	public class VModelFileView extends FileView {
	    private Hashtable icons = new Hashtable(5);
	    private Hashtable fileDescriptions = new Hashtable(5);
	    private Hashtable typeDescriptions = new Hashtable(5);
	
	    public String getName(File f) {
			return null;
	    }
	
	    public void putDescription(File f, String fileDescription) {
			fileDescriptions.put(fileDescription, f);
	    }
	
	    public String getDescription(File f) {
			return (String) fileDescriptions.get(f);
	    };
	
	    public void putTypeDescription(String extension, String typeDescription) {
			typeDescriptions.put(typeDescription, extension);
	    }
	
	    public void putTypeDescription(File f, String typeDescription) {
			putTypeDescription(getExtension(f), typeDescription);
	    }
	
	    public String getTypeDescription(File f) {
			return (String) typeDescriptions.get(getExtension(f));
	    }
	
	    public String getExtension(File f) {
			String name = f.getName();
			if(name != null) {
			    int extensionIndex = name.lastIndexOf('.');
			    if(extensionIndex < 0) {
				return null;
			    }
			    return name.substring(extensionIndex+1).toLowerCase();
			}
			return null;
	    }
	
	    public void putIcon(String extension, Icon icon) {
			icons.put(extension, icon);
	    }
	
	    public Icon getIcon(File f) {
			Icon icon = null;
			String extension = getExtension(f);
			if(extension != null) {
			    icon = (Icon) icons.get(extension);
			}
			return icon;
	    }
	
	    public Boolean isHidden(File f) {
			String name = f.getName();
			if(name != null && !name.equals("") && name.charAt(0) == '.') {
			    return Boolean.TRUE;
			} else {
			    return Boolean.FALSE;
			}
	    };
	
	    public Boolean isTraversable(File f) {
			if(f.isDirectory()) {
			    return Boolean.TRUE;
			} else {
			    return Boolean.FALSE;
			}
	    };
	
	} // innerclass

    
// -----------------------------------------------------------------------------
// End of File: VModelGUI.java                  (c) 2001 Northwestern University
