package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModel.java
//    System: VModel
//   Version: 0.4
//    Author: Leo C. Ureel II
//   Created: June 21, 2001 12:10:52
//   Purpose: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Thursday, June 21, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.lang.*;
import javax.swing.*;
import java.applet.*;
import java.awt.*;
import java.net.*;
import java.util.*;
import java.io.*;

// -----------------------------------------------------------------------------
// NOTES: 
// 1. 1st cut shares everything. Methods should be added later. -ureel
// 2. 2nd cut properties are protected. Only accessors should be used. -ureel
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Class: VModel
// Description: VModel executable class.  This is the primary class used to
//              organize and control access to the modules of the VModel system.  
// -----------------------------------------------------------------------------
public class VModelSystem extends JApplet implements VModelConstants {
    
	// Properties and Accessors
	// --------------------------------------------------------------------------
	protected VModelPreferences preferences = null;
	public VModelPreferences preferences ( ) { return(preferences); }
	public VModelPreferences preferences ( VModelPreferences p ) { return(preferences = p); }
		
	protected VModelOntology ontology = null;
	public VModelOntology ontology ( ) { return(ontology); }
	public VModelOntology ontology ( VModelOntology o ) { return(ontology = o); }
		
	// Methods
	// -----------------------------------------------------------------------
	public void updatePreferences ( ) {

	} // method updatePreferences

	public void save ( ) {
		updatePreferences();
	} // method save

	public void quit ( int rtn ) {
		save( );
		QRGutils.debug("", "Be seeing you!");
		QRGutils.debug(false);
		System.gc( );
		if (!QRGutils.isApplet()) System.exit( rtn );
	} // method quit
	
	

	// Initialization
	// -----------------------------------------------------------------------
	public void vmodelInit() {
		QRGutils.debug(TESTRUN, "VModel.dbg", 500000);
		if (QRGutils.isApplet()) {
			URL foo = null;
			String bar = "";
			try { 
				foo = new URL(getCodeBase().toString()); 
				bar = foo.toString();
			} catch (Exception e) { bar = getCodeBase().toString(); }
			QRGutils.QRGroot(bar);
		} else {
			String userdir =  System.getProperty("user.dir");
			URL fu = getClass().getResource("vmodel.ini");
			String fubar = null;
			if (fu != null) {
				QRGutils.debug("VModelSystem","vmodelInit fu=" + fu);
				fubar = fu.getFile();
				if (fubar != null) fubar = (new File(fubar)).getParent() + "/";
			} else {
				//String userdir = System.getProperty("user.data");
				QRGutils.debug("VModelSystem","vmodelInit -->userdir=" + userdir);
				File userdirpath = new File(userdir);
				File userinifile = new File(userdirpath, "vmodel.ini");
				while((userinifile != null) && (!userinifile.exists())) {
					userdir = userdirpath.getParent(); 
					QRGutils.debug("VModelSystem","vmodelInit userdir=" + userdir);
					if (userdir != null) {
						userdirpath = new File(userdir);
						userinifile = new File(userdirpath, "vmodel.ini");
					} else userinifile = null;
				} // end while
				QRGutils.debug("VModelSystem","vmodelInit userinifile=" + userinifile);
				if (userinifile != null) fubar = userinifile.getParent() + "/";
			} // end if fu
			QRGutils.debug("VModelSystem","vmodelInit fubar=" + fubar);
			if (fubar != null) QRGutils.QRGroot(fubar);
			else QRGutils.QRGroot(VMODELROOT);
		} // end if is applet
		preferences(new VModelPreferences(this));
		preferences().loadDataPrefs();
		ontology(new VModelOntology(this));
		System.gc();
	} // method vmodelInit

	// Applet Methods
	// -----------------------------------------------------------------------
	public void init () {
		super.init();
		QRGutils.isApplet(true);
		vmodelInit();
	} // method init

	public void start () {
		super.start();
	} // method start

	public void stop () {
		super.stop();
	} // method stop

	public void destroy () {
		super.destroy();
	} // method destroy

	public void paint (Graphics g) {
		super.paint(g);
	} // method paint

	// Application Methods
	// -----------------------------------------------------------------------
	public static void main (String[] args) {
		VModelSystem system = null;

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) { e.printStackTrace(); }

		system = new VModelSystem();
		QRGutils.isApplet(false);
		system.vmodelInit();
	} // method main

	// Constructor
	// --------------------------------------------------------------------------
	public VModelSystem() { super(); } // default constructor

} // class VModel

// -----------------------------------------------------------------------------
// End of File: VModelSystem.java               (c) 2001 Northwestern University
