package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelTest.java
//    System: VModel
//   Version: 4.0
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: Test facilities.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Thursday, June 21, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.util.*;
import java.io.FileInputStream;
import java.awt.*;

// -----------------------------------------------------------------------------
// Notes:
// * 
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Class: VModelTest 
// Description: Test facility.
// -----------------------------------------------------------------------------
public class VModelTest implements VModelConstants {
	// Properties & Accessors
	// --------------------------------------------------------------------------
	protected static VModel system = null;
	public static VModel system ( ) { return(system); }
	public static VModel system ( VModel s ) { return(system = s); }

	// Preferences tests
	// --------------------------------------------------------------------------
	public static void printLocalPreferences(VModel system) {
		QRGutils.debug("VModelTest", "Local Preferences:\n" +
							system.preferences().localPreferences());
	} // method printLocalPreferences

	public static void printGlobalPreferences(VModel system) {
		QRGutils.debug("VModelTest", "Global Preferences:\n" +
							system.preferences().globalPreferences());
	} // method printGlobalPreferences

	public static void printDataPreferences(VModel system) {
		QRGutils.debug("VModelTest", "Data Preferences:\n" +
							system.preferences().dataPreferences());
	} // method printDataPreferences

	public static void printPreferences(VModel system) {
		printLocalPreferences(system);
		printGlobalPreferences(system);
		printDataPreferences(system);
	} // method printPreferences

	// Ontology tests
	// --------------------------------------------------------------------------
	public static void printOntologies(VModel system) {
		QRGutils.debug("VModelTest", "ontologies:\n" +
							system.ontology().ontologies());
		QRGutils.debug("VModelTest", "find(\"Library\", \"OBJECT\", \"Thing\"):\n" +
							system.ontology().find("Library", "OBJECT", "Thing"));
		QRGutils.debug("VModelTest", "find(\"BASIC ENTITIES\", \"OBJECT\", \"SUBSTANCE\"):\n" +
							system.ontology().find("BASIC ENTITIES", "OBJECT", "SUBSTANCE"));
		QRGutils.debug("VModelTest", "find(\"OBJECT\", \"PRINCIPLE\"):\n" +
							system.ontology().find("OBJECT", "PRINCIPLE"));
		QRGutils.debug("VModelTest", "find(\"GROUPS-ONTOLOGY-LIBRARY\"):\n" +
							system.ontology().find("GROUPS-ONTOLOGY-LIBRARY"));
		QRGutils.debug("VModelTest", "find(\"OBJECTS-ONTOLOGY-LIBRARY\"):\n" +
							system.ontology().find("OBJECTS-ONTOLOGY-LIBRARY"));
	} // method printOntologies

	// Model tests
	// --------------------------------------------------------------------------
	public static void printAllVModelElements() {
		for(int i = 0; i < VModelElement.allVModelElements().size(); i++) 
			QRGutils.debug("VModelTest AllElements("+i+")", VModelElement.allVModelElements().elementAt(i));
	} // method printAllVModelElements

	// Model tests
	// --------------------------------------------------------------------------
	public static void loadModel(VModel system) {
		//VModelModel model = VModelModel.load(QRGutils.makeQRGpath("resources2/models") + "foo.vmf");
		VModelModel model = VModelModel.load(QRGutils.makeQRGpath("resources2/models") + "test1.vmf");
		model.log(true);
		QRGutils.debug("VModelTest", "MODEL "+model.modelElement());
		QRGutils.debug("VModelTest", "MODEL "+model.modelHistory);
		model.makeEntity(system.ontology().find("Library", "OBJECT", "Thing"));
	} // method loadModel

	public static void makeElephant(VModel system) {
		VModelModel model = new VModelModel(system);
		VModelElement elephant = null;
		VModelElement weight = null;
		VModelElement elephantHasWeight = null;
		model.log(true);
		elephant = model.addObject("THING");
		model.label(elephant, "elephant");
		weight = model.addAttribute("AMOUNT");
		elephantHasWeight = model.addRelationship("HAS-DESCRIBER");
		model.connect(elephantHasWeight, elephant, weight);

		model.addToOntology(elephant);

		QRGutils.debug("VModelTest", "makeElephant elephant=\n" + elephant);
	} // method makeElephant

	public static void makeTrixie(VModel system) {
		VModelModel model = new VModelModel(system);
		VModelElement trixie = null, food = null, fruit = null;
		VModelElement eats = null;
		VModelElement weight = null;
		VModelElement eatingIncreasesWeight = null;
		VModelElement eatingDecreasesFood = null;
		VModelElement trixieDoesEat = null;
		VModelElement trixieHasWeight = null;
		VModelElement trixieGroup = null;
		model.log(true);
		trixie = model.addObject(system.ontology().find("OBJECTS-ONTOLOGY-LIBRARY"), "elephant");
		model.label(trixie, "Trixie the Elephant");

		food = model.addObject("THING");
		model.label(food, "Food");
		model.variablize(food);
		eats = model.addObject("PROCESS");
		model.label(eats, "Eats");
	   weight = trixie.findElement(new VModelAttribute("LABEL","Weight"));
		eatingIncreasesWeight = model.addRelationship("INCREASES");
		model.connect(eatingIncreasesWeight, eats, weight);
		eatingDecreasesFood = model.addRelationship("DECREASES");
		model.connect(eatingDecreasesFood, eats, food);
		trixieDoesEat = model.addRelationship("DOES");
		model.connect(trixieDoesEat, trixie, eats);
		
		trixieGroup = model.makeGroup();
		model.label(trixieGroup, "Trixie");
		model.addGroup(trixieGroup);
		model.addToGroup(trixieGroup, trixie);
		model.addToGroup(trixieGroup, food);
		model.addToGroup(trixieGroup, eats);
		model.addToGroup(trixieGroup, eatingIncreasesWeight);
		model.addToGroup(trixieGroup, eatingDecreasesFood);
		model.addToGroup(trixieGroup, trixieDoesEat);

		fruit = model.addObject("THING");
		model.label(fruit, "Fruits, Nuts, and Berries");
		model.bind(food, fruit);
		
		model.addToOntology(trixieGroup);

		QRGutils.debug("VModelTest", "makeTrixie trixie=\n" + trixieGroup);
	} // method makeTrixie

	public static void makeTrixieModel(VModel system) {
		VModelModel model = new VModelModel(system);
		VModelElement trixie = null, jim = null, food = null, fruit = null;
		VModelElement sings = null, eats = null;
		VModelElement weight = null;
		VModelElement eatingIncreasesWeight = null;
		VModelElement eatingDecreasesFood = null;
		VModelElement jimDoesSing = null;
		VModelElement trixieDoesEat = null;
		VModelElement eatingRequiresSinging = null;
		VModelElement trixieHasWeight = null;
		VModelElement trixieGroup = null, jimGroup = null;
		model.log(true);
		trixieGroup = model.addGroup(system.ontology().find("GROUPS-ONTOLOGY-LIBRARY"), "Trixie");
		trixie = trixieGroup.findElement(new VModelAttribute("LABEL","Trixie the Elephant"));
	   weight = trixieGroup.findElement(new VModelAttribute("LABEL","Weight"));
	   food = trixieGroup.findElement(new VModelAttribute("LABEL","Food"));
	   eats = trixieGroup.findElement(new VModelAttribute("LABEL","Eats"));
	   eatingIncreasesWeight = trixieGroup.findElement(new VModelAttribute("TYPE","INCREASES"));
	   eatingDecreasesFood = trixieGroup.findElement(new VModelAttribute("TYPE","DECREASES"));
	   trixieDoesEat = trixieGroup.findElement(new VModelAttribute("TYPE","DOES"));

		fruit = model.addObject("THING");
		model.label(fruit, "Fruits, Nuts, and Berries");
		model.bind(food, fruit);
		
		model.addToOntology(trixieGroup);

		jim = model.addObject("THING");
		model.label(jim, "Jim");
		sings = model.addObject("PROCESS");
		model.label(sings, "Sings");
		jimDoesSing = model.addRelationship("DOES");
		model.connect(jimDoesSing, jim, sings);
		eatingRequiresSinging = model.addRelationship("REQUIRES");
		model.connect(eatingRequiresSinging, eats, sings);
		
		model.label("Trixie Model");
		
		model.save();

		QRGutils.debug("VModelTest", "makeTrixieModel MODEL "+model.modelElement());

		QRGutils.debug("VModelTest", "makeTrixieModel find(\"MODELS-ONTOLOGY-LIBRARY\"):\n" +
							system.ontology().find("MODELS-ONTOLOGY-LIBRARY"));
//		printAllVModelElements();
	} // method makeTrixieModel

	public static void openTrixieModel(VModel system) {
		VModelElement ontology = null;
		VModelElement element = null;
		VModelModel model = null;
		VModelElement trixie = null, jim = null, food = null, fruit = null;
		VModelElement sings = null, eats = null;
		VModelElement weight = null;
		VModelElement eatingIncreasesWeight = null;
		VModelElement eatingDecreasesFood = null;
		VModelElement jimDoesSing = null;
		VModelElement trixieDoesEat = null;
		VModelElement eatingRequiresSinging = null;
		VModelElement trixieHasWeight = null;
		VModelElement trixieGroup = null, jimGroup = null;
		ontology = system.ontology().find("MODELS-ONTOLOGY-LIBRARY");
		if (ontology != null) element = system().ontology().find(ontology, "MODEL", "Trixie-Model");
		element = (VModelElement) element.clone();
		model = new VModelModel( system, element);
		model.log(true);
		
		QRGutils.debug("VModelTest", "openTrixieModel MODEL "+model.modelElement());

		QRGutils.debug("VModelTest", "openTrixieModel find(\"MODELS-ONTOLOGY-LIBRARY\"):\n" +
							system.ontology().find("MODELS-ONTOLOGY-LIBRARY"));
	} // method openTrixieModel

	public static void useTrixieModel(VModel system) {
		VModelModel model = new VModelModel(system);
		VModelElement trixieModel = null;
		VModelElement trixie = null, jim = null, food = null, fruit = null;
		VModelElement exercises = null, sings = null, eats = null;
		VModelElement weight = null;
		VModelElement eatingIncreasesWeight = null;
		VModelElement exerciseDecreasesWeight = null;
		VModelElement eatingDecreasesFood = null;
		VModelElement jimDoesSing = null;
		VModelElement trixieDoesEat = null;
		VModelElement trixieDoesExercise = null;
		VModelElement eatingRequiresSinging = null;
		VModelElement trixieHasWeight = null;
		VModelElement trixieGroup = null, jimGroup = null;
		model.log(true);
		trixieModel = model.addModel(system.ontology().find("MODELS-ONTOLOGY-LIBRARY"), "Trixie-Model");

		trixie = trixieModel.findElement(new VModelAttribute("LABEL","Trixie the Elephant"));
		trixieGroup = trixieModel.findElement(new VModelAttribute("LABEL","Trixie"));
	   weight = trixieGroup.findElement(new VModelAttribute("LABEL","Weight"));
		exercises = model.addObject("PROCESS");
		model.label(exercises, "Exercises");
		trixieDoesExercise = model.addRelationship("DOES");
		model.connect(trixieDoesExercise, trixie, exercises);
		exerciseDecreasesWeight = model.addRelationship("DECREASES");
		model.connect(exerciseDecreasesWeight, exercises, weight);

		QRGutils.debug("VModelTest", "useTrixieModel MODEL "+model.modelElement());
	} // method useTrixieModel

	public static void buildTrixieModel(VModel system) {
		VModelModel model = new VModelModel(system);
		VModelElement trixieModel = null;
		VModelElement elephant = null, trixie = null, jim = null, food = null, fruit = null;
		VModelElement exercises = null, sings = null, eats = null;
		VModelElement weight = null;
		VModelElement eatingIncreasesWeight = null;
		VModelElement exerciseDecreasesWeight = null;
		VModelElement eatingDecreasesFood = null;
		VModelElement jimDoesSing = null;
		VModelElement trixieDoesEat = null;
		VModelElement trixieDoesExercise = null;
		VModelElement eatingRequiresSinging = null;
		VModelElement trixieHasWeight = null;
		VModelElement trixieGroup = null, jimGroup = null;
		model.log(true);

		trixie = model.addObject("THING");
		model.label(trixie, "elephant");
		weight = model.addAttribute("AMOUNT");
		trixieHasWeight = model.addRelationship("HAS-DESCRIBER");
		model.connect(trixieHasWeight, trixie, weight);
		model.label(trixie, "Trixie the Elephant");

		food = model.addObject("THING");
		model.label(food, "Food");
		model.variablize(food);
		eats = model.addObject("PROCESS");
		model.label(eats, "Eats");
	   eatingIncreasesWeight = model.addRelationship("INCREASES");
		model.connect(eatingIncreasesWeight, eats, weight);
		eatingDecreasesFood = model.addRelationship("DECREASES");
		model.connect(eatingDecreasesFood, eats, food);
		trixieDoesEat = model.addRelationship("DOES");
		model.connect(trixieDoesEat, trixie, eats);
		
		trixieGroup = model.makeGroup();
		model.label(trixieGroup, "Trixie");
		model.addGroup(trixieGroup);
		model.addToGroup(trixieGroup, trixie);
		model.addToGroup(trixieGroup, food);
		model.addToGroup(trixieGroup, eats);
		model.addToGroup(trixieGroup, eatingIncreasesWeight);
		model.addToGroup(trixieGroup, eatingDecreasesFood);
		model.addToGroup(trixieGroup, trixieDoesEat);

		fruit = model.addObject("THING");
		model.label(fruit, "Fruits, Nuts, and Berries");
		model.bind(food, fruit);

		jim = model.addObject("THING");
		model.label(jim, "Jim");
		sings = model.addObject("PROCESS");
		model.label(sings, "Sings");
		jimDoesSing = model.addRelationship("DOES");
		model.connect(jimDoesSing, jim, sings);
		eatingRequiresSinging = model.addRelationship("REQUIRES");
		model.connect(eatingRequiresSinging, eats, sings);
		
		model.label("Trixie Model");

		exercises = model.addObject("PROCESS");
		model.label(exercises, "Exercises");
		trixieDoesExercise = model.addRelationship("DOES");
		model.connect(trixieDoesExercise, trixie, exercises);
		exerciseDecreasesWeight = model.addRelationship("DECREASES");
		model.connect(exerciseDecreasesWeight, exercises, weight);

		QRGutils.debug("VModelTest", "buildTrixieModel MODEL "+model.modelElement());
	} // method buildTrixieModel

	// Constructors
	// --------------------------------------------------------------------------
	public VModelTest ( VModel system ) {
		system(system);
	} // constructor

} // class VModelTest

// -----------------------------------------------------------------------------
// End of File: VModelTest.java                 (c) 2001 Northwestern University