package edu.qrg.vmodel.controllers.macclassroom;

import edu.qrg.utils.*;
import edu.qrg.vmodel.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;
import java.io.File;
import java.lang.Cloneable;
import java.lang.Exception;
import java.lang.Object;
import java.lang.String;
import java.lang.Throwable;
import java.net.URL;
import java.util.EventListener;
import java.util.Stack;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JViewport;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.border.LineBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkEvent. EventType;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/gui
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: HTMLpanel.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: HTMLpanel
* Purpose: Simple web browser.
* ----------------------------------------------------------------------------
* Notes:
* * Very simple to suit the task in VModel.
* * Need to add a toolbar with forward/backward
* -----------------------------------------------------------------------------
* </PRE> */
public class HTMLpanel extends JPanel implements VModelConstants, HyperlinkListener, 
			ActionListener, WindowConstants, SwingConstants, KeyListener {
	// Constructors
	// -------------------------------------------------------------------------
	public HTMLpanel () { 
		super(new BorderLayout()); 
		southpane(new JPanel(new BorderLayout()));
		buttonspane(new JPanel(new GridLayout(1, 2)));
		addKeyListener(this);
		buttonspane().addKeyListener(this);
		back().addActionListener(this);
		back().setEnabled(false);
		back().addKeyListener(this);
		buttonspane().add(back());
		forward().addActionListener(this);
		forward().setEnabled(false);
		forward().addKeyListener(this);
		buttonspane().add(forward());
		southpane().add(buttonspane(), BorderLayout.WEST);
		southpane().add(statusbar(), BorderLayout.CENTER);
		southpane().addKeyListener(this);
		add(southpane(), BorderLayout.SOUTH);
		add(scrollPane(), BorderLayout.CENTER);
		scrollPane().addKeyListener(this);
		viewPane().addKeyListener(this);
		statusbar().addKeyListener(this);
		southpane.setVisible(true);
		buttonspane.setVisible(true);
	} // constructor HTMLpanel
	
	public HTMLpanel (String base, String page) { 
		this();
		gotoPage(base, page);
	} // constructor HTMLpanel
	
	public HTMLpanel (String page) { 
		this(QRGutils.QRGroot() + HELPPATH, page); 
	} // constructor HTMLpanel
	
	// Properties & Accessors 
	// -------------------------------------------------------------------------
	protected JTextPane viewPane = null;
	public JTextPane viewPane () {
		if (viewPane == null) {
			viewPane(new JTextPane());
			viewPane.setEditable(false);
			viewPane.addHyperlinkListener(this);
		} // end if viewPane 
		return(viewPane); 
	} // reader viewPane
	public JTextPane viewPane (JTextPane v) { return(viewPane = v); }

	protected JScrollPane scrollPane = null;
	public JScrollPane scrollPane () {
		if (scrollPane == null) {
			scrollPane(new JScrollPane()); 
			if (scrollPane.getViewport() == null) scrollPane.setViewport(new JViewport());
			scrollPane.getViewport().setView(viewPane());
		} // end if scrollPane
		return(scrollPane); 
	} // reader JScrollPane
	public JScrollPane scrollPane (JScrollPane s) { return(scrollPane = s); }

   protected JLabel statusbar = null;
	public JLabel statusbar() { 
		if (statusbar == null) {
			statusbar(new JLabel(" "));
			statusbar.setBorder(new LineBorder(DARKERGREY, 2));
		} // end if statusbar
		return(statusbar); 
	} // reader statusbar
	public JLabel statusbar(JLabel s) { return(statusbar = s); }

   protected JButton back = new JButton(ImageCache.getImage("back.gif"));
	public JButton back() { return(back); }
	public JButton back(JButton b) { return(back = b); }

   protected JButton forward = new JButton(ImageCache.getImage("forward.gif"));
	public JButton forward() { return(forward); }
	public JButton forward(JButton b) { return(forward = b); }

   protected Stack forwardStack = null;
	public Stack forwardStack() { 
		if (forwardStack == null) forwardStack(new Stack());
		return(forwardStack); 
	} // reader forwardStack
	public Stack forwardStack(Stack b) { return(forwardStack = b); }

   protected Stack backStack = null;
	public Stack backStack() { 
		if (backStack == null) backStack(new Stack());
		return(backStack); 
	} // reader backStack
	public Stack backStack(Stack b) { return(backStack = b); }

   protected String currentAddress = "";
	public String currentAddress() { return(currentAddress); }
	public String currentAddress(String b) { return(currentAddress = b); }

   protected JPanel southpane = null;
	public JPanel southpane() { return(southpane); }
	public JPanel southpane(JPanel j) { return(southpane = j); }

   protected JPanel buttonspane = null;
	public JPanel buttonspane() { return(buttonspane); }
	public JPanel buttonspane(JPanel j) { return(buttonspane = j); }

	// Methods
	// -------------------------------------------------------------------------
	public void gotoPage (String base, String page) {
		if ((page != null) && (page.length() > 0)) {
			setPage(base + page);
			setBase(base);
			setVisible(true);
		} // end if page
	} // setPage

	public void setPage (String address) {
		try {
			File addressFile = new File(address);
			URL addrURL = QRGutils.fileToURL(addressFile);
			setPage(addrURL);
			//setPage(new URL(address)); 	// TRH
			currentAddress(address);
		} catch(Exception e) {
			try { 
				setPage(new URL("file:///" + address));
			currentAddress(address);
			} catch(Exception e2) {
				try { 
					setPage(new URL("file://" + address));
			currentAddress(address);
				} catch(Exception e3) {
					try { 
						setPage(new URL("file","",address));
			currentAddress(address);
					} catch(Exception e6) {
						QRGutils.debug("HTMLpanel", "setPage e6=" + e6);
						QRGutils.debugStackTrace(e6);
					} // end try - catch
				} // end try - catch
			} // end try - catch
		} // end try - catch
	} // setPage

	public void setPage (URL address) throws IOException {
		try { 
			viewPane().setPage(address);
			if ((currentAddress() != null) && (currentAddress().length() > 0)) {
				backStack().push(currentAddress());
				back().setEnabled(true);
			} // end if currentAddress
			currentAddress(address.toString());
			forwardStack(null);
			forward().setEnabled(false);
		} catch(IOException e) {
			QRGutils.debug("HTMLpanel", "setPage url e=" + e);
			QRGutils.debugStackTrace(e);
			throw(e);
		} // end try - catch
	} // setPage

	public void setPageAux (String address) {
//QRGutils.debug("HTMLpanel", "setPageAux address=" + address);
	if (address.startsWith("file:")) address=address.substring(5);
		try { 
			setPageAux(new URL(address)); 
			currentAddress(address);
		} catch(Exception e) {
			try { 
				setPageAux(new URL("file://" + address));
			currentAddress(address);
			} catch(Exception e2) {
					try { 
						setPageAux(new URL("file","",address));
			currentAddress(address);
					} catch(Exception e6) {
						QRGutils.debug("HTMLpanel", "setPageAux e6=" + e6);
						QRGutils.debugStackTrace(e6);
					} // end try - catch
			} // end try - catch
		} // end try - catch
	} // setPageAux

	public void setPageAux (URL address) throws IOException {
		try { 
			viewPane().setPage(address);
		} catch(IOException e) {
			QRGutils.debug("HTMLpanel", "setPageAux url e=" + e);
			throw(e);
		} // end try - catch
	} // setPageAux

	public void setBase (String address) {
		try { setBase(new URL(address)); }
		catch(Exception e) {
			try { setBase(new URL("file://" + address)); }
			catch(Exception e2) {
				QRGutils.debug("HTMLpanel", "setBase e=" + e2);
			} // end try - catch
		} // end try - catch
	} // setBase

	public void setBase (URL address) {
		try { 
			((HTMLDocument) viewPane().getDocument()).setBase(address);
		} catch(Exception e) {
			QRGutils.debug("HTMLpanel", "setBase url e=" + e);
		} // end try - catch
	} // setBase

	public String statusbar(String s) {
		statusbar().setText(s); 
		return(statusbar().getText()); 
	} // method statusbar

	// HyperlinkListener Methods
	// -------------------------------------------------------------------------
	public void hyperlinkUpdate(HyperlinkEvent e) {
		if (e.getEventType() == HyperlinkEvent.EventType.ENTERED) {
			String foo = e.getURL().toString();
			if (foo.length() > 30) foo = foo.substring(0,30) + "...";
			statusbar(foo);
		} else if (e.getEventType() == HyperlinkEvent.EventType.EXITED) {
			statusbar(" ");
		} else if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			JEditorPane pane = (JEditorPane) e.getSource();
			if (e instanceof HTMLFrameHyperlinkEvent) {
				HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
				HTMLDocument doc = (HTMLDocument)pane.getDocument();
				doc.processHTMLFrameHyperlinkEvent(evt);
			} else {
				try {
					setPage(e.getURL());
				} catch (Throwable t) {
					QRGutils.debug("HTMLpanel", "hyperlinkUpdate e=" + e);
					QRGutils.debugStackTrace(t);
				} // end try - catch
			} // end if HTMLFrameHyperlinkEvent
		} // end if eventType
	} // method hyperlinkUpdate
	
	
	// ActionListener Methods
	// -------------------------------------------------------------------------
	public void actionPerformed (ActionEvent e) {
		String foo = "";
		if (e.getSource() == back()) {
			if (!backStack().empty()) {
				if ((currentAddress() != null) && (currentAddress().length() > 0)) {
					forwardStack().push(currentAddress());
					forward.setEnabled(true);
				} // end if currentAddress
				foo = (String) backStack().pop();
				setPageAux(foo);
				if (backStack().empty()) back.setEnabled(false);
			} // end if empty
		} else if (e.getSource() == forward()) {
			if (!forwardStack().empty()) {
				if ((currentAddress() != null) && (currentAddress().length() > 0)) {
					backStack().push(currentAddress());
					back.setEnabled(true);
				} // end if currentAddress
				foo = (String) forwardStack().pop();
				setPageAux(foo);
				if (forwardStack().empty()) forward.setEnabled(false);
			} // end if empty
		} // end if source
	} // method actionPerformed

	// KeyListener methods
	// -----------------------------------------------------------------------
	public void keyTyped(KeyEvent e) {
	} // method keyTyped
	
	public void keyPressed(KeyEvent e) {
		int kcode = e.getKeyCode();
		int kmod = e.getModifiers();

		// Window
		if ((kcode == KeyEvent.VK_W) && 
			((kmod == VKM_COMMAND) || (kmod == VKM_CONTROL))) 
			setVisible(false);

		// model operations
		if ((kcode == KeyEvent.VK_LEFT) || ((QRGutils.detectPlatform() == MACINTOSH) && (kcode == KeyEvent.VK_BACK_SPACE)))
			back().doClick();
		
		if ((kcode == KeyEvent.VK_RIGHT) || ((QRGutils.detectPlatform() == MACINTOSH) && (kcode == KeyEvent.VK_SPACE)))
			forward().doClick();
	} // Method keyPressed
	
	public void keyReleased(KeyEvent e) {
	} // Method keyReleased
	
	public void setBackground(Color c) {
		super.setBackground(c);
		if (buttonspane() != null) buttonspane().setBackground(c);
		if (southpane() != null) southpane().setBackground(c);
		// if (statusbar() != null) statusbar().setBackground(c);
		if (scrollPane() != null) scrollPane().setBackground(c);
		if (viewPane() != null) viewPane().setBackground(c);
	} // method setBackground

} // class HTMLpanel
    
// -----------------------------------------------------------------------------
// End of File: HTMLpanel.java                (c) 2001 Northwestern University
