package edu.qrg.vmodel.controllers.macclassroom;

import edu.qrg.utils.*;
import edu.qrg.vmodel.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;
import java.net.*;

public class TIFF extends JFrame {
	public static final String LITTLE_ENDIAN = "II";
	public static final String BIG_ENDIAN = "MM";
	public static final int TIFF_IDENTIFIER = 42;
	
	// Encoding Types
	public static final int BILEVEL_COLOR = 1;
	public static final int GRAYSCALE_COLOR = 1;
	public static final int INDEXED_COLOR = 1;
	public static final int RGB_COLOR = 1;
	
	// field types
	public static final int BYTE = 1;
	public static final int ASCII = 2;
	public static final int SHORT = 3;
	public static final int LONG = 4;
	public static final int RATIONAL = 5;
	public static final int SBYTE = 6;
	public static final int UNDEFINED = 7;
	public static final int SSHORT = 8;
	public static final int SLONG = 9;
	public static final int SRATIONAL = 10;
	public static final int FLOAT = 11;
	public static final int DOUBLE = 12;
	
	// field tags
	public static final int NEWSUBFILETYPE = 0x00FE;
	public static final int SUBFILETYPE = 0x00FF;
	public static final int IMAGEWIDTH = 0x0100;
	public static final int IMAGEHEIGHT = 0x0101; // IMAGELENGTH
	public static final int BITSPERSAMPLE = 0x0102;
	public static final int COMPRESSION = 0x0103;
	public static final int PHOTOMETRICINTERPRETATION = 0x0106;
	public static final int THRESHHOLDING = 0x0107;
	public static final int CELLWIDTH = 0x0108;
	public static final int CELLHEIGHT = 0x0109;
	public static final int FILLORDER = 0x010A;
	public static final int DOCUMENTNAME = 0x010D;
	public static final int IMAGEDESCRIPTION = 0x010E;
	public static final int MAKE = 0x010F;
	public static final int MODEL = 0x0110;
	public static final int STRIPOFFSETS = 0x0111;
	public static final int ORIENTATION = 0x0112;
	public static final int SAMPLESPERPIXEL = 0x0115;
	public static final int ROWSPERSTRIP = 0x0116;
	public static final int STRIPBYTECOUNTS = 0x0117;
	public static final int MINSAMPLEVALUE = 0x0118;
	public static final int MAXSAMPLEVALUE = 0x0119;
	public static final int XRESOLUTION = 0x011A;
	public static final int YRESOLUTION = 0x011B;
	public static final int PLANARCONFIGURATION = 0x011C;
	public static final int PAGENAME = 0x011D;
	public static final int XPOSITION = 0x011E;
	public static final int YPOSITION = 0x011F;
	public static final int FREEOFFSETS = 0x0120;
	public static final int FREEBYTECOUNTS = 0x0121;
	public static final int GRAYRESPONSEUNIT = 0x0122;
	public static final int GRAYRESPONSECURVE = 0x0123;
	public static final int RESOLUTIONUNIT = 0x0128;
	public static final int SOFTWARE = 0x0131;
	public static final int DATETIME = 0x0132;
	public static final int ARTIST = 0x013B;
	public static final int HOSTCOMPUTER = 0x013C;
	public static final int COLORMAP = 0x0140;
	public static final int EXTRASAMPLES = 0x0152;
	public static final int COPYRIGHT = 0x8298;
	
	
	// PhotometricInterpretation Values
	public static final int WHITEISZERO = 0;
	public static final int BLACKISZERO = 1;
	public static final int RGB = 2;
	public static final int PALETTECOLOR = 3;
	public static final int TRANSPARENCYMASK = 1;
	// Compression Values
	public static final int NOCROMPRESSION = 1;
	public static final int CCITGROUP3 = 2;
	public static final int PACKBITS = 32773;
	// ResolutionUnit Values
	public static final int NOUNIT = 1;
	public static final int INCH = 2;
	public static final int CENTIMETER = 3;

	public int imageWidth = -1;
	public int imageHeight = -1;
	public int dataStart = -1;
	
	public double scale = 1.0;
	public int translatex = 0, translatey = 0;
	
	// File Info
	public long startPos = 0;
	
	// header
	public byte byteOrder[] = new byte[2];
	public String byteOrderString = "";
	public String byteOrderText = "";
	public byte tiffIdentifier[] = new byte[2];
	public byte offsetIFD[] = new byte[4];
	public int offsetIFDint = 0;
	
	// image file directory
	public byte IFDnumberOfFields[] = new byte[2];
	public int IFDnumberOfFieldsInt = 0;
	public IFDfield[] IFDfields = null;
	
	public class IFDfield {
		public int fieldID = -1;
		public byte IFDtag[] = new byte[2];
		public int IFDtagInt = 0;
		public String IFDtagString = "";
		public byte IFDtype[] = new byte[2];
		public int IFDtypeInt = 0;
		public String IFDtypeString = "";
		public int IFDtypeLength = 0;
		public byte IFDtypeCount[] = new byte[4];
		public int IFDtypeCountInt = 0;
		public int IFDtypeCountBytes = 0;
		public byte IFDvalueOffset[] = new byte[4];
		public int IFDvalueOffsetInt = 0;
		public byte IFDvalue[] = new byte[4];
		
		
		
		
		public IFDfield (RandomAccessFile finp, int id) {
			fieldID = id;
			try {
				long cur = finp.getFilePointer();
				finp.read(IFDtag);
				IFDtagInt = bytesToShort(IFDtag);
				IFDtagString = IFDtagString(IFDtagInt);
				finp.read(IFDtype);
				IFDtypeInt = bytesToShort(IFDtype);
				IFDtypeString = IFDtypeString(IFDtypeInt);
				IFDtypeLength = IFDtypeLength(IFDtypeInt);
				finp.read(IFDtypeCount);
				IFDtypeCountInt = bytesToInt(IFDtypeCount);
				IFDtypeCountBytes = IFDtypeCountInt * IFDtypeLength;
				finp.read(IFDvalueOffset);
				
				if (IFDtypeCountBytes <= 4) {
					IFDvalue = IFDvalueOffset;
					if (IFDtypeLength <= 2) IFDvalueOffsetInt = bytesToShort(IFDvalueOffset);
					else IFDvalueOffsetInt = bytesToInt(IFDvalueOffset);
				} else {
					IFDvalue = new byte[IFDtypeCountBytes];
					cur = finp.getFilePointer();
					IFDvalueOffsetInt = bytesToInt(IFDvalueOffset);
					finp.seek(startPos + IFDvalueOffsetInt);
					finp.read(IFDvalue);
					finp.seek(cur);
				} // end if IFDtypeCountBytes
				
				switch(IFDtagInt) {
					//case NEWSUBFILETYPE: {result="NEWSUBFILETYPE"; break;}
					//case SUBFILETYPE: {result="SUBFILETYPE"; break;}
					case IMAGEWIDTH: {imageWidth = IFDvalueOffsetInt; break;}
					case IMAGEHEIGHT: {imageHeight = IFDvalueOffsetInt; break;}
					//case BITSPERSAMPLE: {result="BITSPERSAMPLE"; break;}
					//case COMPRESSION: {result="COMPRESSION"; break;}
					//case PHOTOMETRICINTERPRETATION: {result="PHOTOMETRICINTERPRETATION"; break;}
					//case THRESHHOLDING: {result="THRESHHOLDING"; break;}
					//case CELLWIDTH: {result="CELLWIDTH"; break;}
					//case CELLHEIGHT: {result="CELLHEIGHT"; break;}
					//case FILLORDER: {result="FILLORDER"; break;}
					//case DOCUMENTNAME: {result="DOCUMENTNAME"; break;}
					//case IMAGEDESCRIPTION: {result="IMAGEDESCRIPTION"; break;}
					//case MAKE: {result="MAKE"; break;}
					//case MODEL: {result="MODEL"; break;}
					case STRIPOFFSETS: {dataStart = IFDvalueOffsetInt; break;}
					//case ORIENTATION: {result="ORIENTATION"; break;}
					//case SAMPLESPERPIXEL: {result="SAMPLESPERPIXEL"; break;}
					//case ROWSPERSTRIP: {result="ROWSPERSTRIP"; break;}
					//case STRIPBYTECOUNTS: {result="STRIPBYTECOUNTS"; break;}
					//case MINSAMPLEVALUE: {result="MINSAMPLEVALUE"; break;}
					//case MAXSAMPLEVALUE: {result="MAXSAMPLEVALUE"; break;}
					//case XRESOLUTION: {result="XRESOLUTION"; break;}
					//case YRESOLUTION: {result="YRESOLUTION"; break;}
					//case PLANARCONFIGURATION: {result="PLANARCONFIGURATION"; break;}
					//case PAGENAME: {result="PAGENAME"; break;}
					//case XPOSITION: {result="XPOSITION"; break;}
					//case YPOSITION: {result="YPOSITION"; break;}
					//case FREEOFFSETS: {result="FREEOFFSETS"; break;}
					//case FREEBYTECOUNTS: {result="FREEBYTECOUNTS"; break;}
					//case GRAYRESPONSEUNIT: {result="GRAYRESPONSEUNIT"; break;}
					//case GRAYRESPONSECURVE: {result="GRAYRESPONSECURVE"; break;}
					//case RESOLUTIONUNIT: {result="RESOLUTIONUNIT"; break;}
					//case SOFTWARE: {result="SOFTWARE"; break;}
					//case DATETIME: {result="DATETIME"; break;}
					//case ARTIST: {result="ARTIST"; break;}
					//case HOSTCOMPUTER: {result="HOSTCOMPUTER"; break;}
					//case COLORMAP: {result="COLORMAP"; break;}
					//case EXTRASAMPLES: {result="EXTRASAMPLES"; break;}
					//case COPYRIGHT: {result="COPYRIGHT"; break;}
				}
			} catch (Exception e) {e.printStackTrace();}
		} // end constructor

		public int IFDtypeLength (int type) {
			int result = 0;
			switch(type) {
				case BYTE: {result=1; break;}
				case ASCII: {result=1; break;}
				case SHORT: {result=2; break;}
				case LONG: {result=4; break;}
				case RATIONAL: {result=8; break;}
				case SBYTE: {result=1; break;}
				case UNDEFINED: {result=1; break;}
				case SSHORT: {result=2; break;}
				case SLONG: {result=4; break;}
				case SRATIONAL: {result=8; break;}
				case FLOAT: {result=4; break;}
				case DOUBLE: {result=8; break;}
			} // end switch type
			return(result);
		} // end method IFDtypeLength
		
		public String IFDtagString (int tag) {
			String result = "UNKNOWN";
			switch(tag) {
				case NEWSUBFILETYPE: {result="NEWSUBFILETYPE"; break;}
				case SUBFILETYPE: {result="SUBFILETYPE"; break;}
				case IMAGEWIDTH: {result="IMAGEWIDTH"; break;}
				case IMAGEHEIGHT: {result="IMAGEHEIGHT"; break;}
				case BITSPERSAMPLE: {result="BITSPERSAMPLE"; break;}
				case COMPRESSION: {result="COMPRESSION"; break;}
				case PHOTOMETRICINTERPRETATION: {result="PHOTOMETRICINTERPRETATION"; break;}
				case THRESHHOLDING: {result="THRESHHOLDING"; break;}
				case CELLWIDTH: {result="CELLWIDTH"; break;}
				case CELLHEIGHT: {result="CELLHEIGHT"; break;}
				case FILLORDER: {result="FILLORDER"; break;}
				case DOCUMENTNAME: {result="DOCUMENTNAME"; break;}
				case IMAGEDESCRIPTION: {result="IMAGEDESCRIPTION"; break;}
				case MAKE: {result="MAKE"; break;}
				case MODEL: {result="MODEL"; break;}
				case STRIPOFFSETS: {result="STRIPOFFSETS"; break;}
				case ORIENTATION: {result="ORIENTATION"; break;}
				case SAMPLESPERPIXEL: {result="SAMPLESPERPIXEL"; break;}
				case ROWSPERSTRIP: {result="ROWSPERSTRIP"; break;}
				case STRIPBYTECOUNTS: {result="STRIPBYTECOUNTS"; break;}
				case MINSAMPLEVALUE: {result="MINSAMPLEVALUE"; break;}
				case MAXSAMPLEVALUE: {result="MAXSAMPLEVALUE"; break;}
				case XRESOLUTION: {result="XRESOLUTION"; break;}
				case YRESOLUTION: {result="YRESOLUTION"; break;}
				case PLANARCONFIGURATION: {result="PLANARCONFIGURATION"; break;}
				case PAGENAME: {result="PAGENAME"; break;}
				case XPOSITION: {result="XPOSITION"; break;}
				case YPOSITION: {result="YPOSITION"; break;}
				case FREEOFFSETS: {result="FREEOFFSETS"; break;}
				case FREEBYTECOUNTS: {result="FREEBYTECOUNTS"; break;}
				case GRAYRESPONSEUNIT: {result="GRAYRESPONSEUNIT"; break;}
				case GRAYRESPONSECURVE: {result="GRAYRESPONSECURVE"; break;}
				case RESOLUTIONUNIT: {result="RESOLUTIONUNIT"; break;}
				case SOFTWARE: {result="SOFTWARE"; break;}
				case DATETIME: {result="DATETIME"; break;}
				case ARTIST: {result="ARTIST"; break;}
				case HOSTCOMPUTER: {result="HOSTCOMPUTER"; break;}
				case COLORMAP: {result="COLORMAP"; break;}
				case EXTRASAMPLES: {result="EXTRASAMPLES"; break;}
				case COPYRIGHT: {result="COPYRIGHT"; break;}
			} // end switch type
			return(result);
		} // end method IFDtagString
		
		public String IFDtypeString (int type) {
			String result = "UNKNOWN";
			switch(type) {
				case BYTE: {result="BYTE"; break;}
				case ASCII: {result="ASCII"; break;}
				case SHORT: {result="SHORT"; break;}
				case LONG: {result="LONG"; break;}
				case RATIONAL: {result="RATIONAL"; break;}
				case SBYTE: {result="SBYTE"; break;}
				case UNDEFINED: {result="UNDEFINED"; break;}
				case SSHORT: {result="SSHORT"; break;}
				case SLONG: {result="SLONG"; break;}
				case SRATIONAL: {result="SRATIONAL"; break;}
				case FLOAT: {result="FLOAT"; break;}
				case DOUBLE: {result="DOUBLE"; break;}
			} // end switch type
			return(result);
		} // end method IFDtypeString
	} // end innerclass IFDfields
	
	public TIFF() {
		super();
		setLocation(-2,-2);
		setSize(1,1);
	} // end constructor
	
	public int bytesToShort (byte[] bytes) {
		int result = 0x0000;
		if (byteOrderString == BIG_ENDIAN) 
			result = (((bytes[0] & 0xff) << 8) | (bytes[1] & 0xff));
		else
			result = (((bytes[1] & 0xff) << 8) | (bytes[0] & 0xff));
		return(result);
	} // bytesToShort

	public int bytesToInt (byte[] bytes) {
		int result = 0x00000000;
		if (byteOrderString == BIG_ENDIAN) 
			result = (((bytes[0] & 0xff) << 24) | ((bytes[1] & 0xff) << 16) |
						 ((bytes[2] & 0xff) << 8) | (bytes[3] & 0xff));
		else
			result = (((bytes[3] & 0xff) << 24) | ((bytes[2] & 0xff) << 16) |
						 ((bytes[1] & 0xff) << 8) | (bytes[0] & 0xff));
		return(result);
	} // bytesToShort

	public Image decode (String filename) {
		return(decode(new File(filename)));
	} // end method decode

	public Image decode (File file) {
		Image result = null;
		try {
			RandomAccessFile finp = new RandomAccessFile(file, "r");
			result = decode(finp);
			finp.close();
		} catch (Exception e) { }
		return(result);
	} // end method decode

	public Image decode (RandomAccessFile finp) {
		Image result = null;
		startPos = 0;
		byteOrder = new byte[2];
		byteOrderString = "";
		byteOrderText = "";
		tiffIdentifier = new byte[2];
		offsetIFD = new byte[4];
		offsetIFDint = 0;
		IFDnumberOfFields = new byte[2];
		IFDnumberOfFieldsInt = 0;
		try {
			startPos = finp.getFilePointer();
										 
			System.out.println("FILE INFO");
			System.out.println("StartPosition= " + startPos);			

			finp.read(byteOrder);
			byteOrderString = new String(byteOrder);
			if (byteOrderString.equals(BIG_ENDIAN)) {
				byteOrderString = BIG_ENDIAN;
				byteOrderText = "big-endian";
			} else if (byteOrderString.equals(LITTLE_ENDIAN)) {
				byteOrderString = LITTLE_ENDIAN;
				byteOrderText = "little-endian";
			} // end if byteOrder
			finp.read(tiffIdentifier);
			if (((byteOrderString == BIG_ENDIAN) && (tiffIdentifier[1] == TIFF_IDENTIFIER)) ||
				 ((byteOrderString == LITTLE_ENDIAN) && (tiffIdentifier[0] == TIFF_IDENTIFIER))) {
				finp.read(offsetIFD);
				offsetIFDint = bytesToInt(offsetIFD);
										 
				System.out.println("FILE HEADER");
				System.out.println("ByteOrder= " + byteOrderText);			
				System.out.println("TiffIdentifier= " + TIFF_IDENTIFIER);
				System.out.println("OffsetIFD= "+offsetIFD[0]+" "+offsetIFD[1]+" "+offsetIFD[2]+" "+offsetIFD[3]);
				System.out.println("OffsetIFD= "+offsetIFDint);
				
				finp.seek(startPos + offsetIFDint);
				finp.read(IFDnumberOfFields);
				IFDnumberOfFieldsInt = bytesToShort(IFDnumberOfFields);
				IFDfields = new IFDfield[IFDnumberOfFieldsInt];

				System.out.println("IMAGE FILE DIRECTORY");
				System.out.println("NumberOfFields= " + IFDnumberOfFields[0] + " " + IFDnumberOfFields[1]);			
				System.out.println("NumberOfFields= " + IFDnumberOfFieldsInt);			

				for(int i=0; i<IFDnumberOfFieldsInt; i++) {		
					IFDfields[i] = new IFDfield(finp, i);
					System.out.println("DIRECTORY FIELD");
					System.out.println("Field Number= " + IFDfields[i].fieldID);			
					System.out.println("Tag= " + IFDfields[i].IFDtag[0] + " " + IFDfields[i].IFDtag[1]);			
					System.out.println("Tag= " + IFDfields[i].IFDtagInt);			
					System.out.println("Tag= " + IFDfields[i].IFDtagString);			
					System.out.println("Type= " + IFDfields[i].IFDtype[0] + " " + IFDfields[i].IFDtype[1]);			
					System.out.println("Type= " + IFDfields[i].IFDtypeInt);			
					System.out.println("Type= " + IFDfields[i].IFDtypeString);			
					System.out.println("TypeLength= " + IFDfields[i].IFDtypeLength);			
					System.out.println("Count= "+IFDfields[i].IFDtypeCount[0]+" "+IFDfields[i].IFDtypeCount[1]+" "+IFDfields[i].IFDtypeCount[2]+" "+IFDfields[i].IFDtypeCount[3]);
					System.out.println("Count= "+IFDfields[i].IFDtypeCountInt);
					System.out.println("CountBytes= "+IFDfields[i].IFDtypeCountBytes);
					System.out.println("ValueOffset= "+IFDfields[i].IFDvalueOffset[0]+" "+IFDfields[i].IFDvalueOffset[1]+" "+IFDfields[i].IFDvalueOffset[2]+" "+IFDfields[i].IFDvalueOffset[3]);
					System.out.println("ValueOffset= "+IFDfields[i].IFDvalueOffsetInt);
					System.out.println("Value= "+IFDfields[i].IFDvalue[0]+" "+IFDfields[i].IFDvalue[1]+" "+IFDfields[i].IFDvalue[2]+" "+IFDfields[i].IFDvalue[3]);
				} // end for i
				
				finp.seek(startPos + dataStart);
				byte pixels[] = new byte[imageWidth * imageHeight * 3];
				byte rb=0,gb=0,bb=0;
				int r=0,g=0,b=0;
				int pos=0;
				try {
					int count = finp.read(pixels);
					setVisible(true);
					result = createImage(imageWidth, imageHeight);
					setVisible(false);
					Graphics ig = result.getGraphics();
					for(int y=0; y<imageHeight; y++)
						for(int x=0; x<imageWidth; x++) {
							rb = pixels[pos++];
							gb = pixels[pos++];
							bb = pixels[pos++];
							r = (rb >= 0) ? rb : 255 + rb;
							g = (gb >= 0) ? gb : 255 + gb;
							b = (bb >= 0) ? bb : 255 + bb;
							ig.setColor(new Color(r,g,b));
							ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
						} // end for x
				} catch (Exception e) {result = null;}


			} // end if tiff
		} catch (Exception e) {e.printStackTrace(); }
		return(result);
	} // end method decode
	
	public int encode (Image image, String filename, int format) {
		return(encode(image, new File(filename), format));
	} // end method encode

	public int encode (Image image, File file, int format) {
		int result = -1;
		try {
			FileOutputStream fout = new FileOutputStream(file);
			result = encode(image, fout, format);
			fout.close();
		} catch (Exception e) { }
		return(result);
	} // end method encode

	public int encode (Image image, OutputStream fout, int format) {
		int result = -1;
		try {
			DataOutputStream dout = new DataOutputStream(fout);
			result = encode(image, dout, format);
			dout.close();
		} catch (Exception e) { }
		return(result);
	} // end method encode

	public int encode (Image image, DataOutputStream dout, int format) {
		int result = -1;

		imageWidth = image.getWidth(null);
		imageHeight = image.getHeight(null);

		int data[] = new int[imageWidth*imageHeight];
		int bytecount = imageWidth*imageHeight*3;
		
		byte tiffHeader[] = {0x4D, 0x4D, 0x00, (byte) TIFF_IDENTIFIER, 0x00, 0x00, 0x00, 0x08}; // 8
		byte ifd[] = { 0x00, 0x0A, /*num of fields*/ 
/*NEWSUBFILETYPE*/	0x00, (byte) 0x00FE, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, /* NEWSUBFILETYPE */
/*IMAGEWIDTH*/			0x01, 0x00, 0x00, (byte) LONG, 0x00, 0x00, 0x00, 0x01, (byte) ((imageWidth & 0xff000000) >> 24), (byte) ((imageWidth & 0xff0000) >> 16), (byte) ((imageWidth & 0xff00) >> 8), (byte) (imageWidth & 0xff),
/*IMAGEHEIGTH*/		0x01, 0x01, 0x00, (byte) LONG, 0x00, 0x00, 0x00, 0x01, (byte) ((imageHeight & 0xff000000) >> 24), (byte) ((imageHeight & 0xff0000) >> 16), (byte) ((imageHeight & 0xff00) >> 8), (byte) (imageHeight & 0xff),
/*BITSPERSAMPLE*/		0x01, 0x02, 0x00, (byte) SHORT, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, (byte) 0x0086,
/*COMPRESSION*/		0x01, 0x03, 0x00, (byte) SHORT, 0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x00,
/*PHOTOMETRICINTERPRESTATION*/	0x01, 0x06, 0x00, (byte) SHORT, 0x00, 0x00, 0x00, 0x02, 0x00, 0x02, 0x00, 0x00,
/*STRIPOFFSETS*/		0x01, 0x11, 0x00, (byte) LONG, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, (byte) 0x008C,
/*SAMPLESPERPIXEL*/	0x01, 0x15, 0x00, (byte) SHORT, 0x00, 0x00, 0x00, 0x01, 0x00, 0x03, 0x00, 0x00,
/*ROWSPERSTRIP*/		0x01, 0x16, 0x00, (byte) LONG, 0x00, 0x00, 0x00, 0x01, (byte) ((imageHeight & 0xff000000) >> 24), (byte) ((imageHeight & 0xff0000) >> 16), (byte) ((imageHeight & 0xff00) >> 8), (byte) (imageHeight & 0xff),
/*STRIPBYTECOUNTS*/	0x01, 0x17, 0x00, (byte) LONG, 0x00, 0x00, 0x00, 0x01, (byte) ((bytecount & 0xff000000) >> 24), (byte) ((bytecount & 0xff0000) >> 16), (byte) ((bytecount & 0xff00) >> 8), (byte) (bytecount & 0xff),
							0x00, 0x00, 0x00, 0x00
						 }; // 126
		byte bitsPerSample[] = { 0x00, 0x08, 0x00, 0x08, 0x00, 0x08 }; // 6
		byte pixels[] = new byte[imageWidth*imageHeight*3];
		
		int pos = 0;
		byte r=0,g=0,b=0;
		boolean grabOK = false;
		PixelGrabber pg = new PixelGrabber(image,0,0,imageWidth,imageHeight,data,0,imageWidth);
	   try { 
	   	grabOK= pg.grabPixels();
	    	if (grabOK) {
	    		dout.write(tiffHeader);
	    		dout.write(ifd);
	    		dout.write(bitsPerSample);
	    		for(int i=0; i<(imageWidth*imageHeight); i++) {
	    			pixels[pos++] = (byte) ((data[i] & 0xff0000) >> 16);
	    			pixels[pos++] = (byte) ((data[i] & 0x00ff00) >> 8);
	    			pixels[pos++] = (byte) (data[i] & 0x0000ff);
	    		} // end for
	    		dout.write(pixels,0,imageWidth*imageHeight*3);
	    		result = 0;
	    	} // if grabOK
    	} catch (Exception e) {};
		
		
		return(result);
	} // end method encode
} // end class TIFF

// ----------------------------------------------------------------------------
// END OF FILE: TIFF.java                                 2001 Leo C. Ureel II
