package edu.qrg.vmodel.controllers.macclassroom;

import edu.qrg.utils.*;
import edu.qrg.vmodel.*;
import java.io.File;
import java.lang.Object;
import java.lang.String;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.filechooser.FileFilter;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/gui
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: VModelFileFilter.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: VModelFileFilter
* Purpose: Filter files for file chooser dialog.
* ----------------------------------------------------------------------------
* Notes:
* * This seems like overkill.  Review the code with an eye towards simplificity
* -----------------------------------------------------------------------------
* </PRE> */
public class VModelFileFilter extends javax.swing.filechooser.FileFilter {
    public static final String TYPE_UNKNOWN = "Type Unknown";
    public static final String HIDDEN_FILE = "Hidden File";

    private Hashtable filters = null;
    private String description = null;
    private String fullDescription = null;
    private boolean useExtensionsInDescription = true;

    public VModelFileFilter() { this.filters = new Hashtable(); }

    public VModelFileFilter(String extension) { this(extension,null); }

    public VModelFileFilter(String extension, String description) {
		this();
		if(extension!=null) addExtension(extension);
	 	if(description!=null) setDescription(description);
    }

    public VModelFileFilter(String[] filters) { this(filters, null); }

    public VModelFileFilter(String[] filters, String description) {
		this();
		for (int i = 0; i < filters.length; i++) {
		    // add filters one by one
		    addExtension(filters[i]);
		}
	 	if(description!=null) setDescription(description);
    }

    public boolean accept(File f) {
		if(f != null) {
		    if(f.isDirectory()) {
				return true;
		    }
		    String extension = getExtension(f);
		    if(extension != null && filters.get(getExtension(f)) != null) {
			return true;
		    };
		}
		return false;
    }

     public String getExtension(File f) {
	if(f != null) {
	    String filename = f.getName();
	    int i = filename.lastIndexOf('.');
	    if(i>0 && i<filename.length()-1) {
		return filename.substring(i+1).toLowerCase();
	    };
	}
	return null;
    }

    public void addExtension(String extension) {
		if(filters == null) filters = new Hashtable(5);
		filters.put(extension.toLowerCase(), this);
		fullDescription = null;
    }

    public String getDescription() {
		if(fullDescription == null) {
		    if(description == null || isExtensionListInDescription()) {
		 		fullDescription = description==null ? "(" : description + " (";
				// build the description from the extension list
				Enumeration extensions = filters.keys();
				if(extensions != null) {
				    fullDescription += "." + (String) extensions.nextElement();
				    while (extensions.hasMoreElements()) {
					fullDescription += ", " + (String) extensions.nextElement();
				    }
				}
				fullDescription += ")";
		    } else fullDescription = description;
		}
		return fullDescription;
    }

    public void setDescription(String description) {
		this.description = description;
		fullDescription = null;
    }

    public void setExtensionListInDescription(boolean b) {
		useExtensionsInDescription = b;
		fullDescription = null;
    }

	public boolean isExtensionListInDescription() {
		return useExtensionsInDescription;
    }
} // class 
    
// -----------------------------------------------------------------------------
// End of File: VModelFileFilter.java       (c) 2001 Northwestern University
