package edu.qrg.vmodel;

import edu.qrg.utils.*;

import java.awt.Color;

import java.lang.Double;
import java.lang.Integer;
import java.lang.NumberFormatException;
import java.lang.Object;
import java.lang.String;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.SwingConstants;

import edu.qrg.utils.KeyValuePair.KVPinterface;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/engine
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: VModelPreferences.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: VModelPreferences
* Purpose: Vmodel Preferences Facility.
* ----------------------------------------------------------------------------
* Notes:
* * Do we need to save globalPrefs anymore? I ask because all values are
*   now set only in localPrefs.  See the set function as oppossed to get.
* * I suspect the answer to the above is YES because we need to load some
*   preferences before local users log in.
* ----------------------------------------------------------------------------
* </PRE> */
public class VModelPreferences 
	implements VModelConstants, SwingConstants, PList.PLinterface {
	// Constructors
	// --------------------------------------------------------------------------
	public VModelPreferences(VModel.VMController system) {
		system(system);
		self(this);
//		loadGlobalPrefs();
	} // method VModelPreferences

	// Properties & Accessors
	// ------------------------------------------------------------------------
	protected static PREFERENCES globalPreferences = null;
	public static PREFERENCES globalPreferences () { 
		if (globalPreferences == null) globalPreferences(new PREFERENCES("PREFERENCES"));
		return(globalPreferences); 
	} // reader globalPreferences
	public static PREFERENCES globalPreferences (PREFERENCES prefs) { 
		return(globalPreferences = prefs); 
	} // writer globalPreferences

	protected static PREFERENCES localPreferences = null;
	public static PREFERENCES localPreferences () { 
		if (localPreferences == null) localPreferences(new PREFERENCES("PREFERENCES"));
		return(localPreferences); 
	} // reader localPreferences
	public static PREFERENCES localPreferences (PREFERENCES prefs) { 
		return(localPreferences = prefs); 
	} // writer localPreferences

	protected static PREFERENCES dataPreferences = null;
	public static PREFERENCES dataPreferences () { 
		if (dataPreferences == null) dataPreferences(new PREFERENCES("PREFERENCES"));
		return(dataPreferences); 
	} // reader dataPreferences
	public static PREFERENCES dataPreferences (PREFERENCES prefs) { 
		return(dataPreferences = prefs); 
	} // writer dataPreferences

	protected static VModel.VMController system = null;
	public static VModel.VMController system () { return(system); }
	public static VModel.VMController system (VModel.VMController v) { 
		return(system = v); 
	} // writer
	
	protected static VModelPreferences self = null;
	public static VModelPreferences self () { return(self); }
	public VModelPreferences self (VModelPreferences p) { return(self = p); }
	
	// File IO
	// --------------------------------------------------------------------------
	public static PREFERENCES loadGlobalPrefs(String inifile) {
		PREFERENCES result = null;
		String inifilePath = QRGutils.QRGroot() + inifile;
		result = load(inifilePath);
		if (result != null) globalPreferences(result);
		else QRGutils.debug("VModelPreferences.loadGlobalPrefs", "Global ini file did not load. inifilePath=" + inifilePath);
		return(result);
	} // method loadGlobalPrefs

	public PREFERENCES loadDataPrefs() {
		String inifile = null;
		PREFERENCES result = null;
		if (QRGutils.isApplet()) inifile = QRGutils.makeQRGpath("resources") + DATFILE;
		else inifile = QRGutils.makeQRGpath("resources") + DATFILE;
		result = load(inifile);
		if (result != null) dataPreferences(result);
		return(result);
	} // method loadDataPrefs

	public static PREFERENCES loadLocalPrefs(String inifile) {
		PREFERENCES result = load(QRGutils.QRGuserpath() + inifile);
		if (result != null) localPreferences(result);
		return(result);
	} // method loadLocalPrefs

	public static PREFERENCES load(String inifile) {
		PREFERENCES result = null;
		Object temp = XMLio.load(inifile);
		if (temp instanceof PREFERENCES) {
			result = (PREFERENCES) temp;
		} else if (temp instanceof PList) {
			result = new PREFERENCES(((PList) temp).keyValuePairs(), null);
		} // end if else if
		if (result != null) result.set("FULLFILEPATH", inifile);
		else {
			QRGutils.warn("VModel","Unable to load preference file " + inifile + ".");
		} // end if result else
		return(result);
	} // method load

	public static void saveGlobalPrefs () {
		if (globalPreferences() != null) {
			String filename = (String) globalPreferences().get("FULLFILEPATH");
			if (filename != null) XMLio.save(filename, PREFERENCES_DOCTYPE, globalPreferences());
		} // end if globalPreferences
	} // method saveGlobalPrefs
	
	public static void saveLocalPrefs () {
		if (localPreferences() != null) {
			String filename = QRGutils.QRGuserpath() + "VModel.ini";
			if (filename != null) XMLio.save(filename, PREFERENCES_DOCTYPE, localPreferences());
			else QRGutils.debug("VModelPreferences.saveLocalPrefs","filename=" + filename);
		} // end if localPreferences
	} // method savelocalPrefs
	
	public void save () {
		system().updatePreferences();
		saveLocalPrefs();
		saveGlobalPrefs();
	} // method save
	
	// Key-Value Pairs
	public Vector keyValuePairs () {
	Vector result = localPreferences().keyValuePairs();
	// Should we append global and data preferences to the list?
	return(result);
   } // method

	public Vector keyValuePairs (Vector v) {
	Vector result = localPreferences().keyValuePairs(v);
	// What about global and data preferences?
	return(result);
   } // method

	public Vector keys () {
		Vector result = new Vector();
		KeyValuePair.KVPinterface candidate = null;
		for (Enumeration e = keyValuePairs().elements() ; e.hasMoreElements() ;) {
			candidate =  (KeyValuePair.KVPinterface) e.nextElement();
			result.addElement(candidate.key());
		} // end for
		return(result);
	} // method
	
	public Vector values () {
		Vector result = new Vector();
		KeyValuePair.KVPinterface candidate = null;
		for (Enumeration e = keyValuePairs().elements() ; e.hasMoreElements() ;) {
			candidate =  (KeyValuePair.KVPinterface) e.nextElement();
			result.addElement(candidate.value());
		} // end for
		return(result);
	} // method
	
	// Get keyValuePair values.
	// -----------------------------------------------------------------------
    public Object get (String tag) {
		Object result = localPreferences().get(tag);
		if (result == null) result = globalPreferences().get(tag);
		if (result == null) result = dataPreferences().get(tag);
//		if (result == null) 
//			QRGutils.debug("VModelPreferences", "All preferences = null for get("+tag+")");
		return(result);
    } // method get

    public Color getColor (String tag) {
		Color result = Color.black;
		Object value = null;
		value = localPreferences.get(tag);
		if (value == null) value = globalPreferences().get(tag);
		if (value == null) value = dataPreferences().get(tag);
		if (value != null) result = QRGutils.matchColor((String) value);
//		else QRGutils.debug("VModelPreferences", "getColor("+tag+") returned null.");
		return result;
    } // method getColor

    public int getRGBColor (String tag) {
		int result = 0;
		Color color = getColor(tag);
		if (color != null) result = color.getRGB();
//		else QRGutils.debug("VModelPreferences", "getRGBColor("+tag+") returned null.");
		return(result);
    } // method getRGBColor

    public int getInt (String tag) {
		int result = 0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			try {result = Integer.parseInt(value);}
			catch(NumberFormatException e) {}
		} //else QRGutils.debug("VModelPreferences", "Tag not found for getInt("+tag+")");
		return result;
    } // method getInt

    public double getDouble (String tag) {
		double result = 0.0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			try {result = Double.valueOf(value).doubleValue();}
			catch(NumberFormatException e) {}
		} //else QRGutils.debug("VModelPreferences", "Tag not found for getDouble("+tag+")");
		return(result);
    } // method getDouble

    public boolean getBoolean (String tag) {
		boolean result = false;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			result = value.equalsIgnoreCase("TRUE");
		} //else QRGutils.debug("VModelPreferences", "Tag not found for getBoolean("+tag+")");
		return(result);
    } // method getBoolean

    public String getString (String tag) {
		String result = "";
		String value = (String) get(tag);
		if (value != null) result = value;
//		else QRGutils.debug("VModelPreferences", "Tag not found for getString("+tag+")");
		return(result);
    } // method getString

	public boolean containsString (String key, String target) {
		boolean result = false;
		String value = getString(key);
		result =  (value.indexOf(target)>=0);
		return(result);
	} // method getDouble
	
	public Vector getVector (String tag) {
		Vector result = null;
		Vector value = (Vector) get(tag);
		if (value != null) result = value;
		else { 
//			QRGutils.debug("VModelPreferences", "Tag not found for getString("+tag+")");
			result = new Vector();
			set(tag, result);
		} // end if else
		return(result);
	} // method getString

	// Set KeyValuePair values.
	// -----------------------------------------------------------------------
    public Object set (String tag, Object value) {
		Object result = null;
		result = localPreferences().set(tag, value);
		return(result);
    } // method set

    public Object set (String tag, int value) {
		return(set(tag, "" + value));
    } // method set

    public Object set (String tag, boolean value) {
		return(set(tag, "" + value));
    } // method set

    public Object set (String tag, double value) {
		return(set(tag, "" + value));
    } // method set

    public Object set (String tag, Color value) {
		return(set(tag, "" + value.getRGB()));
    } // method set
    
    // Add & Remove KeyValuePairs
    // -----------------------------------------------------------------------
    public void add (String key, Object value) {
    	localPreferences().add(key, value);
    } // method

    public boolean remove (String key) {
    	return(localPreferences().remove(key));
    } // method
    
	// Stringification.
	// -----------------------------------------------------------------------
	public String stringifyKeyValuePairs (int indent) {
		String result = "";
		int j = 0;
		if (keyValuePairs() != null) {
			j = 0;
			while(j < keyValuePairs().size()) {
				if (j > 0) result += QRGutils.eol() + QRGutils.space(indent);
				else result += " "; 
				result += (KeyValuePair) keyValuePairs().elementAt(j);
				j++;
			} // while
		} // end if
		return(result);
	} // method toString
	
	public String toString (int indent) {
		String result = "";
		int i = indent, j = 3;
		result = QRGutils.space(i) + "<" + "VModelPreferences" /* this.getClass().getName() */ + ">" + QRGutils.eol();
		result += QRGutils.space(j) + XMLio.commentate("Local Preferences") + QRGutils.eol();
		result += localPreferences().toString(j);
		result += QRGutils.space(j) + XMLio.commentate("Global Preferences") + QRGutils.eol();
		result += globalPreferences().toString(j);
		result += QRGutils.space(j) + XMLio.commentate("Data Preferences") + QRGutils.eol();
		result += dataPreferences().toString(j);
		result += QRGutils.space(i) + "</" + "VModelPreferences" /* this.getClass().getName() */ + ">" + QRGutils.eol();
		return(result);
	} // method toString
	
	public String toString () { 
		return(toString(0)); 
	} // method toString
	
} // class 

// ---------------------------------------------------------------------------
// END OF FILE: VModelPreferences.java           2003 Northwestern University