package edu.qrg.vmodel;

import edu.qrg.utils.*;
import edu.qrg.utils.CMLio.CMLable;
import edu.qrg.vmodel.Definition;
import edu.qrg.utils.KeyValuePair.KVPinterface;
import edu.qrg.vmodel.Model;
import edu.qrg.vmodel.Ontology.Element;
import edu.qrg.utils.PList.PLinterface;
import edu.qrg.vmodel.VModel.VMController;
import edu.qrg.utils.XMLio.XMLable;
import java.awt.Color;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Double;
import java.lang.Integer;
import java.lang.Math;
import java.lang.NumberFormatException;
import java.lang.Object;
import java.lang.String;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/references
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: Instance.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: Instance
* Purpose: Instance of a Definition.  For every definition there may be
*          many instances in a model.
* ----------------------------------------------------------------------------
* Rich in Content; Rich in Tradition; Rich in History
* Modern Masters Holiday Special - HGTV
* </PRE> */
public class Instance extends PList
		implements Cloneable, Serializable, VModelConstants, CMLio.CMLable, XMLio.XMLable {
	// Constructors
	// -------------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: General initialization.
	* ------------------------------------------------------------------------
	* </PRE>
	*/
	public Instance ( ) {
		id( composeUniqueIdentifier( ) );
		// Instances can be searched in a model.
		// Is there any reason to cache all of them?
//		allInstances( ).addElement( this );
	} // constructor

    public Instance( Instance original ) {
    	this( );
    	String id = id( );
    	plist( copyKeyValuePairs( original ).plist() );
    	id( id );
    } // constructor

    public Instance( Definition source ) {
    	this( );
    	plist( copyKeyValuePairs( source ).plist() );
    } // constructor

	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Create an instance for the referenced definition in the
	*          specified model.
	* Notes:
	* * Sets default state for QUANTITY and CAUSAL definitions.
	*   * QUANTITY states: DEFAULT, INCREASING, DECREASING, CONSTANT, UNKNOWN,
	*                      UNCONNECTED, CONTRADICTION
	*   * CAUSAL states: NORMAL, DOMINANT, NEGLIGABLE
	* * Reasons: DEFAULTASSUMPTION, USERASSUMPTION
	* ------------------------------------------------------------------------
	* </PRE>
	*/
	public Instance ( Model model, Instance instance ) {
		this( instance );
		model( model );
		geneticParent( instance );
		if ( typep( ATTRIBUTE ) || typep( PROCESSRATE ) ) {
			initialState( DEFAULT ); // DEFAULT, INCREASING, DECREASING, CONSTANT, UNKNOWN, UNCONNECTED, CONTRADICTION
			initialReason( DEFAULTASSUMPTION );
			state( initialState( ) );
			reasons( new Vector( ) ).addElement( initialReason( ) );
			predictedState( UNKNOWN );
		} else if ( typep( CAUSES ) ) {
			initialState( NORMAL ); // NORMAL, DOMINANT, NEGLIGABLE
			initialReason( DEFAULTASSUMPTION );
			state( initialState( ) );
			reasons( new Vector( ) ).addElement( initialReason( ) );
		} // end if
	} // constructor

	// Methods
	// ------------------------------------------------------------------------
	// Type
	/** Type check */
	public boolean typep ( String typeID ) {
		boolean result = false;
		Definition mumble = null;
		result = id( ).equalsIgnoreCase( typeID ) ||
			name( ).equalsIgnoreCase( typeID ) ||
			category( ).equalsIgnoreCase( typeID ) ||
			type( ).equalsIgnoreCase( typeID ) ||
			typeChain( ).contains( typeID ) ||
			typeNameChain( ).contains( typeID ) ||
			Otypep( typeID ) ||
			Jtypep( typeID );
		return( result );
	} // method

	/** Jtype check - like instanceof */
	public boolean Jtypep ( String typeID ){
		boolean result = Jtype( ).equalsIgnoreCase( typeID );
		if ( ! result ) {
			try {
				Class c = Class.forName( typeID );
				result = c.isInstance( this );
			} catch ( ClassNotFoundException e ) { }
		} // end if
		return( result );
	} // method

	/** Otype check */
	public boolean Otypep ( String typeID ){
		boolean result = Otype( ).equalsIgnoreCase( typeID );
		return( result );
	} // method

	// Properties & Accessors
	// -------------------------------------------------------------------------
	/** Use the accessors.
	    This is transient because peer information is generated upon
	    loading a model which allows us to utilize different controllers
	    in different situations. */
	public transient Peer peer = null;
	/** Peer is usually a gui artifact. */
	public Peer peer ( ) { return( peer ); }
	/** @see #peer( ) */
	public Peer peer ( Peer p ) { return( peer = p ); }

	/** Use the accessors.
	    @serial */
	public Model model = null;
	/** Model that contains the instance. */
	public Model model ( ) { return( model ); }
	/** @see #model( ) */
	public Model model ( Model m ) {
		return( model = m );
	} // writer

	/** Returns the id of the definition object from which this instance was generated. */
	public String definitionID ( ) {
		return( getString( "DEFINITIONID" ) );
	} // method
	public String definitionID ( String s ) {
		return( (String) set( "DEFINITIONID", s ) );
	} // method
	/** Returns the Definition upon which the instance is based. */
//	public Definition definition ( ) {
//		Definition result = controller( ).findDefinition( model( ), definitionID( ) );
//		return( result );
//	} // method

	/** Use the accessors.
	    @serial */
	public Instance geneticParent = null;
	/** Reference to definition upon which the instance is based. */
	public Instance geneticParent ( ) { return( geneticParent ); }
	/** @see #reference( ) */
	public Instance geneticParent ( Instance r ) {
		return( geneticParent = r );
	} // method

	// Should this be a string?
	/** Use the accessors.
	    @serial */
	public Vector parents = null;
	/**  instance in which the object is a participant, attribute,
	     condition, or consequence. */
	public Vector parents ( ) {
		if ( parents == null ) parents = new Vector( );
		return( parents );
	} // end if

	// Ontology.Element methods
	// -----------------------------------------------------------------------
	// Unique Identifier
	/** String identifier for object. */
	public String id ( ) { return( getString( ID ) ); }
	/** @see #id( ) */
	public String id ( String s ) { return( (String) set( ID, s ) ); }

	/** String label for object.
	* * May contain spaces.
	* * Doesn't have to be unique.
	*/
	public String name ( ) { return( getString( NAME ) ); }
	/** @see #name( ) */
	public String name ( String s ) { return( (String) set( NAME, s ) ); }

	// Help
	/** Documentation string for object. */
	public String documentation ( ) { return( getString( DESCRIPTION ) ); }
	/** @see #documentation( ) */
	public String documentation ( String s ) { return( (String) set( DESCRIPTION, s ) ); }

	/** URL of web-based help file. */
	public String helpURL ( ) { return( getString( HELPURL ) ); }
	/** @see #helpURL( ) */
	public String helpURL ( String s ) { return( (String) set( HELPURL, s ) ); }

	/** Purpose: Returns the string filename of an image representing
	             the Definition. */
	public String icon ( ) { return( getString( ICON ) ); }
	/** @see #icon( ) */
	public String icon ( String s ) { return( (String) set( ICON, s ) ); }

	// Category
	/** Purpose: Returns the string id of the category to which this
	             category belongs. */
	public String category ( ) { return( getString( CATEGORY ) ); }
	/** @see #category( ) */
	public String category ( String s ) { return( (String) set( CATEGORY, s ) ); }

	// type
	/** Type of definition. */
	public String type ( ) { return( getString( TYPE ) ); }
	/** @see #type( ) */
	public String type ( String s ) { return( (String) set( TYPE, s ) ); }

	/** Returns the name of type definition. */
	public String typeName ( ) {
		return( getString( TYPENAME ) );
	} // method
	/** @see #typeName( ) */
	public String typeName ( String s ) {
		return( (String) set( TYPENAME, s ) );
	} // method

	// Otype
	/** ID of base Ontology type */
	public String Otype ( ) { return( getString( QPTYPE ) ); }
	/** @see #Otype( ) */
	public String Otype ( String s ) {
		return( (String) set( QPTYPE, s ) );
	} // writer

	/** Inheritence chain of definition types. */
	public Vector typeChain ( ) { return( getVector( TYPECHAIN ) ); }
	/** @see #typeChain( ) */
	public Vector typeChain ( Vector v ) { return( (Vector) set( TYPECHAIN, v ) ); }

	/** Inheritence chain of definition names. */
	public Vector typeNameChain ( ) { return( getVector( SUBCLASSOF ) ); }
	/** @see #typeNameChain( ) */
	public Vector typeNameChain ( Vector v ) { return( (Vector) set( SUBCLASSOF, v ) ); }

	// Properties & Accessors
	// ------------------------------------------------------------------------
	// Jtype
	/** Java type */
	public String Jtype ( ) { return( getClass( ).getName( ) ); }

	/** List of relationships connected to the definition. */
	public Vector connections ( ) {
		return( getVector( CONNECTIONS ) );
	} // reader

	/** List of instances connected by a relationship. */
	public Vector connectors ( ) {
		return( getVector( CONNECTORS ) );
	} // reader

	// or should these be instances?
	public String input ( ) { return( getString( INPUT ) ); }
	public String input ( String s ) { return( (String) set( INPUT, s ) ); }
	public String output ( ) { return( getString( OUTPUT ) ); }
	public String output ( String s ) { return( (String) set( OUTPUT, s ) ); }

	public Vector causalInputs ( ) { return( getVector( CAUSALINPUTS ) ); }
	public Vector causalOutputs ( ) { return( getVector( CAUSALOUTPUTS ) ); }

	public Vector inputs ( ) { return( getVector( INPUTS ) ); }
	public Vector outputs ( ) { return( getVector( OUTPUTS ) ); }

	public String predictedState ( ) { return( getString( TARGET ) ); }
	public String predictedState ( String s ) { return( (String) set( TARGET, s ) ); }

	public String state ( ) { return( getString( STATE ) ); }
	public String state ( String s ) { return( (String) set( STATE, s ) ); }

	public Vector reasons ( ) { return( getVector( REASONS ) ); }
	public Vector reasons ( Vector v ) { return( (Vector) set( REASONS, v ) ); }

	public String initialState ( ) { return( getString( INITIALSTATE ) ); }
	public String initialState ( String s ) { return( (String) set( INITIALSTATE, s ) ); }

	public String initialReason ( ) { return( getString( INITIALREASON ) ); }
	public String initialReason ( String s ) { return( (String) set( INITIALREASON, s ) ); }

	public String notes ( ) { return( getString( NOTES ) ); }
	public String notes ( String s ) { return( (String) set( NOTES, s ) ); }

	VModel.VMController controller ( ) { return( (VModel.VMController) VModel.controller( ) ); }

	public Vector childInstances ( ) { return( getVector( CHILDINSTANCES ) ); }
	public Vector childInstances ( Vector v ) { return( (Vector) set( CHILDINSTANCES, v ) ); }

	// Methods
	// -------------------------------------------------------------------------
	public Enumeration children ( ) {
		Vector mumble = new Vector( );
		String grumble = "";
		Instance humble = null;
		for( Enumeration enum = childInstances( ).elements( ); enum.hasMoreElements( ); ) {
//			grumble = (String) enum.nextElement( );
			humble = (Instance) enum.nextElement( );
			if (humble != null) mumble.addElement(humble);
		} // end for
		return( mumble.elements( ) );
	} // method

	// Methods
	// ------------------------------------------------------------------------
	// Uniquify
	protected static long sessionCount = 0;

	public String composeUniqueIdentifier ( ) {
		String result = "VMODEL"; //getClass( ).getName( );
		result += "-" + ( sessionCount++ );
		result += "-" + ( new Date( ) ).getTime( );
		result += "-" + hashCode( );
		result += "-" + String.valueOf( (char) ( ( Math.random( ) * 25 ) + 65 ) );
		return( result );
	} // method

//	public Definition subtype ( String name ) { return( null ); }

 	// CMLio.CMLable
	// -----------------------------------------------------------------------
	public String CMLcommand ( ) { return( "defInstance" ); }
//	public String CMLcommand ( String s ) { return( "defInstance" ); }

	public String toCML ( ) {
		String result = "(" + CMLcommand( ) + " " + id( );
		KeyValuePair mumble = null;
		result += QRGutils.eol( ) + "   :Otype " + Otype( );
		result += QRGutils.eol( ) + "   :type " + type( );
//		result += QRGutils.eol( ) + "   :name \"" + name( ) + "\"";
//		result += QRGutils.eol( ) + "   :id " + definitionID( );
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			mumble = (KeyValuePair) e.nextElement();
			if ( ! mumble.key( ).equalsIgnoreCase( ID ) ) {
				if ( mumble.value( ) instanceof Vector ) {
					result += QRGutils.eol( ) + "   :" + mumble.key( ) + " (";
					for ( Enumeration values = ((Vector) mumble.value( )).elements( ); values.hasMoreElements( ); ) {
						result += values.nextElement( );
						if ( values.hasMoreElements( ) ) result += " ";
					} // end for
					result += ")";
				} else if ( mumble.key( ).equalsIgnoreCase( REFERENCE ) ) result += QRGutils.eol( ) + "   :" + mumble.key( ) + " " + mumble.value( );
				else result += QRGutils.eol( ) + "   :" + mumble.key( ) + " \"" + mumble.value( ) + "\"";
			} // end if
		} // end for
		result += ")" + QRGutils.eol( );
		return( result );
	} // method

	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Saves the ontology to filename in CML format.
	* ------------------------------------------------------------------------
	* </PRE>
	*/
	public void exportCML ( String filename ) {
		CMLio.save(  QRGutils.QRGuserpath( ) + filename + "." + CML_SUFFIX , this );
	} // method

	// XMLio.XMLable
	// -----------------------------------------------------------------------
	public String xmlDef ( ) {
		String result = 	"";
		KeyValuePair mumble = null;
		String classname = "Instance"; //this.getClass( ).getName( );
		result += "<!DOCTYPE " + classname + " [\r\n";
		result += "<!ELEMENT " + classname + " EMPTY >\r\n";
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			mumble = (KeyValuePair) e.nextElement();
			result += "<!ATTLIST " + classname + " " + mumble.key( ).toUpperCase( ) + " CDATA #REQUIRED >\r\n";
		} // end for
		result += "]>";
		return( result );
	} // method

	public String xmlify ( ) {
		String result = stringify( );
		return(result);
	} // method

	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Saves the ontology to filename in XML format.
	* ------------------------------------------------------------------------
	* </PRE>
	*/
	public void exportXML ( String filename ) {
		XMLio.save(  QRGutils.QRGuserpath( ) + filename + "." + XML_SUFFIX , this );
	} // method

	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Initializes slots after object is created by XML loader.
	* ------------------------------------------------------------------------
	* </PRE>
	*/
	public void XMLioObjectInit ( Vector attr, Vector elmnt ) {
		Object mumble = null;
		VectorElement grumble = null;
		keyValuePairs( attr );
		for( Enumeration enum = elmnt.elements( ); enum.hasMoreElements( ); ) {
			mumble = enum.nextElement( );
			if ( mumble instanceof VectorElement ) {
				grumble = (VectorElement) mumble;
				getVector( grumble.getString( TAG ) ).addElement( grumble.get( VALUE ) );
			} // end if
		} // end for
	} // constructor PList

	// Stringification.
	// -----------------------------------------------------------------------
	public String stringifyKeyValuePairs ( int indent ) {
		String result = "";
		int j = 0;
		KeyValuePair candidate = null;
		if (keyValuePairs() != null) {
			j = 0;
			while( j < keyValuePairs().size( ) ) {
				candidate = (KeyValuePair) keyValuePairs().elementAt(j);
				if ( ! ( candidate.value( ) instanceof Vector ) ) {
					if (j > 0) result += QRGutils.eol( ) + QRGutils.space(indent);
					else result += " ";
					result += candidate;
				} // end if
				j++;
			} // while
		} // end if
		return(result);
	} // method toString

	public String stringify ( int indent ) {
		String result = "";
		KeyValuePair pair = null;
		Object mumble = null;
		final VectorElement grumble = new VectorElement( );
		boolean flag = true;
		result = QRGutils.space( indent ) + "<" + "Instance";
		result += stringifyKeyValuePairs( result.length( ) + 1 );

		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			pair = (KeyValuePair) e.nextElement( );
			if ( ( pair.value( ) instanceof Vector ) &&
					( ! ( (Vector) pair.value( ) ).isEmpty( ) ) ) {
				flag = false;
				break;
			} // end if
		} // end for
		int tabs = indent + 3;
		if ( flag ) result += " />" + QRGutils.eol( );
		else {
			result += " >" + QRGutils.eol( );
			for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
				pair = (KeyValuePair) e.nextElement( );
				if ( pair.value( ) instanceof Vector ) {
					for (Enumeration p = ((Vector) pair.value( )).elements( ) ; p.hasMoreElements() ;) {
						mumble = p.nextElement( );
						grumble.set( TAG, pair.key( ) );
						grumble.set( VALUE, mumble );
						result += QRGutils.space( tabs ) + grumble.toString( );
					} // end for
				} // end if
			} // end for
			result += QRGutils.space( indent ) + "</" + "Instance" + ">" + QRGutils.eol( );
		} // end if
//		result += QRGutils.eol( );
		return( result );
	} // method toString

	public String stringify ( ) {
		return( stringify( 0 ) );
	} // method toString

	public String toString ( int i ) {
		return( stringify( i ) );
	} // method

	public String toString ( ) {
		return( id( ) );
	} // method

	public boolean checkInstance ( ) {
		boolean result = true;
		boolean check = false;
		Instance source = null;
		Instance target = null;
		set( "PROBLEMS", new Vector( ) );
		if ( typep( RELATIONSHIP ) ) {
			if (( input( ) != null ) && (! input( ).equalsIgnoreCase("") ) ) source = model( ).findInstance( input( ) );
			if (( output( ) != null ) && (! output( ).equalsIgnoreCase("") ) ) target = model( ).findInstance( output( ) );
			check = ( source != null );
			if ( ! check ) {
				problems( ).addElement( "Error #0" ); // Relationship not fully connected.
				problems( ).addElement( "This relationship is not connected to an input." ); // Relationship not fully connected.
				result = false;
			} else {
			check = ( target != null );
			if ( ! check ) {
				problems( ).addElement("Error #1" ); // Relationship not fully connected.
				problems( ).addElement( "This relationship is not connected to an output." ); // Relationship not fully connected.
				result = false;
			} else {
			check = ( source != null ) && ( target != null ) && ( target != source );
			if ( ! check ) {
				problems( ).addElement("Error #2"); // Both ends of relationship connected to same object.
				problems( ).addElement( "The input and output of this relationship are the same." ); // Both ends of relationship connected to same object.
				result = false;
			} // end if
			if ( typep(DIRECTINFLUENCE) ) {
				check = ( source != null ) && ( target != null ) && (source.typep( PROCESSRATE ) || source.typep( RATE ) );
				if ( ! check ) {
					problems( ).addElement("Error #3"); // Improperly connected Direct Influence
					problems( ).addElement( "Only a process " + typeName() + " something." ); // Improperly connected Direct Influence
					result = false;
				} // end if
				check = ( source != null ) && ( target != null ) && (source.typep( PROCESSRATE ) || source.typep( RATE ) ) && ( target.typep( QUANTITY ) || target.typep( PROCESSRATE ) );
				if ( ! check ) {
					problems( ).addElement("Error #4"); // Improperly connected Direct Influence
					problems( ).addElement(typeName( ) + " can only connect a process with a rate and a measurable." ); // Improperly connected Direct Influence
					result = false;
				} // end if
			} else if ( typep(INDIRECTINFLUENCE) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && ( source.typep( QUANTITY ) || source.typep( PROCESSRATE ) ) && ( target.typep( QUANTITY ) || target.typep( PROCESSRATE ) );
				if ( ! check ) {
					problems( ).addElement("Error #5"); // Improperly connected Indirect Influence
					problems( ).addElement( typeName( ) + " can only come from or go to measurables or processes with rates." ); // Improperly connected Indirect Influence
					result = false;
				} // end if
			} else if ( typep(TOUCHES) || typep( CONTAINS ) || typep( ISPARTOF ) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) &&
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) )  && ( target.typep( ENTITY ) && ( ! target.typep( PROCESS ) ) );
				if ( ! check ) {
					problems( ).addElement( "Error #6" ); // Improperly connected Spatial Constraints
					problems( ).addElement( typeName( ) + " must be connected between two things, multiple things, or substances." ); // Improperly connected Spatial Constraints
					result = false;
				} // end if
			} else if ( typep(DOES) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) &&
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) );
				if ( ! check ) {
					problems( ).addElement( "Error #7" ); // Improperly connected Does
					problems( ).addElement( "The does arrow should start at a thing, multiple thing or substance." ); // Improperly connected Does
					result = false;
				} // end if
				check = ( source != null ) && ( target != null ) && ( source != target ) &&
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) )  && ( target.typep( PROCESS ) );
				if ( ! check ) {
					problems( ).addElement("Error #8"); // Improperly connected Does
					problems( ).addElement( "Nothing can do a " + target.typeName( ) ); // Improperly connected Does
					result = false;
				} // end if
			} else if ( typep(HASATTRIBUTE) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) &&
					( source.typep( ENTITY ) )  && ( target.typep( ATTRIBUTE ) );
				if ( ! check ) {
					problems( ).addElement( "Error #9" ); // Improperly connected Has Attribute
					problems( ).addElement( "Has-Attribute can only connect a thing, multiple thing, substance, or process with a measurable." ); // Improperly connected Has Attribute
					result = false;
				} // end if
			} else if ( typep(HASPARTICIPANT) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) &&
					( source.typep( PROCESS ) )  && ( target.typep( ENTITY ) && ( ! target.typep( PROCESS ) ) );
				if ( ! check ) {
					problems( ).addElement( "Error #10" ); // Improperly connected Has Participant
					problems( ).addElement( "Has-Participant can only connect a process with a thing, multiple thing, or substance." ); // Improperly connected Has Participant
					result = false;
				} // end if
			} else if ( typep(PARTICIPATESIN) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) &&
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) )  && ( target.typep( PROCESS ) );
				if ( ! check ) {
					problems( ).addElement( "Error #11" ); // Improperly connected Particpates In
					problems( ).addElement( "Participates-In can only connect a thing, multiple thing, or substance with a process." ); // Improperly connected Particpates In
					result = false;
				} // end if
			} else if ( typep(CONSTRAINT) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) &&
					( source.typep( QUANTITY ) )  && ( target.typep( QUANTITY ) );
				if ( ! check ) {
					problems( ).addElement( "Error #12" ); // Improperly connected Constraint
					problems( ).addElement("A comparer can only connect two measurables."); // Improperly connected Constraint
					result = false;
				} // end if
			} // end if
			} // end if
			} // end if
		} else if ( typep( QUANTITY ) || typep( PROCESSRATE ) ) {
			Vector direct = new Vector( );
			Vector indirect = new Vector( );
			String connectionID = null;
			Instance instance = null;
			for( Enumeration connections = causalInputs().elements( ); connections.hasMoreElements(); ) {
				connectionID = (String) connections.nextElement();
				if ( connectionID != null ) {
					instance = model( ).findInstance(connectionID);
					if ( instance != null ) {
						if ( instance.typep( DIRECTINFLUENCE ) ) direct.addElement( instance );
						if ( instance.typep( INDIRECTINFLUENCE ) ) indirect.addElement( instance );
					} // end if
				} // end if
			} // end for
			check = ( ! ( ( direct.size() > 0 ) && ( indirect.size() > 0 ) ) );
			if ( ! check ) {
				problems( ).addElement( "Error #13" ); // Direct and Indirect connected to same attribute
				problems( ).addElement( name( ) + " cannot have both an influences and a increases or decreases as input at the same time." ); // Direct and Indirect connected to same attribute
				result = false;
			} // end if
		} // end if
		return( result );
	} // method

	public Vector problems ( ) { return( getVector( "PROBLEMS" ) ); }
	public Vector problems ( String s ) {
		Vector result = problems( );
		result.addElement( s );
		return( result );
	} // method

	public void updatePeer ( ) {
		if ( peer( ) != null ) peer( ).updatePeer( );
	} // method

} // class

// ---------------------------------------------------------------------------
// End of File: Instance.java              (c) 2003 Northwestern University
