//package edu.qrg.vmodel.references;

import CMLio.CMLable;
import Definition;
import KeyValuePair.KVPinterface;
import Model.Interface;
import Ontology.Element;
import PList.PLinterface;
import Reference.RefInterface;
import VModel.VMController;
import XMLio.XMLable;
import java.awt.Color;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Double;
import java.lang.Integer;
import java.lang.Math;
import java.lang.NumberFormatException;
import java.lang.Object;
import java.lang.String;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/references
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: Instance.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: Instance
* Purpose: Instance of a Definition.  For every definition there may be
*          many instances in a model. 
* ----------------------------------------------------------------------------
* Rich in Content; Rich in Tradition; Rich in History
* Modern Masters Holiday Special - HGTV
* </PRE> */
public class Instance implements Cloneable, Serializable, Instance.InstInterface {
	// Constructors
	// -------------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: General initialization.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public Instance ( ) {
		id( composeUniqueIdentifier( ) );
		// Instances can be searched in a model.
		// Is there any reason to cache all of them?
//		allInstances( ).addElement( this );
	} // constructor
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Create an instance for the referenced definition in the 
	*          specified model.
	* Notes: 
	* * Sets default state for QUANTITY and CAUSAL definitions.
	*   * QUANTITY states: DEFAULT, INCREASING, DECREASING, CONSTANT, UNKNOWN,
	*                      UNCONNECTED, CONTRADICTION
	*   * CAUSAL states: NORMAL, DOMINANT, NEGLIGABLE
	* * Reasons: DEFAULTASSUMPTION, USERASSUMPTION
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public Instance ( Model.Interface model, Reference.RefInterface reference ) {
		this( );
		model( model );
		reference( reference );
		if ( typep( ATTRIBUTE ) || typep( PROCESSRATE ) ) {
			initialState( DEFAULT ); // DEFAULT, INCREASING, DECREASING, CONSTANT, UNKNOWN, UNCONNECTED, CONTRADICTION
			initialReason( DEFAULTASSUMPTION );
			state( initialState( ) );
			reasons( new Vector( ) ).addElement( initialReason( ) );
			predictedState( UNKNOWN );
		} else if (  controller( ).findDefinition( model, reference.reference( ) ).typep( CAUSES ) ) {
			initialState( NORMAL ); // NORMAL, DOMINANT, NEGLIGABLE
			initialReason( DEFAULTASSUMPTION );
			state( initialState( ) );
			reasons( new Vector( ) ).addElement( initialReason( ) );
		} // end if
/*
		if ( typep( PROCESSRATE ) ) set( INITIALSTATE, DEFAULT );
		if ( typep( PROCESSRATE ) ) set( INITIALSTATE, DEFAULTASSUMPTION );
		if ( typep( ATTRIBUTE ) ) set( INITIALSTATE, DEFAULT );
		if ( typep( ATTRIBUTE ) ) set( INITIALSTATE, DEFAULTASSUMPTION );
		if ( typep( CAUSALRELATIONSHIP ) ) set( INITIALSTATE, NORMAL );
		if ( typep( CAUSALRELATIONSHIP ) ) set( INITIALSTATE, DEFAULTASSUMPTION );
*/		
	} // constructor
	
	// Interface
	// ------------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Interface: Instance.InstInterface
	* Purpose: In general, anything implementing this interface should be
	*          acceptable as an Instance.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public interface InstInterface extends Cloneable, Serializable, VModelConstants, 
			CMLio.CMLable, XMLio.XMLable, PList.PLinterface {
		/** Peer is an interface representing a gui artifact that may be tied
		    to an instance.  Instances and Peers are bound in a 1-1 pairing.
		    Each pairing must be unique. */
		public Peer peer ( );
		/** @see #peer( ) */
		public Peer peer ( Peer p );

		/** Model to which instance belongs. */
		public Model.Interface model ( );
		/** @see #model( ) */
		public Model.Interface model ( Model.Interface m );
		
		/** Returns the id of the definition object upon which the
		    instance is based. */
		public String definitionID ( ); 
		/** Returns the Definition upon which the instance is based. */
		public Definition definition ( );

		/** Reference to definition upon which the instance is based. */
		public Reference.RefInterface reference ( );
		/** @see #reference( ) */
		public Reference.RefInterface reference ( Reference.RefInterface r );

		/** Instance that contains this one as a connector or connection. */
		public Instance.InstInterface parent ( );
		/** @see #parent( ) */
		public Instance.InstInterface parent ( Instance.InstInterface p );

		/** Return the unique string identifier for the object. */
		public String id ( );
		/** Set & return the unique string identifier for the object. */
		public String id ( String s );

		/** Return a string name for the object. */
		public String name ( );
		/** Set & return a string name for the object. */
		public String name ( String s );

		/** Return a documentation string for the object. */
		public String documentation ( );
		/** Sets & return a documentation string for the object. */
		public String documentation ( String s );
		
		/** Return a string filename pointing to a web-based
		    documentation file for the object. */
		public String helpURL ( );
		/** Sets & returns a string filename pointing to a web-based
		    documentation file for the object. */
		public String helpURL ( String s );
		
		// Do Instances need an Icon?
		/** Returns a string filename pointing to a gif or jpg file.
		    Filename does not have to include file-extension. */
//		public String icon ( );
		/** Sets * returns a string filename pointing to a gif or jpg file.
		    Filename does not have to include file-extension. */
//		public String icon ( String name );
//		public String icon ( boolean expanded );
		
		/** Id of category to which the ontological element belongs. */
		public String category ( );
		/** @see #category( ) */
		public String category ( String s );

		/** Type of ontological element. */
		public String type ( );
		/** @see #type( ) */
		public String type ( String s );
	
		/** Base ontological or semantic type of the definition. */
		public String Otype ( );
		/** @see #Otype( ) */
		public String Otype ( String s );

		/** Java type of the definition - corresponds to class type. */
		public String Jtype ( );
		/** @see #Jtype( ) */
		public String Jtype ( String s );

		/** Type check. */
		public boolean typep ( String type );
		/** Otype check. */
		public boolean Otypep ( String Otype );
		/** Jtype check. ( similar to instanceof ) */
		public boolean Jtypep ( String Jtype );
		
		/** Because type is related to the unique ids,
		    a type name must be looked up.
		    Returns the name of the parent type. */
		public String typeName ( );
		/** @see #typeName( ) */
		public String typeName ( String s );

		/** inheritence chain of ids */
		public Vector typeChain ( );
		/** inheritence chain of names */
		public Vector typeNameChain ( );

		// hmmmmm......
//		public Definition subtype ( String name );

		/** Represent object as string. */
		public String stringify ( );
		/** Represent object as string with indentation. */
		public String stringify ( int indent );

		/** References to relationship definitions. */
		public Vector connections ( );
		/** References to Process, Entity, Model, and Attribute definitions. */
		public Vector connectors ( );

		/** List of all connected elements (connectors). */
		public Enumeration children ( );

		// Split out into specialized classes?
//		public Vector participants ( );
//		public Vector quantities ( );
//		public Vector conditions ( );
//		public Vector consequences ( );
//		public Vector structure ( );

		/** Set the string used to represent the definition in CML. */
//		public String CMLcommand ( String s );

		// Local requirements
		/** Student annotations. */
		public String notes ( );
		/** @see #notes( ) */
		public String notes ( String s );

		// For relationship instances.
		/** instance connected to input of relationship instance. */
		public String input ( );
		/** @see #input( ) */
		public String input ( String s );
		/** Instance connected to output of relationship instance. */
		public String output ( );
		/** @see #output( ) */
		public String output ( String s );

		// For entities and attributes
		/** Relationships connected as inputs. */
		public Vector inputs ( );
		/** Relationships connected as outputs. */
		public Vector outputs ( );
		/** Causal relationships connected as inputs. */
		public Vector causalInputs ( );
		/** Causal relationships connected as outputs. */
		public Vector causalOutputs ( );
	
		/** List of connected entities. */
		public Vector childInstances ( );
		/** @see #childInstances( ) */
		public Vector childInstances ( Vector v );

		// Attributes
		/** Predicted result of simulation. */
		public String predictedState ( );
		/** @see #predictedState( ) */
		public String predictedState ( String s );

		/** Result of simulation. */
		public String state ( );
		/** @see #state( ) */
		public String state ( String s );
	
		/** Reason for the simulation result. */
		public Vector reasons ( );
		/** @see #reasons( ) */
		public Vector reasons ( Vector v );

		/** Initial state at start of simulation. */
		public String initialState ( );
		/** @see #initialState( ) */
		public String initialState ( String s );

		/** Reason for initial state. */
		public String initialReason ( );
		/** @see #initialReason( ) */
		public String initialReason ( String s );

		public boolean checkInstance ( );
	} // interface

	// Properties & Accessors
	// -------------------------------------------------------------------------
	/** Use accessor. 
	    @serial */
	public PList plist = null;
	/** Property list for stashing local values. */
	public PList plist ( ) {
		if ( plist == null ) plist = new PList( );
		return( plist );
	} // reader
	
	/** List of key-value pairs contained in plist. */
	public Vector keyValuePairs ( ) { return( plist( ).keyValuePairs( ) ); }
	/** @see #keyValuePairs( ) */
	public Vector keyValuePairs ( Vector v ) { return( plist( ).keyValuePairs( v ) ); }

	/** Use the accessors. 
	    This is transient because peer information is generated upon
	    loading a model which allows us to utilize different controllers
	    in different situations. */
	public transient Peer peer = null;
	/** Peer is usually a gui artifact. */
	public Peer peer ( ) { return( peer ); }
	/** @see #peer( ) */
	public Peer peer ( Peer p ) { return( peer = p ); }

	/** Use the accessors. 
	    @serial */
	public Model.Interface model = null;
	/** Model that contains the instance. */
	public Model.Interface model ( ) { return( model ); }
	/** @see #model( ) */
	public Model.Interface model ( Model.Interface m ) { 
		return( model = m ); 
	} // writer

	/** Returns the id of the definition object upon which the object is based. */
	public String definitionID ( ) { 
		return( reference( ).reference( ) ); 
	} // method
	/** Returns the Definition upon which the instance is based. */
	public Definition definition ( ) { 
		Definition result = controller( ).findDefinition( model( ), definitionID( ) );
		return( result );
	} // method

	/** Use the accessors.
	    @serial */
	public Reference.RefInterface reference = null; 
	/** Reference to definition upon which the instance is based. */
	public Reference.RefInterface reference ( ) { return( reference ); }
	/** @see #reference( ) */
	public Reference.RefInterface reference ( Reference.RefInterface r ) { 
		return( reference = r ); 
	} // method

	// Should this be a string?
	/** Use the accessors. 
	    @serial */
	public Instance.InstInterface parent = null;
	/**  instance in which the object is a participant, attribute, 
	     condition, or consequence. */
	public Instance.InstInterface parent ( ) { return( parent ); }
	/** @see #parent( ) */
	public Instance.InstInterface parent ( Instance.InstInterface p ) {
		return( parent = p );
	} // writer
	
	/** Unique identifier. */
	public String id ( ) { return( getString( ID ) ); }
	/** @see #id( ) */
	public String id ( String s ) { return( (String) set( ID, s ) ); }
	
	/** Return a string name for the object. */
	public String name ( ) { return( definition( ).name( ) ); }
	/** Set & return a string name for the object. */
	public String name ( String s ) { 
		controller( ).rename( model( ), this, s );
		return( name( ) ); 
	} // method
	
	/** Return a documentation string for the object. */
	public String documentation ( ) { return( definition( ).documentation( ) ); }
	/** Sets & return a documentation string for the object. */
	public String documentation ( String s ) { return( definition( ).documentation( s ) ); }
	
	/** Return a string filename pointing to a web-based
	    documentation file for the object. */
	public String helpURL ( ) { return( definition( ).helpURL( ) ); }
	/** Sets & returns a string filename pointing to a web-based
	    documentation file for the object. */
	public String helpURL ( String s ) { return( definition( ).helpURL( s ) ); }
	
	/** Id of category to which the ontological element belongs. */
	public String category ( ) { return( definition( ).category( ) ); }
	/** @see #category( ) */
	public String category ( String s ) { return( definition( ).category( s ) ); }

	/** Type of ontological element. */
	public String type ( ) { return( definition( ).type( ) ); }
	/** @see #type( ) */
	public String type ( String s ) { return( definition( ).type( s ) ); }

	/** Because type is related to the unique ids,
	    a type name must be looked up.
	    Returns the name of the parent type. */
	public String typeName ( ) { return( definition( ).typeName( ) ); }
	/** @see #typeName( ) */
	public String typeName ( String s ) { return( definition( ).typeName( s ) ); }

	/** inheritence chain of ids */
	public Vector typeChain ( ) { return( definition( ).typeChain( ) ); }
	/** inheritence chain of names */
	public Vector typeNameChain ( ) { return( definition( ).typeNameChain( ) ); }

	/** Base ontological or semantic type of the definition. */
	public String Otype ( ) { return( definition( ).Otype( ) ); }
	/** @see #Otype( ) */
	public String Otype ( String s ) { return( definition( ).Otype( s ) ); }

	/** Java type of the definition - corresponds to class type. */
	public String Jtype ( ) { return( definition( ).Jtype( ) ); }
	/** @see #Jtype( ) */
	public String Jtype ( String s ) { return( definition( ).Jtype( s ) ); }

	/** Type check. */
	public boolean typep ( String type ) { return( definition( ).typep( type ) ); }
	/** Otype check. */
	public boolean Jtypep ( String type ) { return( definition( ).Jtypep( type ) ); }
	/** Jtype check. */
	public boolean Otypep ( String type ) { return( definition( ).Otypep( type ) ); }
		
	// or should these be instances?
	public String input ( ) { return( getString( INPUT ) ); }
	public String input ( String s ) { return( (String) set( INPUT, s ) ); }
	public String output ( ) { return( getString( OUTPUT ) ); }
	public String output ( String s ) { return( (String) set( OUTPUT, s ) ); }

	public Vector causalInputs ( ) { return( getVector( CAUSALINPUTS ) ); }
	public Vector causalOutputs ( ) { return( getVector( CAUSALOUTPUTS ) ); }

	public Vector inputs ( ) { return( getVector( INPUTS ) ); }
	public Vector outputs ( ) { return( getVector( OUTPUTS ) ); }

	public String predictedState ( ) { return( getString( TARGET ) ); }
	public String predictedState ( String s ) { return( (String) set( TARGET, s ) ); }

	public String state ( ) { return( getString( STATE ) ); }
	public String state ( String s ) { return( (String) set( STATE, s ) ); }

	public Vector reasons ( ) { return( getVector( REASONS ) ); }
	public Vector reasons ( Vector v ) { return( (Vector) set( REASONS, v ) ); }

	public String initialState ( ) { return( getString( INITIALSTATE ) ); }
	public String initialState ( String s ) { return( (String) set( INITIALSTATE, s ) ); }

	public String initialReason ( ) { return( getString( INITIALREASON ) ); }
	public String initialReason ( String s ) { return( (String) set( INITIALREASON, s ) ); }

	public String notes ( ) { return( getString( NOTES ) ); }
	public String notes ( String s ) { return( (String) set( NOTES, s ) ); }

	VModel.VMController controller ( ) { return( (VModel.VMController) VModel.controller( ) ); }

	public Vector connections ( ) { return( definition( ).connections( ) ); }

	public Vector connectors ( ) { return( definition( ).connectors( ) ); }

//	public Vector participants ( ) { 
//		return( definition( ).participants( ) ); 
//		return( null ); 
//	} // method

//	public Vector quantities ( ) { 
//		return( definition( ).quantities( ) ); 
//		return( null ); 
//	} // method
	
//	public Vector conditions ( ) { 
//		return( definition( ).conditions( ) ); 
//		return( null ); 
//	} // method

//	public Vector consequences ( ) { 
//		return( definition( ).consequences( ) ); 
//		return( null ); 
//	} // method
	
//	public Vector structure ( ) { 
//		return( definition( ).structure( ) ); 
//		return( null ); 
//	} // method

	public static final String CHILDPEERS = "CHILDPEERS";
	public Vector childInstances ( ) { return( getVector( CHILDPEERS ) ); }
	public Vector childInstances ( Vector v ) { return( (Vector) set( CHILDPEERS, v ) ); }

	// Methods
	// -------------------------------------------------------------------------
	public Enumeration children ( ) {
		Vector mumble = new Vector( );
		String grumble = "";
		Instance.InstInterface humble = null;
		for( Enumeration enum = childInstances( ).elements( ); enum.hasMoreElements( ); ) {
//			grumble = (String) enum.nextElement( );
			humble = (Instance) enum.nextElement( );
			if (humble != null) mumble.addElement(humble);
		} // end for
		return( mumble.elements( ) );
	} // method
	
//	public String icon ( ) { return( getString( ICON ) ); }
//	public String icon ( String s ) { return( (String) set( ICON, s ) ); }
//	public String icon ( boolean expanded ) { 
//		String result = icon( );
//		if ( expanded ) result += "Open";
//		else result += "Closed";
//		return( result ); 
//	} // method
	
//	protected static Vector allInstances = null;
//	public static Vector allInstances ( ) {
//		if ( allInstances == null ) allInstances = new Vector( ); 
//		return( allInstances ); 
//	} // reader
//	public static Vector allInstances ( Vector v ) { 
//		return( allInstances = v ); 
//	} // writer allInstances
	
	// Methods
	// ------------------------------------------------------------------------
	// Uniquify
	protected static long sessionCount = 0;

	public String composeUniqueIdentifier ( ) {
		String result = getClass( ).getName( ); 
		result += "-" + ( sessionCount++ );
		result += "-" + ( new Date( ) ).getTime( ); 
		result += "-" + hashCode( );
		result += "-" + String.valueOf( (char) ( ( Math.random( ) * 25 ) + 65 ) );
		return( result );
	} // method 
	
	public Definition subtype ( String name ) { return( null ); }

	public Vector keys ( ) {
		Vector result = new Vector( );
		KeyValuePair.KVPinterface candidate = null;
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			candidate =  (KeyValuePair.KVPinterface) e.nextElement();
			result.addElement( candidate.key( ) );
		} // end for
		return( result );
	} // method
	
	public Vector values ( ) {
		Vector result = new Vector( );
		KeyValuePair.KVPinterface candidate = null;
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			candidate =  (KeyValuePair.KVPinterface) e.nextElement();
			result.addElement( candidate.value( ) );
		} // end for
		return( result );
	} // method
	
    // Add & Remove KeyValuePairs
    // -----------------------------------------------------------------------
    public void add ( String key, Object value ) {
    	plist( ).add( key, value );
    } // method

    public boolean remove ( String key ) {
    	return( plist( ).remove( key ) );
    } // method
    
	// Get keyValuePair values.
	// -----------------------------------------------------------------------
    public Object get ( String tag ) {
		Object result = plist().get(tag);
		return(result);
    } // method get

    public Color getColor ( String tag ) {
		Color result = Color.black;
		Object value = null;
		value = plist( ).get(tag);
		if ( value != null ) result = QRGutils.matchColor((String) value);
		return result;
    } // method getColor

    public int getRGBColor ( String tag ) {
		int result = 0;
		Color color = getColor(tag);
		if (color != null) result = color.getRGB();
		return(result);
    } // method getRGBColor

    public int getInt ( String tag ) {
		int result = 0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			try {result = Integer.parseInt(value);}
			catch(NumberFormatException e) {}
		} // end if
		return result;
    } // method getInt

    public double getDouble ( String tag ) {
		double result = 0.0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			try {result = Double.valueOf(value).doubleValue();}
			catch(NumberFormatException e) {}
		} // end if
		return(result);
    } // method getDouble

    public boolean getBoolean ( String tag ) {
		boolean result = false;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			result = value.equalsIgnoreCase("TRUE");
		} // end if
		return(result);
    } // method getBoolean

    public String getString ( String tag ) {
		String result = "";
		String value = (String) get(tag);
		if (value != null) result = value;
		return(result);
    } // method getString

	public boolean containsString ( String key, String target ) {
		boolean result = false;
		String value = getString(key);
		result =  (value.indexOf(target)>=0);
		return(result);
	} // method getDouble
	
	public Vector getVector ( String tag ) {
		Vector result = null;
		Vector value = (Vector) get(tag);
		if (value != null) result = value;
		else { 
			result = new Vector( );
			set( tag, result );
		} // end if else
		return(result);
	} // method getString

	// Set KeyValuePair values.
	// -----------------------------------------------------------------------
    public Object set ( String tag, Object value ) {
		Object result = null;
		result = plist().set(tag, value);
		return(result);
    } // method set

    public Object set ( String tag, int value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, boolean value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, double value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, Color value ) {
		return(set(tag, "" + value.getRGB()));
    } // method set
    
 	// CMLio.CMLable
	// -----------------------------------------------------------------------
	public String CMLcommand ( ) { return( "defInstance" ); }
	public String CMLcommand ( String s ) { return( "defInstance" ); }

	public String toCML ( ) {
		String result = "(" + CMLcommand( ) + " " + id( );
		KeyValuePair mumble = null;
		result += QRGutils.eol( ) + "   :Otype " + Otype( );
		result += QRGutils.eol( ) + "   :type " + type( );
		result += QRGutils.eol( ) + "   :name \"" + name( ) + "\"";
		result += QRGutils.eol( ) + "   :id " + definition( ).id( );
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			mumble = (KeyValuePair) e.nextElement();
			if ( ! mumble.key( ).equalsIgnoreCase( ID ) ) {
				if ( mumble.value( ) instanceof Vector ) {
					result += QRGutils.eol( ) + "   :" + mumble.key( ) + " (";
					for ( Enumeration values = ((Vector) mumble.value( )).elements( ); values.hasMoreElements( ); ) {
						result += values.nextElement( );
						if ( values.hasMoreElements( ) ) result += " ";
					} // end for
					result += ")";
				} else if ( mumble.key( ).equalsIgnoreCase( REFERENCE ) ) result += QRGutils.eol( ) + "   :" + mumble.key( ) + " " + mumble.value( );
				else result += QRGutils.eol( ) + "   :" + mumble.key( ) + " \"" + mumble.value( ) + "\"";
			} // end if
		} // end for
		result += ")" + QRGutils.eol( );
		return( result );
	} // method
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Saves the ontology to filename in CML format.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public void exportCML ( String filename ) {
		CMLio.save(  QRGutils.QRGuserpath( ) + filename + "." + CML_SUFFIX , this );
	} // method
	
	// XMLio.XMLable
	// -----------------------------------------------------------------------
	public String xmlDef ( ) { 
		String result = 	"";
		KeyValuePair mumble = null;
		String classname = this.getClass( ).getName( );
		result += "<!DOCTYPE " + classname + " [\r\n";
		result += "<!ELEMENT " + classname + " EMPTY >\r\n";
		for (Enumeration e = plist( ).keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			mumble = (KeyValuePair) e.nextElement();
			result += "<!ATTLIST " + classname + " " + mumble.key( ).toUpperCase( ) + " CDATA #REQUIRED >\r\n";
		} // end for
		result += "]>";
		return( result ); 
	} // method
	
	public String xmlify ( ) {
		String result = stringify( );
		return(result);
	} // method
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Saves the ontology to filename in XML format.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public void exportXML ( String filename ) {
		XMLio.save(  QRGutils.QRGuserpath( ) + filename + "." + XML_SUFFIX , this );
	} // method
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Initializes slots after object is created by XML loader.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public void XMLioObjectInit ( Vector attr, Vector elmnt ) {
		Object mumble = null;
		VectorElement grumble = null;
		keyValuePairs( attr );
		for( Enumeration enum = elmnt.elements( ); enum.hasMoreElements( ); ) {
			mumble = enum.nextElement( );
			if ( mumble instanceof VectorElement ) {
				grumble = (VectorElement) mumble;
				getVector( grumble.getString( TAG ) ).addElement( grumble.get( VALUE ) );
			} // end if
		} // end for
//		allInstances( ).addElement( this );
	} // constructor PList
	
	// Stringification.
	// -----------------------------------------------------------------------
	public String stringifyKeyValuePairs ( int indent ) {
		String result = "";
		int j = 0;
		KeyValuePair candidate = null;
		if (plist( ).keyValuePairs() != null) {
			j = 0;
			while( j < plist( ).keyValuePairs().size( ) ) {
				candidate = (KeyValuePair) plist( ).keyValuePairs().elementAt(j);
				if ( ! ( candidate.value( ) instanceof Vector ) ) {
					if (j > 0) result += QRGutils.eol( ) + QRGutils.space(indent);
					else result += " "; 
					result += candidate;
				} // end if
				j++;
			} // while
		} // end if
		return(result);
	} // method toString
	
	public String stringify ( int indent ) {
		String result = "";
		KeyValuePair pair = null;
		Object mumble = null;
		final VectorElement grumble = new VectorElement( );
		boolean flag = true;
		result = QRGutils.space( indent ) + "<" + this.getClass( ).getName( );
		result += stringifyKeyValuePairs( result.length( ) + 1 );
		for (Enumeration e = plist( ).keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			pair = (KeyValuePair) e.nextElement( );
			if ( ( pair.value( ) instanceof Vector ) &&
					( ! ( (Vector) pair.value( ) ).isEmpty( ) ) ) {
				flag = false;
				break;
			} // end if
		} // end for
		int tabs = indent + 3;
		if ( ! flag ) result += " />" + QRGutils.eol( );
		else {
			result += " >" + QRGutils.eol( );
			for (Enumeration e = plist( ).keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
				pair = (KeyValuePair) e.nextElement( );
				if ( pair.value( ) instanceof Vector ) {
					for (Enumeration p = ((Vector) pair.value( )).elements( ) ; p.hasMoreElements() ;) {
						mumble = p.nextElement( );
						grumble.set( TAG, pair.key( ) );
						grumble.set( VALUE, mumble );
						result += QRGutils.space( tabs ) + grumble.toString( );
					} // end for
				} // end if
			} // end for
			result += QRGutils.space( indent ) + "</" + this.getClass( ).getName( ) + ">" + QRGutils.eol( );
		} // end if 
//		result += QRGutils.eol( );
		return( result );
	} // method toString
	
	public String stringify ( ) { 
		return( stringify( 0 ) ); 
	} // method toString
	
	public String toString ( int i ) {
		return( stringify( i ) );
	} // method
	
	public String toString ( ) {
		return( id( ) );
	} // method

	public boolean checkInstance ( ) {
		boolean result = true;
		boolean check = false;
		Instance.InstInterface source = null;
		Instance.InstInterface target = null;
		set( "PROBLEMS", new Vector( ) );
		if ( typep( RELATIONSHIP ) ) {
			if (( input( ) != null ) && (! input( ).equalsIgnoreCase("") ) ) source = model( ).findInstance( input( ) );
			if (( output( ) != null ) && (! output( ).equalsIgnoreCase("") ) ) target = model( ).findInstance( output( ) );
			check = ( source != null );
			if ( ! check ) {
				getVector( "PROBLEMS" ).addElement( "Error #0" ); // Relationship not fully connected.
				getVector( "PROBLEMS" ).addElement( "This relationship is not connected to an input." ); // Relationship not fully connected.
				result = false;
			} else {
			check = ( target != null );
			if ( ! check ) {
				getVector( "PROBLEMS" ).addElement("Error #1" ); // Relationship not fully connected.
				getVector( "PROBLEMS" ).addElement( "This relationship is not connected to an output." ); // Relationship not fully connected.
				result = false;
			} else {
			check = ( source != null ) && ( target != null ) && ( target != source );
			if ( ! check ) {
				getVector( "PROBLEMS" ).addElement("Error #2"); // Both ends of relationship connected to same object.
				getVector( "PROBLEMS" ).addElement( "The input and output of this relationship are the same." ); // Both ends of relationship connected to same object.
				result = false;
			} // end if
			if ( typep(DIRECTINFLUENCE) ) {
				check = ( source != null ) && ( target != null ) && (source.typep( PROCESSRATE ) || source.typep( RATE ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement("Error #3"); // Improperly connected Direct Influence
					getVector( "PROBLEMS" ).addElement( "Only a process " + typeName() + " something." ); // Improperly connected Direct Influence
					result = false;
				} // end if
				check = ( source != null ) && ( target != null ) && (source.typep( PROCESSRATE ) || source.typep( RATE ) ) && ( target.typep( QUANTITY ) || target.typep( PROCESSRATE ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement("Error #4"); // Improperly connected Direct Influence
					getVector( "PROBLEMS" ).addElement(typeName( ) + " can only connect a process with a rate and a measurable." ); // Improperly connected Direct Influence
					result = false;
				} // end if
			} else if ( typep(INDIRECTINFLUENCE) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && ( source.typep( QUANTITY ) || source.typep( PROCESSRATE ) ) && ( target.typep( QUANTITY ) || target.typep( PROCESSRATE ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement("Error #5"); // Improperly connected Indirect Influence
					getVector( "PROBLEMS" ).addElement( typeName( ) + " can only come from or go to measurables or processes with rates." ); // Improperly connected Indirect Influence
					result = false;
				} // end if
			} else if ( typep(TOUCHES) || typep( CONTAINS ) || typep( ISPARTOF ) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && 
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) )  && ( target.typep( ENTITY ) && ( ! target.typep( PROCESS ) ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement( "Error #6" ); // Improperly connected Spatial Constraints
					getVector( "PROBLEMS" ).addElement( typeName( ) + " must be connected between two things, multiple things, or substances." ); // Improperly connected Spatial Constraints
					result = false;
				} // end if
			} else if ( typep(DOES) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && 
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement( "Error #7" ); // Improperly connected Does
					getVector( "PROBLEMS" ).addElement( "The does arrow should start at a thing, multiple thing or substance." ); // Improperly connected Does
					result = false;
				} // end if
				check = ( source != null ) && ( target != null ) && ( source != target ) && 
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) )  && ( target.typep( PROCESS ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement("Error #8"); // Improperly connected Does
					getVector( "PROBLEMS" ).addElement( "Nothing can do a " + target.typeName( ) ); // Improperly connected Does
					result = false;
				} // end if
			} else if ( typep(HASATTRIBUTE) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && 
					( source.typep( ENTITY ) )  && ( target.typep( ATTRIBUTE ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement( "Error #9" ); // Improperly connected Has Attribute
					getVector( "PROBLEMS" ).addElement( "Has-Attribute can only connect a thing, multiple thing, substance, or process with a measurable." ); // Improperly connected Has Attribute
					result = false;
				} // end if
			} else if ( typep(HASPARTICIPANT) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && 
					( source.typep( PROCESS ) )  && ( target.typep( ENTITY ) && ( ! target.typep( PROCESS ) ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement( "Error #10" ); // Improperly connected Has Participant
					getVector( "PROBLEMS" ).addElement( "Has-Participant can only connect a process with a thing, multiple thing, or substance." ); // Improperly connected Has Participant
					result = false;
				} // end if
			} else if ( typep(PARTICIPATESIN) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && 
					( source.typep( ENTITY ) && ( ! source.typep( PROCESS ) ) )  && ( target.typep( PROCESS ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement( "Error #11" ); // Improperly connected Particpates In
					getVector( "PROBLEMS" ).addElement( "Participates-In can only connect a thing, multiple thing, or substance with a process." ); // Improperly connected Particpates In
					result = false;
				} // end if
			} else if ( typep(CONSTRAINT) ) {
				check = ( source != null ) && ( target != null ) && ( source != target ) && 
					( source.typep( QUANTITY ) )  && ( target.typep( QUANTITY ) );
				if ( ! check ) {
					getVector( "PROBLEMS" ).addElement( "Error #12" ); // Improperly connected Constraint
					getVector( "PROBLEMS" ).addElement("A comparer can only connect two measurables."); // Improperly connected Constraint
					result = false;
				} // end if
			} // end if
			} // end if
			} // end if
		} else if ( typep( QUANTITY ) || typep( PROCESSRATE ) ) {
			Vector direct = new Vector( );
			Vector indirect = new Vector( );
			String connectionID = null;
			Instance.InstInterface instance = null;
			for( Enumeration connections = causalInputs().elements( ); connections.hasMoreElements(); ) {
				connectionID = (String) connections.nextElement();
				if ( connectionID != null ) {
					instance = model( ).findInstance(connectionID);
					if ( instance != null ) {
						if ( instance.typep( DIRECTINFLUENCE ) ) direct.addElement( instance );
						if ( instance.typep( INDIRECTINFLUENCE ) ) indirect.addElement( instance );
					} // end if
				} // end if
			} // end for
			check = ( ! ( ( direct.size() > 0 ) && ( indirect.size() > 0 ) ) );
			if ( ! check ) {
				getVector( "PROBLEMS" ).addElement( "Error #13" ); // Direct and Indirect connected to same attribute
				getVector( "PROBLEMS" ).addElement( name( ) + " cannot have both an influences and a increases or decreases as input at the same time." ); // Direct and Indirect connected to same attribute
				result = false;
			} // end if
		} // end if
		return( result );
	} // method
	
} // class 

// ---------------------------------------------------------------------------
// End of File: Instance.java              (c) 2003 Northwestern University
