//package edu.qrg.vmodel.library.ontologies;

import CMLio.CMLable;
import Definition.DefInterface;
import Library.Category;
import Library.Element;
import KeyValuePair;
import KeyValuePair.KVPinterface;
import Library.LibInterface;
//import Ontology.Category;
//import Ontology.Element;
import Ontology.OntInterface;
import PList.PLinterface;
import XMLio.XMLable;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;

/** <PRE> 
* Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/library/ontologies
* ---------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ---------------------------------------------------------------------------
* File name: Category.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ---------------------------------------------------------------------------
*   Class: Category
* Purpose: Categories are used to group ontological elements. Categories
*          reside within an ontology. Categories may contain sub-categories.
* ---------------------------------------------------------------------------
* function is bridge to teleology
* teleology is purpose
* </PRE> */
public class QPlibrary extends Library
		implements Cloneable, Serializable, VModelConstants, Library.LibInterface {
	// Constructors
	// ------------------------------------------------------------------------
	public QPlibrary ( ) {
		super( );
		id( QPLIBRARY );
		name( "QP Ontology" );
		documentation( "The QP Ontology is..." );
		helpURL( makeHelpURL( QPLIBRARY ) );
		makeInitialElements( );
    } // default constructor
		
	// Properties
	// ------------------------------------------------------
	public static String QPLIBRARY = "QPLIBRARY";
	
	// Methods
	// ------------------------------------------------------------------------
	/** Construct a URL to a help file specified by base. */
	public String makeHelpURL( String base ) {
//		return( QRGutils.QRGroot( ) + HELPPATH + base + "." + HTML_SUFFIX );
		return( QRGutils.replaceExtension( base, HTML_SUFFIX ) );
	} // method

	/** Construct unique identifier for ontological element. */
	public void setID ( Ontology.Element e ) {
		e.id( "QP-" + e.type( ).toUpperCase( ) );
	} // method
	
	/** Generate ontological elements to be contained in the object. */
	public void makeInitialElements( ) {
		if ( ! BaseOntology.initialized ) new BaseOntology( );
		// Entities
		Library.Category entities = new Library(
			"QPLIBRARY-" + ENTITIES,
			"Entities",
			"Entities are...",
			makeHelpURL( ENTITIES ) );
		add( entities );

		Definition.DefInterface process = BaseOntology.process.subtype( "Process" );
		setID( process );
		entities.add( process );

		Definition.DefInterface entity = BaseOntology.entity.subtype( "Entity" );
		setID( entity );
		entities.add( entity );

		Definition.DefInterface collection = BaseOntology.collection.subtype( "Collection" );
		setID( collection );
		entities.add( collection );

		//Attributes
		Library.Category attributes = new Library(
			"QPLIBRARY-" + ATTRIBUTES,
			"Attributes",
			"Attributes are...",
			makeHelpURL( ATTRIBUTES ) );
		add( attributes );
			
		Definition.DefInterface attribute = BaseOntology.attribute.subtype( "Attribute" );
		setID( attribute );
		attributes.add( attribute );

		Definition.DefInterface quantity = BaseOntology.quantity.subtype( "Quantity" );
		setID( quantity );
		attributes.add( quantity );

		Definition.DefInterface rate = BaseOntology.rate.subtype( "Rate" );
		setID( rate );
		attributes.add( rate );

		Definition.DefInterface amount = BaseOntology.amount.subtype( "Amount" );
		setID( amount );
		attributes.add( amount );

		Definition.DefInterface level = BaseOntology.level.subtype( "Level" );
		setID( level );
		attributes.add( level );

		Definition.DefInterface magnitude = BaseOntology.magnitude.subtype( "Magnitude" );
		setID( magnitude );
		attributes.add( magnitude );

		Definition.DefInterface chance = BaseOntology.chance.subtype( "Probability" );
		setID( chance );
		attributes.add( chance );

		// Connectors
		Library.Category connectors = new Library(
			"QPLIBRARY-" + STRUCTURALRELATIONSHIPS,
			"Connectors",
			"Connectors are...",
			makeHelpURL( STRUCTURALRELATIONSHIPS ) );
		add( connectors );
			
		Definition.DefInterface hasAttribute = BaseOntology.hasAttribute.subtype( "Has Attribute" );
		setID( hasAttribute );
		connectors.add( hasAttribute );

		Definition.DefInterface hasParticipant = BaseOntology.hasParticipant.subtype( "Has Participant" );
		setID( hasParticipant );
		connectors.add( hasParticipant );

		// Causal
		Library.Category causalrelationships = new Library(
			"QPLIBRARY-" + CAUSALRELATIONSHIPS,
			"Causal Influences",
			"Causal Relationships are...",
			makeHelpURL( CAUSALRELATIONSHIPS ) );
		add( causalrelationships );
			
		Definition.DefInterface iPlus = BaseOntology.iPlus.subtype( "I+" );
		setID( iPlus );
		causalrelationships.add( iPlus );

		Definition.DefInterface iMinus = BaseOntology.iMinus.subtype( "I-" );
		setID( iMinus );
		causalrelationships.add( iMinus );

		Definition.DefInterface cPlus = BaseOntology.cPlus.subtype( "C+" );
		setID( cPlus );
		causalrelationships.add( cPlus );

		Definition.DefInterface cMinus = BaseOntology.cMinus.subtype( "C-" );
		setID( cMinus );
		causalrelationships.add( cMinus );

		Definition.DefInterface qpropPlus = BaseOntology.qpropPlus.subtype( "qprop+" );
		setID( qpropPlus );
		causalrelationships.add( qpropPlus );

		Definition.DefInterface qpropMinus = BaseOntology.qpropMinus.subtype( "qprop-" );
		setID( qpropMinus );
		causalrelationships.add( qpropMinus );

		// Constraint
		Library.Category constraintrelationships = new Library(
			"QPLIBRARY-" + CONSTRAINTRELATIONSHIPS,
			"Constraints",
			"Constraint Relationships are...",
			makeHelpURL( CONSTRAINTRELATIONSHIPS ) );
		add( constraintrelationships );
		
		Definition.DefInterface isGreaterThan = BaseOntology.isGreaterThan.subtype( "Greater Than" );
		setID( isGreaterThan );
		constraintrelationships.add( isGreaterThan );

		Definition.DefInterface isLessThan = BaseOntology.isLessThan.subtype( "Less Than" );
		setID( isLessThan );
		constraintrelationships.add( isLessThan );

		Definition.DefInterface isEqualTo = BaseOntology.isEqualTo.subtype( "Equals" );
		setID( isEqualTo );
		constraintrelationships.add( isEqualTo );

		Definition.DefInterface touches = BaseOntology.touches.subtype( "Touches" );
		setID( touches );
		constraintrelationships.add( touches );

		Definition.DefInterface contains = BaseOntology.contains.subtype( "Contains" );
		setID( contains );
		constraintrelationships.add( contains );

		Definition.DefInterface isPartOf = BaseOntology.isPartOf.subtype( "Part Of" );
		setID( isPartOf );
		constraintrelationships.add( isPartOf );

		// Justification
		Library.Category justification = new Library(
			"QPLIBRARY-" + JUSTIFICATION,
			"Justification",
			"Justification Elements are...",
			makeHelpURL( JUSTIFICATION ) );
		add( justification );
		
		Definition.DefInterface data = BaseOntology.data.subtype( "Data" );
		setID( data );
		justification.add( data );

		Definition.DefInterface supports = BaseOntology.supports.subtype( "Supports" );
		setID( supports );
		justification.add( supports );

		Definition.DefInterface hypothesis = BaseOntology.hypothesis.subtype( "Hypothesis" );
		setID( hypothesis );
		justification.add( hypothesis );

		Definition.DefInterface predicts = BaseOntology.predicts.subtype( "Predicts" );
		setID( predicts );
		justification.add( predicts );
	} // method
} // class

// ---------------------------------------------------------------------------
// End of File: QPlibrary.java                 (c) 2003 Northwestern University