//package edu.qrg.vmodel.library.ontologies;

import CMLio.CMLable;
import Definition;
import Library.Category;
import Library.Element;
import KeyValuePair;
import KeyValuePair.KVPinterface;
import Library.LibInterface;
//import Library.Category;
//import Library.Element;
import Ontology.OntInterface;
import PList.PLinterface;
import XMLio.XMLable;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;

/** <PRE> 
* Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/library/ontologies
* ---------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ---------------------------------------------------------------------------
* File name: Category.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ---------------------------------------------------------------------------
*   Class: Category
* Purpose: Categories are used to group ontological elements. Categories
*          reside within an ontology. Categories may contain sub-categories.
* ---------------------------------------------------------------------------
* function is bridge to teleology
* teleology is purpose
* </PRE> */
public class ClassroomLibrary extends Library
		implements Cloneable, Serializable, VModelConstants, Library.LibInterface {
	// Constructors
	// ------------------------------------------------------------------------
	public ClassroomLibrary ( ) {
		super( );
		id( CLASSROOM );
		name( "Building Blocks" );
		documentation( "Building Blocks are..." );
		helpURL( makeHelpURL( CLASSROOM ) );
		makeInitialElements( );
    } // default constructor
		
	// Properties
	// ------------------------------------------------------
	public static String CLASSROOM = "CLASSROOM";
	
	// Methods
	// ------------------------------------------------------------------------
	/** Construct a URL to a help file specified by base. */
	public String makeHelpURL( String base ) {
		return( QRGutils.replaceExtension( base, HTML_SUFFIX ) );
	} // method

	/** Construct unique identifier for ontological element. */
	public void setID ( Library.Element e ) {
		e.id( "QP-" + e.type( ).toUpperCase( ) );
	} // method
	
	/** Generate ontological elements to be contained in the object. */
	public void makeInitialElements( ) {
		if ( ! BaseOntology.initialized ) new BaseOntology( );
		// Entities
		Library.Category entities = new Library(
			"CLASSROOM-" + ENTITIES,
			"Basic Stuff",
			"Entities are...",
			makeHelpURL( ENTITIES ) );
		entities.icon("iconModel");
		entities.helpURL("help-building-blocks/basic-objs.htm");
		add( entities );

		Definition process = BaseOntology.processRate.subtype( "Process" );
		setID( process );
		process.typeName("Process");
		process.helpURL( "help-building-blocks/process.htm" );
		entities.add( process );

		Definition thing = BaseOntology.entity.subtype( "Thing" );
		setID( thing );
		thing.typeName("Thing");
		thing.helpURL( "help-building-blocks/thing.htm" );
		entities.add( thing );

		Definition multipleThing = BaseOntology.collection.subtype( "Multiple Thing" );
		setID( multipleThing );
		multipleThing.typeName("Multiple Thing");
		multipleThing.helpURL( "help-building-blocks/multiple-thing.htm" );
		entities.add( multipleThing );

		Definition substance = thing.subtype( "Substance" );
		setID( substance );
		substance.typeName("Substance");
		substance.helpURL( "help-building-blocks/substance.htm" );
		entities.add( substance );

		Definition measurable = BaseOntology.quantity.subtype( "Measurable" );
		setID( measurable );
		measurable.typeName("Measurable");
		entities.add( measurable );

		// Connectors
		Library.Category connectors = new Library(
			"CLASSROOM-" + STRUCTURALRELATIONSHIPS,
			"Connectors",
			"Connectors are...",
			makeHelpURL( STRUCTURALRELATIONSHIPS ) );
		connectors.icon("black-arrows.jpg");
		connectors.helpURL("help-building-blocks/hook-ups.htm");
		add( connectors );
			
		Definition does = BaseOntology.does.subtype( "Does" );
		setID( does );
		does.typeName("Does");
		does.helpURL( "help-building-blocks/does.htm" );
		connectors.add( does );

		Definition touches = BaseOntology.touches.subtype( "Touches" );
		setID( touches );
		touches.typeName("Touches");
		touches.helpURL( "help-building-blocks/touches.htm" );
		connectors.add( touches );

		Definition contains = BaseOntology.contains.subtype( "Contains" );
		setID( contains );
		contains.typeName("Contains");
		contains.helpURL( "help-building-blocks/contains.htm" );
		connectors.add( contains );

		Definition isPartOf = BaseOntology.isPartOf.subtype( "Part Of" );
		setID( isPartOf );
		isPartOf.typeName("Part Of");
		isPartOf.helpURL( "help-building-blocks/part-of.htm" );
		connectors.add( isPartOf );

		// Causal
		Library.Category causalrelationships = new Library(
			"CLASSROOM-" + CAUSALRELATIONSHIPS,
			"Causes",
			"Causes are...",
			makeHelpURL( CAUSALRELATIONSHIPS ) );
		causalrelationships.icon("IconCausal");
		causalrelationships.helpURL("help-building-blocks/causes.htm");
		add( causalrelationships );
			
		Definition increases = BaseOntology.iPlus.subtype( "Increases" );
		setID( increases );
		increases.typeName("Increases");
		increases.icon("IconCausal");
		increases.helpURL( "help-building-blocks/increases.htm" );
		causalrelationships.add( increases );

		Definition decreases = BaseOntology.iMinus.subtype( "Decreases" );
		setID( decreases );
		decreases.typeName("Decreases");
		decreases.icon("IconCausal");
		decreases.helpURL( "help-building-blocks/decreases.htm" );
		causalrelationships.add( decreases );

		Definition qpropPlus = BaseOntology.qpropPlus.subtype( "Influences" );
		setID( qpropPlus );
		qpropPlus.typeName("Influences");
		qpropPlus.icon("IconCausal");
		qpropPlus.helpURL( "help-building-blocks/influences.htm" );
		causalrelationships.add( qpropPlus );

		Definition qpropMinus = BaseOntology.qpropMinus.subtype( "Influences Opposite" );
		setID( qpropMinus );
		qpropMinus.typeName("Influences Opposite");
		qpropMinus.icon("IconCausal");
		qpropMinus.helpURL( "help-building-blocks/opposite-influences.htm" );
		causalrelationships.add( qpropMinus );

		// Constraint
		Library.Category constraintrelationships = new Library(
			"CLASSROOM-" + CONSTRAINTRELATIONSHIPS,
			"Comparisons",
			"Comparison Relationships are...",
			makeHelpURL( CONSTRAINTRELATIONSHIPS ) );
		constraintrelationships.icon("black-arrows.jpg");
		constraintrelationships.helpURL("help-building-blocks/number-comparers.htm");
		add( constraintrelationships );
		
		Definition isGreaterThan = BaseOntology.isGreaterThan.subtype( "Greater Than" );
		setID( isGreaterThan );
		isGreaterThan.typeName("Greater Than");
		isGreaterThan.helpURL( "help-building-blocks/greater-than.htm" );
		constraintrelationships.add( isGreaterThan );

		Definition isLessThan = BaseOntology.isLessThan.subtype( "Less Than" );
		setID( isLessThan );
		isLessThan.typeName("Less Than");
		isLessThan.helpURL( "help-building-blocks/less-than.htm" );
		constraintrelationships.add( isLessThan );

		Definition isEqualTo = BaseOntology.isEqualTo.subtype( "Equals" );
		setID( isEqualTo );
		isEqualTo.typeName("Equals");
		isEqualTo.helpURL( "help-building-blocks/equals.htm" );
		constraintrelationships.add( isEqualTo );
	} // method
} // class

// ---------------------------------------------------------------------------
// End of File: ClassroomLibrary.java                 (c) 2003 Northwestern University