package edu.qrg.vmodel;

import edu.qrg.utils.*;
import edu.qrg.utils.CMLio.CMLable;
import edu.qrg.vmodel.Definition;
import edu.qrg.utils.KeyValuePair.KVPinterface;
import edu.qrg.vmodel.Ontology.OntInterface;
import edu.qrg.utils.PList.PLinterface;
import edu.qrg.utils.XMLio.XMLable;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;

/** <PRE> 
* Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/library/ontologies
* ---------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ---------------------------------------------------------------------------
* File name: Category.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ---------------------------------------------------------------------------
*   Class: Category
* Purpose: Categories are used to group ontological elements. Categories
*          reside within an ontology. Categories may contain sub-categories.
* ---------------------------------------------------------------------------
* function is bridge to teleology
* teleology is purpose
* </PRE> */
public class ClassroomOntology extends Ontology
		implements Cloneable, Serializable, VModelConstants, Ontology.OntInterface {
	// Constructors
	// ------------------------------------------------------------------------
	/** General initialization. */
	public ClassroomOntology () {
		super();
		id(CLASSROOM);
		name("Building Blocks");
		documentation("Use these to make the model.");
		helpURL(makeHelpURL(CLASSROOM));
		makeInitialElements();
    } // default constructor
		
	// Properties
	// ------------------------------------------------------
	public static String CLASSROOM = "CLASSROOM";
	
	// InnerClasses
	// ------------------------------------------------------------------------
	/** <PRE> 
	* ---------------------------------------------------------------------------
	*   Class: Ontology
	* Purpose: Categories are used to group ontological elements. Categories
	*          reside within an ontology. Categories may contain sub-categories.
	* ---------------------------------------------------------------------------
	* </PRE> */
//	public class Ontology extends Ontology
//			implements Cloneable, Serializable, VModelConstants, Ontology.Ontology {
//		// Constructors
//		// -----------------------------------------------------------------------
//		/** Purpose: General initialization. */
//		public Ontology () {
//			super();
//	   } // constructor
//			
//		/** Purpose: Initializes most frequently used slots. */
//	   public Ontology(String id, String name, String documentation, String helpURL) {
//	   	this();
//	    	id(id);
//	    	name(name);
//	    	documentation(documentation);
//	    	helpURL(helpURL);
//	   } // constructor
//
//		// Properties & Accessors
//		// -----------------------------------------------------------------------
//		/** Purpose: Returns the string id of the category to which this
//		             category belongs. */
//		public String category () { return(getString(CATEGORY)); }
//		/** @see #category() */
//		public String category (String s) { return((String) set(CATEGORY, s)); }
//		
//		/** Purpose: Returns the string filename of an image representing
//		             the category. */
//		public String icon () { return(getString(ICON)); }
//		/** @see #icon() */
//		public String icon (String s) { return((String) set(ICON, s)); }
//		/** Purpose: Returns the string filename of an image representing
//		             the category in an Open or Close state. */
//		public String icon (boolean expanded) { 
//			String result = icon();
//			if (expanded) result += "Open";
//			else result += "Closed";
//			return(result); 
//		} // method
//		
//		/** Type of ontological element. */
//		public String type () { return(CATEGORY); }
//		/** @see #type() */
//		public String type (String s) { return(CATEGORY); }
//	
//		/** Type check */
//		public boolean typep (String typeID) {
//			boolean result = false;
//			Definition mumble = null;
//			result = id().equalsIgnoreCase(typeID) ||
//				name().equalsIgnoreCase(typeID) ||
//				category().equalsIgnoreCase(typeID) ||
//				type().equalsIgnoreCase(typeID);
//			return(result);
//		} // method
//	
//		/** Type of ontological element. */
//		public String typeName () { return("COREONTOLOGY"); }
//		/** @see #type() */
//		public String typeName (String s) { return("COREONTOLOGY"); }
//
//	} // class Ontology
	
	// Properties
	// ------------------------------------------------------
	public static Ontology entities = null;
	public static Definition entity = null;
	public static Definition process = null;
	public static Definition processRate = null;
	public static Definition agent = null;
	public static Definition collection = null;
	public static Definition collectionAmount = null;
	public static Definition modelFragment = null;
	public static Model model = null;
	
	public static Ontology attributes = null;
	public static Definition attribute =null;
	public static Ontology quantities = null;
	public static Definition quantity = null;
	public static Definition rate = null;
	public static Definition amount = null;
	public static Definition level = null;
	public static Definition magnitude = null;
	public static Definition chance = null;
	
	public static Ontology relationships = null;
	public static Definition relationship = null;
	public static Ontology structuralrelationships = null;
	public static Definition structure = null;
	public static Definition hasAttribute = null;
	public static Definition hasParticipant = null;
	public static Definition participatesIn = null;
	public static Definition does = null;
	public static Definition boundTo = null;
	public static Ontology causalrelationships = null;
	public static Definition causes = null;
	public static Definition directInfluence = null;
	public static Definition iPlus = null;
	public static Definition iMinus = null;
	public static Definition indirectInfluence = null;
	public static Definition qpropPlus = null;
	public static Definition qpropMinus = null;
	public static Definition cPlus = null;
	public static Definition cMinus = null;
	public static Ontology constraintrelationships = null;
	public static Definition constrains = null;
	public static Definition isGreaterThan = null;
	public static Definition isLessThan = null;
	public static Definition isEqualTo = null;
	public static Definition isTheSameAs = null;
	public static Definition isDifferentFrom = null;
	public static Ontology spatialrelationships = null;
	public static Definition touches = null;
	public static Definition contains = null;
	public static Definition isPartOf = null;
	
	public static Ontology justification = null;
	public static Definition data = null;
	public static Definition hypothesis = null;
	public static Definition supports = null;
	public static Definition predicts = null;
	
	public static boolean initialized = false;
	public static ClassroomOntology self = null;
	
	// Methods
	// ------------------------------------------------------------------------
	/** Construct a URL to a help file specified by base. */
	public String makeHelpURL(String base) {
//		return(QRGutils.QRGroot() + HELPPATH + base + "." + HTML_SUFFIX);
		return(QRGutils.replaceExtension(base, HTML_SUFFIX));
	} // method
	
	public void fillSlots (Definition definition, String name) {
		definition.id("CLASSROOM-" + name);
		definition.name(name);
		definition.documentation("A " + name + " is...");
		definition.helpURL(makeHelpURL(name));
		definition.type(name);
		definition.typeName(name);
	} // method
	
	public void fillSlots (Instance instance, String name) {
		instance.id("CLASSROOM-" + name);
		instance.name(name);
		instance.documentation("A " + name + " is...");
		instance.helpURL(makeHelpURL(name));
		instance.type(name);
		instance.typeName(name);
	} // method
	
	public Definition makeDefinition (Ontology category, String name) {
		Definition result = makeDefinition(category, null, name);
		return(result);
	} // method
	
	public Definition makeDefinition (Ontology category, Definition base, String name) {
		Definition result = null;
		Instance instance = null;
		if (base == null) result =  new Definition();
		else result = new Definition(base);
		fillSlots(result, name);
		result.removeAll();
		instance = new Instance(result);
		result.add(instance);
		if (category != null) category.add(result);
		else add(result);
		return(result);
	} // method
	
	/** Generate ontological elements to be contained in the object. */
	public void makeInitialElements() {
try {
		if (! BaseOntology.initialized) new BaseOntology();
		if (initialized) {

			add(entities);
				
			add(attributes);
				
			add(relationships);
				
			add(justification);
				
		} else {
			self = this;
			initialized = true;

			processRate =  makeDefinition(null, BaseOntology.processRate, "Process");
			processRate.documentation("Processes cause changes by increasing or decreasing parameters.");
			((Instance) processRate.connectors().elementAt(0)).documentation("Processes cause changes by increasing or decreasing parameters.");
//			processRate.CMLcommand("defprocessrate");
			processRate.helpURL("help-building-blocks/process.htm");
			((Instance) processRate.connectors().elementAt(0)).helpURL("help-building-blocks/process.htm");
			processRate.icon("processbox.gif");
			((Instance) processRate.connectors().elementAt(0)).icon("processbox.gif");
//			processRate.typeName("Process");

			// Categories
			entities = new Ontology(
			"CLASSROOM-" + ENTITIES,
			"Basic Stuff",
			"These are the model building blocks.",
			makeHelpURL(ENTITIES));
			entities.icon("iconModel");
			entities.helpURL("help-building-blocks/basic-objs.htm");
			add(entities);
				
			// Entities
			Instance instance = null;
			
			entity = makeDefinition(entities, BaseOntology.entity, "Thing");
			entity.documentation("A thing is something you can touch or think about.");
			((Instance) entity.connectors().elementAt(0)).documentation("A thing is something you can touch or think about.");
			entity.helpURL("help-building-blocks/thing.htm");
			((Instance) entity.connectors().elementAt(0)).helpURL("help-building-blocks/thing.htm");
				
			collection =  makeDefinition(entities, BaseOntology.collection, "MultipleThing");
			collection.typeName("Multiple Thing");
			((Instance) collection.connectors().elementAt(0)).typeName("Multiple Thing");
			collection.helpURL("help-building-blocks/multiple-thing.htm");
			((Instance) collection.connectors().elementAt(0)).helpURL("help-building-blocks/multiple-thing.htm");
			collection.documentation("A MultipleThing is a group of things that act the same way.");
			((Instance) collection.connectors().elementAt(0)).documentation("A MultipleThing is a group of things that act the same way.");
			
			Definition substance =  makeDefinition(entities, BaseOntology.entity, "Substance");
			substance.documentation("A Substance is something that doesn't have a definite shape.");
			((Instance) substance.connectors().elementAt(0)).documentation("A Substance is something that doesn't have a definite shape.");
			substance.helpURL("help-building-blocks/substance.htm");
			((Instance) substance.connectors().elementAt(0)).helpURL("help-building-blocks/substance.htm");
			
			// Agents are special entities.
			// They are assumed to be autonomous.
			// And have the ability to initiate processes w/o participation.
//			agent =  makeDefinition(entities, BaseOntology.agent, AGENT);


			attributes = new Ontology(
			"CLASSROOM-" + "PARAMETERS",
			"Parameters",
			"Parameters describe basic objects.  They can be increased or decreased by processes. ",
			makeHelpURL("parameters"));
			attributes.icon("iconAttribute");
			attributes.helpURL("help-building-blocks/parameters.htm");
			add(attributes);

			quantity =  makeDefinition(attributes, BaseOntology.quantity, "Parameter");
			quantity.documentation("A parameter can describe basic objects.  It can be increased or decreased by processes.");
			((Instance) quantity.connectors().elementAt(0)).documentation("A parameter can describe basic objects.  It can be increased or decreased by processes.");
			quantity.helpURL("help-building-blocks/parameter.htm");
			((Instance) quantity.connectors().elementAt(0)).helpURL("help-building-blocks/parameter.htm");
			
			amount =  makeDefinition(attributes, BaseOntology.amount, "Amount");
			amount.typeChain().addElement("Parameter");
			amount.typeNameChain().addElement("Parameter");
			((Instance) amount.connectors().elementAt(0)).typeChain().addElement("Parameter");
			((Instance) amount.connectors().elementAt(0)).typeNameChain().addElement("Parameter");
			amount.documentation("A Amount is how much or how many of something there is.");
			((Instance) amount.connectors().elementAt(0)).documentation("A Amount is how much or how many of something there is.");
			amount.helpURL("help-building-blocks/amount.htm");
			((Instance) amount.connectors().elementAt(0)).helpURL("help-building-blocks/amount.htm");

			level =  makeDefinition(attributes, BaseOntology.level, "Level");
			level.typeChain().addElement("Parameter");
			level.typeNameChain().addElement("Parameter");
			((Instance) level.connectors().elementAt(0)).typeChain().addElement("Parameter");
			((Instance) level.connectors().elementAt(0)).typeNameChain().addElement("Parameter");
			level.documentation("A Level is something that can go up or down.");
			((Instance) level.connectors().elementAt(0)).documentation("A Level is something that can go up or down.");
			level.helpURL("help-building-blocks/level.htm");
			((Instance) level.connectors().elementAt(0)).helpURL("help-building-blocks/level.htm");

			rate =  makeDefinition(attributes, BaseOntology.rate, "Rate");
			rate.typeChain().addElement("Parameter");
			rate.typeNameChain().addElement("Parameter");
			((Instance) rate.connectors().elementAt(0)).typeChain().addElement("Parameter");
			((Instance) rate.connectors().elementAt(0)).typeNameChain().addElement("Parameter");
			rate.documentation("A Rate is how fast or slow something happens.");
			((Instance) rate.connectors().elementAt(0)).documentation("A Rate is how fast or slow something happens.");
			rate.helpURL("help-building-blocks/rate.htm");
			((Instance) rate.connectors().elementAt(0)).helpURL("help-building-blocks/rate.htm");

//			magnitude =  makeDefinition(attributes, quantity, MAGNITUDE);

//			chance =  makeDefinition(attributes, quantity, CHANCE);
			

			structuralrelationships = new Ontology(
			"CLASSROOM-" + STRUCTURALRELATIONSHIPS,
			"Connectors",
			"These connect basic objects to each other.",
			makeHelpURL(STRUCTURALRELATIONSHIPS));
			structuralrelationships.icon("black-arrows.jpg");
			structuralrelationships.helpURL("help-building-blocks/connectors.htm");
			add(structuralrelationships);

//			hasAttribute =  makeDefinition(structuralrelationships, structure, HASATTRIBUTE);

//			hasParticipant =  makeDefinition(structuralrelationships, structure, HASPARTICIPANT);

			// Connects Agent to Process
			does =  makeDefinition(structuralrelationships, BaseOntology.does, "Does");
			does.helpURL("help-building-blocks/does.htm");
			((Instance) does.connections().elementAt(0)).helpURL("help-building-blocks/does.htm");
			does.documentation("Use does to connect a process with a basic object.");
			((Instance) does.connections().elementAt(0)).documentation("Use does to connect a process with a basic object.");
			does.icon("black-arrow.gif");
			((Instance) does.connections().elementAt(0)).icon("black-arrow.gif");

//			Definition requires =  makeDefinition(structuralrelationships, BaseOntology.requires, "Requires");

			touches =  makeDefinition(structuralrelationships, BaseOntology.touches, "Touches");
			touches.helpURL("help-building-blocks/touches.htm");
			((Instance) touches.connections().elementAt(0)).helpURL("help-building-blocks/touches.htm");
			touches.documentation("Use touches when entities are in contact with each other.");
			((Instance) touches.connections().elementAt(0)).documentation("Use touches when entities are in contact with each other.");
			touches.icon("black-arrow.gif");
			((Instance) touches.connections().elementAt(0)).icon("black-arrow.gif");

			contains =  makeDefinition(structuralrelationships, BaseOntology.contains, "Contains");
			contains.helpURL("help-building-blocks/contains.htm");
			((Instance) contains.connections().elementAt(0)).helpURL("help-building-blocks/contains.htm");
			contains.documentation("Use contains when one basic object is inside the other.");
			((Instance) contains.connections().elementAt(0)).documentation("Use contains when one basic object is inside the other.");
			contains.icon("black-arrow.gif");
			((Instance) contains.connections().elementAt(0)).icon("black-arrow.gif");

			isPartOf =  makeDefinition(structuralrelationships, BaseOntology.isPartOf, "isPartOf");
			isPartOf.typeName("Part Of");
			((Instance) isPartOf.connections().elementAt(0)).typeName("Part Of");
			isPartOf.helpURL("help-building-blocks/part-of.htm");
			((Instance) isPartOf.connections().elementAt(0)).helpURL("help-building-blocks/part-of.htm");
			isPartOf.documentation("One basic object can be a part of another.  Example, the engine is part of the car.");
			((Instance) isPartOf.connections().elementAt(0)).documentation("One basic object can be a part of another.  Example, the engine is part of the car.");
			isPartOf.icon("black-arrow.gif");
			((Instance) isPartOf.connections().elementAt(0)).icon("black-arrow.gif");
				
			causalrelationships = new Ontology(
				"CLASSROOM-" + CAUSALRELATIONSHIPS,
				"Causes",
				"These show how changes happen.",
				makeHelpURL(CAUSALRELATIONSHIPS));
			causalrelationships.icon("blue-arrows.gif");
			causalrelationships.helpURL("help-building-blocks/causes.htm");
			add(causalrelationships);
			
			// Integration
			iPlus =  makeDefinition(causalrelationships, BaseOntology.iPlus, "Increases");
			iPlus.icon("inc-arrow.gif");
			((Instance) iPlus.connections().elementAt(0)).icon("inc-arrow.gif");
			iPlus.helpURL("help-building-blocks/increases.htm");
			((Instance) iPlus.connections().elementAt(0)).helpURL("help-building-blocks/increases.htm");
			iPlus.documentation("This goes from a process to a parameter.");
			((Instance) iPlus.connections().elementAt(0)).documentation("This goes from a process to a parameter.");

			iMinus =  makeDefinition(causalrelationships, BaseOntology.iMinus, "Decreases");
			iMinus.icon("dec-arrow.gif");
			((Instance) iMinus.connections().elementAt(0)).icon("dec-arrow.gif");
			iMinus.helpURL("help-building-blocks/decreases.htm");
			((Instance) iMinus.connections().elementAt(0)).helpURL("help-building-blocks/decreases.htm");
			iMinus.documentation("This goes from a process to a parameter.");
			((Instance) iMinus.connections().elementAt(0)).documentation("This goes from a process to a parameter.");
			
			qpropPlus =  makeDefinition(causalrelationships,  BaseOntology.qpropPlus, "Influences");
			qpropPlus.icon("inf-arrow.gif");
			((Instance) qpropPlus.connections().elementAt(0)).icon("inf-arrow.gif");
			qpropPlus.helpURL("help-building-blocks/influences.htm");
			((Instance) qpropPlus.connections().elementAt(0)).helpURL("help-building-blocks/influences.htm");
			qpropPlus.documentation("When one parameter changes, it can influence another.  That means that when it goes up, the other one does too.");
			((Instance) qpropPlus.connections().elementAt(0)).documentation("When one parameter changes, it can influence another.  That means that when it goes up, the other one does too.");

			qpropMinus =  makeDefinition(causalrelationships,  BaseOntology.qpropMinus, "InfluencesOpposite");
			qpropMinus.typeName("Influences Opposite");
			((Instance) qpropMinus.connections().elementAt(0)).typeName("Influences Opposite");
			qpropMinus.icon("in-opp-arrow.gif");
			((Instance) qpropMinus.connections().elementAt(0)).icon("in-opp-arrow.gif");
			qpropMinus.helpURL("help-building-blocks/opposite-influences.htm");
			((Instance) qpropMinus.connections().elementAt(0)).helpURL("help-building-blocks/opposite-influences.htm");
			qpropMinus.documentation("When one parameter changes, it can influence another in an opposite direction.  That means that when it goes up, the other one goes down.");
			((Instance) qpropMinus.connections().elementAt(0)).documentation("When one parameter changes, it can influence another in an opposite direction.  That means that when it goes up, the other one goes down.");


			constraintrelationships = new Ontology(
				"CLASSROOM-" + CONSTRAINTRELATIONSHIPS,
				"Comparisons",
				"These compare the size of parameters.",
				makeHelpURL(CONSTRAINTRELATIONSHIPS));
			constraintrelationships.icon("blue-arrows.jpg");
			constraintrelationships.helpURL("help-building-blocks/number-comparers.htm");
			add(constraintrelationships);
				
			// Constraints
			isGreaterThan =  makeDefinition(constraintrelationships, BaseOntology.isGreaterThan, ISGREATERTHAN);
			isGreaterThan.name("Greater Than");
			((Instance) isGreaterThan.connections().elementAt(0)).name("Greater Than");
			isGreaterThan.typeName("Greater Than");
			((Instance) isGreaterThan.connections().elementAt(0)).typeName("Greater Than");
			isGreaterThan.helpURL("help-building-blocks/greater-than.htm");
			((Instance) isGreaterThan.connections().elementAt(0)).helpURL("help-building-blocks/greater-than.htm");
			isGreaterThan.documentation("Use this to compare parameters.");
			((Instance) isGreaterThan.connections().elementAt(0)).documentation("Use this to compare parameters.");
			isGreaterThan.icon("blue-arrow.gif");
			((Instance) isGreaterThan.connections().elementAt(0)).icon("blue-arrow.gif");

			isLessThan =  makeDefinition(constraintrelationships, BaseOntology.isLessThan, ISLESSTHAN);
			isLessThan.name("Less Than");
			((Instance) isLessThan.connections().elementAt(0)).name("Less Than");
			isLessThan.typeName("Less Than");
			((Instance) isLessThan.connections().elementAt(0)).typeName("Less Than");
			isLessThan.helpURL("help-building-blocks/less-than.htm");
			((Instance) isLessThan.connections().elementAt(0)).helpURL("help-building-blocks/less-than.htm");
			isLessThan.documentation("Use this to compare parameters.");
			((Instance) isLessThan.connections().elementAt(0)).documentation("Use this to compare parameters.");
			isLessThan.icon("blue-arrow.gif");
			((Instance) isLessThan.connections().elementAt(0)).icon("blue-arrow.gif");

			isEqualTo =  makeDefinition(constraintrelationships, BaseOntology.isEqualTo, "Equals");
			isEqualTo.helpURL("help-building-blocks/equals.htm");
			((Instance) isEqualTo.connections().elementAt(0)).helpURL("help-building-blocks/equals.htm");
			isEqualTo.documentation("Use this to compare parameters.");
			((Instance) isEqualTo.connections().elementAt(0)).documentation("Use this to compare parameters.");
			isEqualTo.icon("blue-arrow.gif");
			((Instance) isEqualTo.connections().elementAt(0)).icon("blue-arrow.gif");

		} // end if
} catch ( Exception e ) { QRGutils.debugStackTrace( e ); }
	} // method
} // class

// ---------------------------------------------------------------------------
// End of File: ClassroomLibrary.java                 (c) 2003 Northwestern University