//package edu.qrg.vmodel.library.references;

import Definition;
import PList.PLinterface;
import XMLio.XMLable;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Math;
import java.lang.Object;
import java.lang.String;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/current development/code/library/references
* ---------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ---------------------------------------------------------------------------
* File name: Reference.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ---------------------------------------------------------------------------
*   Class: Reference
* Purpose: A collection of pointers into the ontological libraries.
* ---------------------------------------------------------------------------
* Notes:
* * cooking terms poster
* ---------------------------------------------------------------------------
* </PRE>
* @see PList
* @see KeyValuePair
*/
public class Reference extends PList 
		implements Cloneable, Serializable, XMLio.XMLable, Reference.RefInterface {
	// Constructors
	// -----------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: General Intialization.
	*          A unique identifier is provided for each reference.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public Reference() {
	   super( );
		id( composeUniqueIdentifier( ) );
	} // default constructor 
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Intialization. Duplicate base reference values.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public Reference( Reference base ) {
		this( );
		copyKeyValuePairs( base );
		id( composeUniqueIdentifier( ) );
//		reference( new String( base.reference( ) ) );
//		Otype( base.Otype( ) );
		ancestor( base );
		base.descendent( this );
//		owner( base.owner( ) );
		definition( base.definition( ) );
//		input( base.input( ) );
//		output( base.output( ) );
//		selfconnect( base.selfconnect( ) );
	} // constructor 
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Intialization. Sets the Element being referenced.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public Reference( Definition definition ) {
		this( );
	   reference( definition.id( ) );
	   Otype( definition.Otype( ) );
	   definition( definition );
	} // constructor 
	
	// Interface
	// -----------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Interface: Reference.RefInterface
	* Purpose: In general, anything implementing this interface should be
	*          acceptable as a Reference.
	* ------------------------------------------------------------------------
	* </PRE> 
	* @see PList
	* @see XMLio
	*/
	public interface RefInterface 
			extends PList.PLinterface, VModelConstants, XMLio.XMLable {
		// Properties & Accessors
		// --------------------------------------------------------------------
		/** A unique identifier. */
		public String id ( );
		/** @see #id( ) */
		public String id ( String s );
		
		/** The id of definition pointed to by the object. */
		public String reference ( );
		/** @see #reference( ) */
		public String reference ( String s );
		
		/** The definition pointed to by the object. */
		public Definition definition ( );
		/** @see #definition( ) */
		public Definition definition ( Definition d );
		
		/** Otype of referenced definition. */
		public String Otype ( );
		/** @see #Otype( ) */
		public String Otype ( String s );
		
		/** ID of definition that contains the reference. */
		public String owner ( );
		/** @see #owner( ) */
		public String owner ( String s );
		
		/** Base reference from which fields were duplicated. */
		public Reference.RefInterface ancestor ( );
		/** Target reference that is copying fields. */
		public Reference.RefInterface descendent ( );
		
		public Reference input ( );
		public Reference input ( Reference r );
		
		public Reference output ( );
		public Reference output ( Reference r );
		
		public boolean selfconnect ( );
		public boolean selfconnect ( boolean b );
		
	} // interface
	
	// Properties & Accessors
	// -----------------------------------------------------------------------
		public Reference input ( ) { return( (Reference) get( INPUT ) ); }
		public Reference input ( Reference r ) { return( (Reference) set( INPUT, r ) ); }
		
		public Reference output ( ) { return( (Reference) get( OUTPUT ) ); }
		public Reference output ( Reference r ) { return( (Reference) set( OUTPUT, r ) ); }
		
		public boolean selfconnect ( ) { return( getBoolean( "SELFCONNECT" ) ); }
		public boolean selfconnect ( boolean b ) { 
			set( "SELFCONNECT", b ); 
			return( b ); 
		} // method

	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: String identifier for object.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String id ( ) { return( getString( ID ) ); }
	/** @see #id( ) */
	public String id ( String s ) { return( (String) set( ID, s ) ); }
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: ID of Definition being referenced.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String reference ( ) { return( getString( REFERENCE ) ); }
	/** @see #reference( ) */
	public String reference ( String s ) { return( (String) set( REFERENCE, s ) ); }
	
	// Hmmmm.... Would this violate the uniqueness of the definitions?
	// Better think carefully before using.
	/** The definition pointed to by the Reference. 
	    @see #definition( )
	    @serial
	    */
	protected Definition definition = null;
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: The definition pointed to by the reference.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public Definition definition ( ) { return( definition ); }
	/** @see #definition( ) */
	public Definition definition ( Definition d ) {
		return( definition = d );
	} // method
		
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Otype of Definition being referenced.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String Otype ( ) { return( getString( QPTYPE ) ); }
	/** @see #Otype( ) */
	public String Otype ( String s ) { return( (String) set( QPTYPE, s ) ); }
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: ID of Definition containing the reference.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String owner ( ) { return( getString( OWNER ) ); }
	/** @see #owner( ) */
	public String owner ( String s ) { return( (String) set( OWNER, s ) ); }
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Base Reference from which fields were copied.
	* Notes:
	* * Do not use directly - Use the accessors.
	* ------------------------------------------------------------------------
	* </PRE> 
	* @see #ancestor( )
	* @see #ancestor( Reference.RefInterface )
	*/
	protected transient Reference.RefInterface ancestor = null;
	/** @see #ancestor */
	public Reference.RefInterface ancestor ( ) { return( ancestor ); }
	/** @see #ancestor */
	protected Reference.RefInterface ancestor ( Reference.RefInterface r ) {
		return( ancestor = r );
	} // writer
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Target Reference to which fields are being copied.
	* Notes:
	* * Do not use directly - Use the accessors.
	* ------------------------------------------------------------------------
	* </PRE> 
	* @see #descendent( )
	* @see #descendent( Reference.RefInterface )
	*/
	protected transient Reference.RefInterface descendent = null;
	/** @see #descendent */
	public Reference.RefInterface descendent ( ) { return( descendent ); }
	/** @see #descendent */
	protected Reference.RefInterface descendent ( Reference.RefInterface r ) {
		return( descendent = r );
	} // writer
	
	// Methods
	// ------------------------------------------------------------------------
	// Stringification
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Stringifies the reference.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String toString ( ) { 
		String result =  "(" + id( ) + " :type " + reference( ) + ")";
		if ( Otype( ).equalsIgnoreCase( RELATIONSHIP ) ) { 
			result = "(" + reference( ) + " " + printIO( INPUT ) + " " + printIO( OUTPUT ) + ")";
		} // end if 
//		QRGutils.debug( "Reference", "toString", "Otype=" + Otype( ) );
		return( result ); 
	} // method toString
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Prints a property.
	* Notes:
	* * This is clunky.  We need a way to print for variables.  That means
	*   that in addition to printing the input and output properties, we need
	*   to print the variable name.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String printIO ( String var ) {
		String result = getString( var );
		// Need to find accessor path.
		// How?
		return( result );
	} // end method
	
	// Uniquify
	/** Counter of the number of reference objects created in session. */
	protected static long sessionCount = 0;

	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Generates string for id uniquification.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String composeUniqueIdentifier ( ) {
		String result = getClass( ).getName( ); 
		result += "-" + ( sessionCount++ );
		result += "-" + ( new Date( ) ).getTime( ); 
		result += "-" + hashCode( );
		result += "-" + String.valueOf( (char) ( ( Math.random( ) * 25 ) + 65 ) );
		return( result );
	} // method 
	
	// XMLio.
	// -----------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Stringify the an XML definition for the object.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String xmlDef ( ) { 
		String result = 	"";
		KeyValuePair mumble = null;
		String classname = this.getClass( ).getName( );
		result += "<!DOCTYPE " + classname + " [\r\n";
		result += "<!ELEMENT " + classname + " EMPTY >\r\n";
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			mumble = (KeyValuePair) e.nextElement();
			result += "<!ATTLIST " + classname + " " + mumble.key( ).toUpperCase( ) + " CDATA #REQUIRED >\r\n";
		} // end for
		result += "]>";
		return( result ); 
	} // method
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Stringify the object into the XML format.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public String xmlify ( ) {
		String result = "";
		KeyValuePair pair = null;
		Object mumble = null;
		final VectorElement grumble = new VectorElement( );
		final ObjectElement bumble = new ObjectElement( );
		Vector fumble = new Vector( );
		boolean flag = true;
		int indent = 0;
		result = QRGutils.space( indent ) + "<" + this.getClass( ).getName( );
		indent = result.length( ) + 1;
//		result += stringifyKeyValuePairs( result.length( ) + 1 );
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			pair = (KeyValuePair) e.nextElement( );
			if ( ( ( pair.value( ) instanceof Vector ) &&
					( ! ( (Vector) pair.value( ) ).isEmpty( ) ) ) ||
					( pair.value( ) instanceof XMLio.XMLable ) ) {
				fumble.addElement( pair );
			} else {
				result += QRGutils.eol( ) + QRGutils.space(indent);
				result += pair;
			} // end if
		} // end for
		int tabs = indent + 3;
		if ( fumble.isEmpty( ) ) result += " />" + QRGutils.eol( );
		else {
			result += " >" + QRGutils.eol( );
			for (Enumeration e = fumble.elements( ) ; e.hasMoreElements() ;) {
				pair = (KeyValuePair) e.nextElement( );
				if ( pair.value( ) instanceof Vector ) {
					for (Enumeration p = ((Vector) pair.value( )).elements( ) ; p.hasMoreElements() ;) {
						mumble = p.nextElement( );
						grumble.set( TAG, pair.key( ) );
						grumble.set( VALUE, mumble );
						result += QRGutils.space( tabs ) + grumble.xmlify( );
					} // end for
				} else {
					bumble.set( TAG, pair.key( ) );
					bumble.set( VALUE, pair.value( ) );
					result += QRGutils.space( tabs ) + bumble.xmlify( );
				} // end if
			} // end for
			result += QRGutils.space( indent ) + "</" + this.getClass( ).getName( ) + ">" + QRGutils.eol( );
		} // end if 
//		result += QRGutils.eol( );
		return( result );
	}
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Saves the ontology to filename in XML format.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public void exportXML ( String filename ) {
		XMLio.save(  QRGutils.QRGuserpath( ) + filename + "." + XML_SUFFIX , this );
	} // method
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Initializes slots after object is created by XML loader.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public void XMLioObjectInit ( Vector attr, Vector elmnt ) {
		id( composeUniqueIdentifier( ) );
		keyValuePairs( attr );
		Object mumble = null;
		VectorElement grumble = null;
		ObjectElement fumble = null;
		for( Enumeration enum = elmnt.elements( ); enum.hasMoreElements( ); ) {
			mumble = enum.nextElement( );
			if ( mumble instanceof VectorElement ) {
				grumble = (VectorElement) mumble;
				getVector( grumble.getString( TAG ) ).addElement( grumble.get( VALUE ) );
			} else if ( mumble instanceof ObjectElement ) {
				fumble = (ObjectElement) mumble;
				set( fumble.getString( TAG ), fumble.get( VALUE ) );
			} // end if
		} // end for
	
	} // method
	
} // class Reference

// ---------------------------------------------------------------------------
// End of File: Reference.java             2003 Northwestern University