package edu.qrg.utils;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.PrintWriter;
import java.lang.Cloneable;
import java.lang.Exception;
import java.lang.Integer;
import java.lang.NumberFormatException;
import java.lang.Object;
import java.lang.String;
import java.lang.System;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/utils
* ---------------------------------------------------------------------------
* 2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ---------------------------------------------------------------------------
* File name: QRGutils.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ---------------------------------------------------------------------------
*   Class: QRGutils
* Purpose: This is a set of common utilities that keep popping up. This class
*          is comprised of fuctions meant to be called statically.
* ---------------------------------------------------------------------------
* </PRE>
*/
public final class QRGutils implements Cloneable, QRGconstants {
	// Constants
	// -----------------------------------------------------------------------
	// Project Info - Hmmm.... Do these belong here?  Are the necessary?
	/** Project name. */
	public static final String PROJECT = "QRG";
	/** Project version. */
	public static final double VERSION = 1.0;
	/** Year of copyright. */
	public static final int COPYRIGHT = 2003;
	/** Copyright attribution. */
	public static final String COPYRIGHTATTRIBUTION = "Northwestern University, Kenneth D. Forbus";

	// Properties & Accessors
	// -----------------------------------------------------------------------
	// Project Info (Name, Version, Attribution, etc....)
	/** * Do not access directly. USE THE ACCESSORS.
	* @see #projectName()
	* @see #projectName(String)
	*/
	protected static String projectName = PROJECT;
	/** @see #projectName */
	public static String projectName () { return(projectName); }
	/** @see #projectName */
	public static String projectName (String s) {
		return(projectName = s);
	} // writer

	/** * Do not access directly. USE THE ACCESSORS.
	* @see #projectVersion()
	* @see #projectVersion(double)
	*/
	protected static double projectVersion = 1.0;
	/** @see #projectVersion */
	public static double projectVersion () { return(projectVersion); }
	/** @see #projectVersion */
	public static double projectVersion (double d) {
		return(projectVersion = d);
	} // writer

	/** * Do not access directly. USE THE ACCESSORS.
	* @see #projectCopyright()
	* @see #projectCopyright(int)
	*/
	protected static int projectCopyright = 2003;
	/** @see #projectCopyright */
	public static int projectCopyright () { return(projectCopyright); }
	/** @see #projectCopyright */
	public static int projectCopyright (int i) {
		return(projectCopyright = i);
	} // writer

	/** * Do not access directly. USE THE ACCESSORS.
	* @see #projectCopyrightAttribution()
	* @see #projectCopyrightAttribution(String)
	*/
	protected static String projectCopyrightAttribution = "Northwestern University, Kenneth D. Forbus";
	/** @see #projectCopyrightAttribution */
	public static String projectCopyrightAttribution () {
		return(projectCopyrightAttribution);
	} // reader
	/** @see #projectCopyrightAttribution */
	public static String projectCopyrightAttribution (String s) {
		return(projectCopyrightAttribution = s);
	} // writer

	/** Vector of names of the authors of the project / implemented system.
	* * Do not access directly. USE THE ACCESSORS.
	* @see #projectAuthors()
	* @see #projectAuthors(Vector)
	*/
	protected static Vector projectAuthors = null;
	/** @see #projectAuthors */
	public static Vector projectAuthors () {
		if (projectAuthors == null) projectAuthors = new Vector();
		return(projectAuthors);
	} // reader
	/** @see #projectAuthors */
	public static Vector projectAuthors (Vector v) {
		return(projectAuthors = v);
	} // writer

	// FilePath Abstraction Properties
	/** String representing the root path applied to path construction.
	* * Do not access directly. USE THE ACCESSORS.
	* @see #QRGroot()
	* @see #QRGroot(String)
	*/
	protected static String QRGroot = "/QRG/";
	/** @see #QRGroot */
	public static String QRGroot () {
		if ((QRGroot == null) || (QRGroot.equalsIgnoreCase("null")) ||
		(QRGroot.equalsIgnoreCase("null/")) // || (QRGroot.equalsIgnoreCase("/"))
		) QRGroot = "";
		return(QRGroot);
	} // reader
	/** @see #QRGroot */
	public static String QRGroot (String p) {
		String result = p;
		if ((result == null) || (result.equalsIgnoreCase("null")) || (result.equalsIgnoreCase("null/"))) result = "";
		return(QRGroot = result);
	} // writer

	/** String representing the user path applied to path construction.
	* * Do not access directly. USE THE ACCESSORS.
	* @see #QRGuserpath()
	* @see #QRGuserpath(String)
	*/
	protected static String QRGuserpath = System.getProperty("user.home");
	/** @see #QRGuserpath */
	public static String QRGuserpath () {
		if (QRGuserpath == null) { QRGuserpath = QRGroot(); };
		return(QRGuserpath);
	} // reader
	/** @see #QRGuserpath */
	public static String QRGuserpath (String p) {
		File foo = new File(p);
		foo.mkdirs();
		return(QRGuserpath = p);
	} // writer

	/** Character used to separate path elements.
	* * Do not access directly. USE THE ACCESSORS.
	* @see #QRGpathseparator()
	* @see #QRGpathseparator(String)
	*/
	protected static String QRGpathseparator = null;
	/** @see #QRGpathseparator */
	public static String QRGpathseparator () { 
		if ( QRGpathseparator == null ) {
			switch ( detectPlatform() ) {
//				case MACINTOSH: QRGpathseparator = ":";
				case MACINTOSH: QRGpathseparator = "/";
					break;
				case WINDOWS: QRGpathseparator = "\\";
					break;
				case UNIX: QRGpathseparator = "/";
					break;
				default: QRGpathseparator = "\\";
			} // end switch
		} // end if
		return(QRGpathseparator); 
	} // method
	/** @see #QRGpathseparator */
	public static String QRGpathseparator (String s) {
		return(QRGpathseparator = s);
	} // writer

	// Applet Properties, Accessors, & Methods
	/** A safe place to stash the currently executing applet.
	* * Do not access directly. USE THE ACCESSORS.
	* @see #applet()
	* @see #applet(Applet)
	*/
	protected static Applet applet = null;
	/** @see #applet */
	public static Applet applet () { return(applet); }
	/** @see #applet */
	public static Applet applet (Applet a) { return(applet = a); }

	/** Has the applet property been set?
	* @see #applet()
	* @see #applet(Applet)
	* @see #isApplet(Object)
	*/
	public static boolean isApplet () { return(applet() != null); }
	/** Determines if arg is an applet and sets the applet property.
	* @see #applet()
	* @see #applet(Applet)
	* @see #isApplet()
	*/
	public static boolean isApplet (Applet a) {
		applet(a);
		return(isApplet());
	} // method
	//** @see #isApplet(Applet) */
	public static boolean isApplet (Object o) {
		applet(null);
		return(isApplet());
	} // method

	// System/GUI Properties
	/**  QRG default toolkit.  Defaults to system default toolkit.
	* * Do not access directly. USE THE ACCESSORS.
	* * Is this really more efficient/saves that much typing?
	* @see #toolkit()
	* @see #toolkit(Toolkit)
	*/
	protected static Toolkit toolkit = null;
	/** @see #toolkit */
	public static Toolkit toolkit () {
		return((toolkit == null) ? toolkit(Toolkit.getDefaultToolkit()) : toolkit);
	} // reader
	/** @see #toolkit */
	public static Toolkit toolkit (Toolkit t) { return(toolkit =t); }
	
	public static Vector mergeVector ( Vector a, Vector b ) {
		Vector result = new Vector( );
		for( Enumeration items = a.elements( ); items.hasMoreElements(); )
			result.addElement( items.nextElement( ) );
		for( Enumeration items = b.elements( ); items.hasMoreElements(); )
			result.addElement( items.nextElement( ) );
		return( result );
	} // method

	/** Stash for screen dimensions.
	* * Do not access directly. USE THE ACCESSORS.
	* * Is the toolkit call expensive enough to merit this?
	* @see #screenDimensions()
	* @see #screenDimensions(Dimension)
	*/
	protected static Dimension screenDimensions = null;
	/** @see #screenDimensions
	*/
	public static Dimension screenDimensions () {
		if(screenDimensions == null)
			screenDimensions = toolkit().getScreenSize();
		return(screenDimensions);
	} // reader
	/** @see #screenDimensions */
	public static Dimension screenDimensions (Dimension s) {
		return(screenDimensions = s);
	} // writer

	// DateTime Properties
	/** Standard simple date format.
	* * Do not access directly. USE THE ACCESSORS.
	* * Should this be a constant?
	* @see #dateformat()
	* @see #dateformat(SimpleDateFormat)
	*/
	protected static SimpleDateFormat dateformat = null;
	/** Reader for dateformat property.
	* * Defaults to "MM.dd.yyyy hh:mm:ss ".
	* @see #dateformat
	*/
	public static SimpleDateFormat dateformat () {
		if (dateformat == null)
			dateformat(new SimpleDateFormat ("MM.dd.yyyy hh:mm:ss "));
		return(dateformat);
	} // reader
	/** @see #dateformat */
	public static SimpleDateFormat dateformat (SimpleDateFormat d) {
		return(dateformat = d);
	} // writer

	// Detect Platform
	// -----------------------------------------------------------------------
	/** @see #detectPlatform() */
	protected static int detectedPlatform = PLATFORM_UNKNOWN;
	/** Detects the current hardware platform.
	* Default: PLATFORM_UNKNOWN
	* * Is system call that expensive?
	* @see QRGconstants#PLATFORM_UNKNOWN
	* @see QRGconstants#MACINTOSH
	* @see QRGconstants#WINDOWS
	* @see QRGconstants#UNIX
	* @see #detectedPlatform
	*/
	public static int detectPlatform () {
		int result = detectedPlatform;
		String os = "";
		if (detectedPlatform < 1) {
			os = (System.getProperty("os.name")).toUpperCase();
			if (os.startsWith("MAC")) result = MACINTOSH;
			else if (os.startsWith("WIN")) result = WINDOWS;
			else if (os.startsWith("UNIX")) result = UNIX;
			detectedPlatform = result;
		} // end if
		return(result);
	} // reader

	// Platform End-Of-Line
	// -----------------------------------------------------------------------
	/** Returns the end-of-line characters for the detected platform.
	* @see QRGconstants#EOLS
	* @see #detectPlatform
	*/
	public static String eol () {
		String result = EOLS[ 0 ];
		try { result =  EOLS[ detectPlatform() ]; }
		catch (Exception e) { }
		return(result);
	} // method

	// Mouse Methods
	// -----------------------------------------------------------------------
	/** Purpose: Returns true if the mouse event indicates a right-click. */
	public static boolean isRightClick(MouseEvent evt) {
		boolean result = (((evt.getModifiers() & InputEvent.BUTTON3_MASK) != 0) ||
							   ((evt.getModifiers() & InputEvent.BUTTON2_MASK) != 0));
		return(result);
	} // method

	// Set and return the QRG root path.
	// -----------------------------------------------------------------------
	/** Concatenates a path from QRGroot + QRGseparator + arg.
	* @see #QRGroot()
	* @see #QRGpathseparator()
	*/
	public static String makeQRGpath (String pathstring) {
		String result = QRGroot() + pathstring;
		return(result + QRGpathseparator());
	} // method

	/** @see #makeQRGpath(String) */
	public static String makeQRGpath (String[] pathstrings) {
		String result = QRGroot();
		for(int i = 0; i < pathstrings.length; i++)
			result += pathstrings[i] + QRGpathseparator();
		return(result);
	} // method

	// Set and return the QRG user path.
	// -----------------------------------------------------------------------
	/** Initialize the QRG user root path.
	* Default: System.getProperty("user.home")
	* * On Windows systems, initializes to "My Documents/QRG" folder.
	* * if System.getProperty("user.home") is undefined, uses QRGroot().
	* @see #QRGroot()
	* @see #QRGpathseparator()
	* @see #QRGuserpath()
	*/
	public static String initUserPath () {
		String result = System.getProperty("user.home");
		if (result == null) result = QRGroot();
		else {
			if (detectPlatform() == WINDOWS)
				result += QRGpathseparator() + "My Documents";
			result += QRGpathseparator() + "QRG";
		} // end if
		QRGuserpath(result + QRGpathseparator());
		File foo = new File(QRGuserpath());
		foo.mkdirs();
		return(QRGuserpath());
	} // method makeUserPath

	/** Concatenates a path from QRGuserpath + QRGseparator + arg.
	* @see #QRGuserpath()
	* @see #QRGpathseparator()
	*/
	public static String makeUserPath (String pathstring) {
		String result = QRGuserpath() + pathstring;
		return(result + QRGpathseparator());
	} // method makeUserPath

	/** @see #QRGuserpath() */
	public static String makeUserPath (String[] pathstrings) {
		String result = QRGuserpath();
		for(int i = 0; i < pathstrings.length; i++)
			result += pathstrings[i] + QRGpathseparator();
		return(result);
	} // method makeUserPath

	// Add or replace the file extension in a filename.
	// -----------------------------------------------------------------------
	/** If needed, appends extension to the end of filename. */
	public static String addExtension(String filename, String ext) {
		String result = filename;
		if (! filename.endsWith(ext)) result = filename + "." + ext;
		return(result);
	} // method

	/** Replaces or appends extension to the end of filename as needed. */
	public static String replaceExtension(String filename, String ext) {
		String result = filename;
		int index = -1;
		if (! filename.endsWith(ext)) {
			index = filename.lastIndexOf('.');
			if (index == -1) {
				// no '.' in name
				result = filename + "." + ext;
			} else {
				// there is a '.' but the extension is wrong
				result = filename.substring(0, index) + "." + ext;
			} // end if else
		} // end if
		return(result);
	} // method

	/** Removes extension from the end of filename.
	* * Assumes single dot in filename.
	* * Dot separates filename and extension.
	*/
	public static String stripExtension(String filename) {
		int index = filename.lastIndexOf('.');
		String result = "";
		if (index == -1) result = filename;
		else result = filename.substring(0, index);
		return(result);
	} // method

	// Added (TRH)
	/** Returns a class name without package modifiers **/
	public static String rawClassName(String className) {
		int index = className.lastIndexOf(".");
		String result = className;
		if (index > -1)
			result = className.substring(index + 1);
		return(result);
	} // method

	// Method: space
	/** Returns a String of spaces that is length i. */
	public static String space (int i) {
		String result = "";
		for(int j = 0; j < i; j++)
			result += " ";
		return(result);
	} // method

	// Method: isWhiteSpace
	/** Returns true if char is whitespace.
	* * Maybe we should make this more flexible. Perhaps working from an
	*   array of acceptible whitespace characters.
	*/
	public static boolean isWhiteSpace (int c) {
		return((c == 0x20) || (c == 0xA0) || (c == 0x0D) ||
				 (c == 0x8D) || (c == 0x09) || (c == 0x0A));
	} // method

	// Method: gobbleWhiteSpace
	/** gobbles whitespace in a stream. */
	public static int gobbleWhiteSpace (InputStream inp) {
		int c = 0;
		try {c = inp.read();}
		catch (Exception e) {c = -1;}
		while((c != -1) && isWhiteSpace(c)) {
			try {c = inp.read();}
			catch (Exception e) {c = -1;}
		} // end while
		return(c);
	} // method

	// Method: replaceStrings
	/** Replaces all occurrances of specified substrings in a String. */
	public static String replaceSubstring (String string, String oldS, String newS) {
		String workspace = string;
		int pos = -1;
		int l = 0;
		pos = -1;
		l = oldS.length();
		try {pos = workspace.indexOf(oldS);}
		catch(Exception e) { }
		while(pos != -1) {
			workspace = workspace.substring(0,pos) + newS +
							workspace.substring(pos + l);
			pos = -1;
			try {pos = workspace.indexOf(oldS);}
			catch(Exception e) { }
		} // end while
		return(workspace);
	} // method

	/** @see #replaceSubstring */
	public static String replaceSubstrings (String string, String [] oldS,
		String [] newS) {
		String workspace = string;
		for(int i = 0; i < oldS.length; i++)
			workspace = replaceSubstring(workspace, oldS[i], newS[i]);
		return(workspace);
	} // method

	// Replace characters in a string.
	// -----------------------------------------------------------------------
	/** Replace characters in a string.
	* * Hmmm.... Flesh this out more: extend and add flexibility.
	*/
	public static String replaceIllegalFileSystemChars (String string) {
		String result = replaceSubstring(string, " ", "-");
		result = replaceSubstring(result, "\r", "");
		result = replaceSubstring(result, "\n", "");
		result = replaceSubstring(result, "\"", "");
		result = replaceSubstring(result, "&", "AND");
		result = replaceSubstring(result, "<", "-");
		result = replaceSubstring(result, ">", "-");
		result = replaceSubstring(result, ".", "-");
		return(result);
	} // method

	/** Replace spaces with dashes. */
	public static String replaceSpaces (String string) {
		return(replaceSubstring(string, " ", "-"));
	} // method

	/** Replace spaces with replacement character. */
	public static String replaceSpaces (String string, String replacement) {
		return(replaceSubstring(string, " ", replacement));
	} // method

	/** Replace escape sequences with characters. */
	public static String decodeEscapeChars (String thing) {
		String[] oldS = {"\\r", "\\n", "&nbsp;", "&quot;", "&amp;"};
		String[] newS = {"\r", "\n", " ", "\"", "&"};
		return(replaceSubstrings(thing, oldS, newS));
	} // method

	/** @deprecated Use decodeEscapeChars(String). */
	public static String replaceEscapeChars (String thing) {
		return(decodeEscapeChars(thing));
	} // method

	/** Replace characters with escape sequences. */
	public static String encodeEscapeChars (String thing) {
		String[] newS = {"\\r", "\\n", "&nbsp;", "&quot;", "&amp;"};
		String[] oldS = {"\r", "\n", " ", "\"", "&"};
		return(replaceSubstrings(thing, oldS, newS));
	} // method

	// Method: stringify
	/** Generates a String representation for an object. */
	public static String stringify (String tag, Object thing,
		int indent, boolean r) {
		String result = "";
		result += space(indent) + tag + " = " + thing;
		if (r) result += "\n";
		return(result);
	} // method

	/** @see #stringify */
	public static String stringify (Object thing, int indent, boolean r) {
		String result = "";
		result += space(indent) + thing.getClass().getName() + " = " + thing;
		if (r) result += "\n";
		return(result);
	} // method

	// Method: calcScreenSize
	/** Returns an int indicating the size of the screen.
	* * Same metrics as in CGX.
	*/
	public static int calcScreenSize () {
		int screensize = SIXbyFOUR;
		if ((screenDimensions().width >= 1600) && (screenDimensions().height >= 1200))
			screensize = SIXTEENbyTWELVE;
		else if ((screenDimensions().width >= 1280) && (screenDimensions().height >= 1024))
			screensize = TWELVEbyTEN;
		else if ((screenDimensions().width >= 1152) && (screenDimensions().height >= 882))
			screensize = ELEVENbyEIGHT;
		else if ((screenDimensions().width >= 1152) && (screenDimensions().height >= 864))
			screensize = ELEVENbyEIGHT;
		else if ((screenDimensions().width >= 1024) && (screenDimensions().height >= 768))
			screensize = TENbySEVEN;
		else if ((screenDimensions().width >= 800) && (screenDimensions().height >= 600))
			screensize = EIGHTbySIX;
		else if ((screenDimensions().width >= 640) && (screenDimensions().height >= 480))
			screensize = SIXbyFOUR;
		else screensize = SIXbyFOUR;
		return(screensize);
	} // method

	// Method: calcWindowSize
	/** Returns a Dimension indicating the default frame size.
	* * Same metrics as in CyclePad.
	*/
	public static Dimension calcWindowSize () {
		Dimension windowsize = new Dimension(600, 400);
		if ((screenDimensions().width >= 1600) && (screenDimensions().height >= 1200))
			windowsize = new Dimension(1550, 1150);
		else if ((screenDimensions().width >= 1280) && (screenDimensions().height >= 1024))
			windowsize = new Dimension(1200, 1000);
		else if ((screenDimensions().width >= 1152) && (screenDimensions().height >= 882))
			windowsize = new Dimension(1100, 800);
		else if ((screenDimensions().width >= 1152) && (screenDimensions().height >= 864))
			windowsize = new Dimension(1100, 800);
		else if ((screenDimensions().width >= 1024) && (screenDimensions().height >= 768))
			windowsize = new Dimension(1000, 700);
		else if ((screenDimensions().width >= 800) && (screenDimensions().height >= 600))
			windowsize = new Dimension(750, 550);
		else if ((screenDimensions().width >= 640) && (screenDimensions().height >= 480))
			windowsize = new Dimension(600, 400);
		else windowsize = new Dimension(600, 400);
		return(windowsize);
	} // method

	/** @see #calcWindowSize() */
	public static Dimension calcWindowSize (Dimension dimensions) {
		Dimension windowsize = new Dimension(600, 400);
		if ((dimensions.width >= 1600) && (dimensions.height >= 1200))
			windowsize = new Dimension(1550, 1150);
		else if ((dimensions.width >= 1280) && (dimensions.height >= 1024))
			windowsize = new Dimension(1200, 1000);
		else if ((dimensions.width >= 1152) && (dimensions.height >= 882))
			windowsize = new Dimension(1100, 800);
		else if ((dimensions.width >= 1152) && (dimensions.height >= 864))
			windowsize = new Dimension(1100, 800);
		else if ((dimensions.width >= 1024) && (dimensions.height >= 768))
			windowsize = new Dimension(1000, 700);
		else if ((dimensions.width >= 800) && (dimensions.height >= 600))
			windowsize = new Dimension(750, 550);
		else if ((dimensions.width >= 640) && (dimensions.height >= 480))
			windowsize = new Dimension(600, 400);
		else if ((dimensions.width >= 320) && (dimensions.height >= 240))
			windowsize = new Dimension(300, 200);
		else windowsize = new Dimension(150, 100);
		return(windowsize);
	} // method

	// Method: center
	/** Centers a region with respect to screen coordinates. */
	public static Point center (Dimension dimension) {
		Point result = null;
		if (dimension != null) {
			calcScreenSize();
			if (dimension.height > screenDimensions().height)
				dimension.height = screenDimensions().height;
			if (dimension.width > screenDimensions().width)
				dimension.width = screenDimensions().width;
			result = new Point((screenDimensions().width - dimension.width) / 2,
									 (screenDimensions().height - dimension.height) / 2);
		} // end if
		return(result);
	} // method

	/** @see #center(Dimension) */
	public static Point center (Dimension outside, Dimension inside) {
		Point result = null;
		Dimension box = new Dimension(inside);
		if ((outside != null) && (inside != null)) {
			if (box.height > outside.height) box.height = outside.height;
			if (box.width > outside.width) box.width = outside.width;
			result = new Point((outside.width - box.width) / 2,
									 (outside.height - box.height) / 2);
		} // end if
		return(result);
	} // method

	/** @see #center(Dimension) */
	public static Point center (Dimension outside, int width, int height) {
		Point result = null;
		Dimension box = new Dimension(width, height);
		if (outside != null) {
			if (box.height > outside.height) box.height = outside.height;
			if (box.width > outside.width) box.width = outside.width;
			result = new Point((outside.width - box.width) / 2,
									 (outside.height - box.height) / 2);
		} // end if
		return(result);
	} // method

	// Method: matchColor
	/** Given the string name of a color, returns the color.
	* * Color names are taken from the list below:
	* @see QRGconstants#RED
	* @see QRGconstants#DARKRED
	* @see QRGconstants#GREEN
	* @see QRGconstants#BRIGHTGREEN
	* @see QRGconstants#MEDIUMGREEN
	* @see QRGconstants#DARKGREEN
	* @see QRGconstants#GRASSGREEN
	* @see QRGconstants#DARKGRASSGREEN
	* @see QRGconstants#LIGHTBLUE
	* @see QRGconstants#BLUE
	* @see QRGconstants#SKYBLUE
	* @see QRGconstants#DARKSKYBLUE
	* @see QRGconstants#ROYALBLUE
	* @see QRGconstants#PURPLE
	* @see QRGconstants#LIGHTYELLOW
	* @see QRGconstants#YELLOW
	* @see QRGconstants#DARKYELLOW
	* @see QRGconstants#LIGHTPINK
	* @see QRGconstants#PINK
	* @see QRGconstants#DARKPINK
	* @see QRGconstants#ORANGE
	* @see QRGconstants#MAGENTA
	* @see QRGconstants#CYAN
	* @see QRGconstants#LAVENDER
	* @see QRGconstants#DARKLAVENDER
	* @see QRGconstants#PLUM
	* @see QRGconstants#DARKPLUM
	* @see QRGconstants#YUCKYPUCE
	* @see QRGconstants#DARKYUCKYPUCE
	* @see QRGconstants#LIGHTGRAY
	* @see QRGconstants#GRAY
	* @see QRGconstants#DARKGRAY
	* @see QRGconstants#LIGHTGREY
	* @see QRGconstants#GREY
	* @see QRGconstants#DARKGREY
	* @see QRGconstants#WHITE
	* @see QRGconstants#GOLD
	* @see QRGconstants#DARKGOLD
	* @see QRGconstants#NUTBROWN
	* @see QRGconstants#LIGHTBROWN
	* @see QRGconstants#REDBROWN
	* @see QRGconstants#REDORANGE
	* @see QRGconstants#GREENBROWN
	* @see QRGconstants#BLACK
	* @see QRGconstants#SLATEGREY
	*/
	public static Color matchColor (String value) {
		Color result = Color.black;
		if (value != null) {
			value = value.trim();
			if (value==null || value.equals("")) result = Color.black;
			else {
				try {
					int foo = Integer.parseInt(value);
					result = new Color(foo);
				} catch (NumberFormatException e) {
					if (value.equalsIgnoreCase("RED"))                 result = RED;
					else if (value.equalsIgnoreCase("DARKRED"))        result = DARKRED;
					else if (value.equalsIgnoreCase("GREEN"))          result = GREEN;
					else if (value.equalsIgnoreCase("BRIGHTGREEN"))    result = BRIGHTGREEN;
					else if (value.equalsIgnoreCase("MEDIUMGREEN"))    result = MEDIUMGREEN;
					else if (value.equalsIgnoreCase("DARKGREEN"))      result = DARKGREEN;
					else if (value.equalsIgnoreCase("GRASSGREEN"))     result = GRASSGREEN;
					else if (value.equalsIgnoreCase("DARKGRASSGREEN")) result = DARKGRASSGREEN;
					else if (value.equalsIgnoreCase("LIGHTBLUE"))      result = LIGHTBLUE;
					else if (value.equalsIgnoreCase("BLUE"))           result = BLUE;
					else if (value.equalsIgnoreCase("SKYBLUE"))        result = SKYBLUE;
					else if (value.equalsIgnoreCase("DARKSKYBLUE"))    result = DARKSKYBLUE;
					else if (value.equalsIgnoreCase("ROYALBLUE"))      result = ROYALBLUE;
					else if (value.equalsIgnoreCase("PURPLE"))         result = PURPLE;
					else if (value.equalsIgnoreCase("LIGHTYELLOW"))    result = LIGHTYELLOW;
					else if (value.equalsIgnoreCase("YELLOW"))         result = YELLOW;
					else if (value.equalsIgnoreCase("DARKYELLOW"))     result = DARKYELLOW;
					else if (value.equalsIgnoreCase("LIGHTPINK"))      result = LIGHTPINK;
					else if (value.equalsIgnoreCase("PINK"))           result = PINK;
					else if (value.equalsIgnoreCase("DARKPINK"))       result = DARKPINK;
					else if (value.equalsIgnoreCase("ORANGE"))         result = ORANGE;
					else if (value.equalsIgnoreCase("MAGENTA"))        result = MAGENTA;
					else if (value.equalsIgnoreCase("CYAN"))           result = CYAN;
					else if (value.equalsIgnoreCase("LAVENDER"))       result = LAVENDER;
					else if (value.equalsIgnoreCase("DARKLAVENDER"))   result = DARKLAVENDER;
					else if (value.equalsIgnoreCase("PLUM"))           result = PLUM;
					else if (value.equalsIgnoreCase("DARKPLUM"))       result = DARKPLUM;
					else if (value.equalsIgnoreCase("YUCKYPUCE"))      result = YUCKYPUCE;
					else if (value.equalsIgnoreCase("DARKYUCKYPUCE"))  result = DARKYUCKYPUCE;
					else if (value.equalsIgnoreCase("LIGHTGRAY"))      result = LIGHTGRAY;
					else if (value.equalsIgnoreCase("GRAY"))           result = GRAY;
					else if (value.equalsIgnoreCase("DARKGRAY"))       result = DARKGRAY;
					else if (value.equalsIgnoreCase("LIGHTGREY"))      result = LIGHTGREY;
					else if (value.equalsIgnoreCase("GREY"))           result = GREY;
					else if (value.equalsIgnoreCase("DARKGREY"))       result = DARKGREY;
					else if (value.equalsIgnoreCase("WHITE"))          result = WHITE;
					else if (value.equalsIgnoreCase("GOLD"))           result = GOLD;
					else if (value.equalsIgnoreCase("DARKGOLD"))       result = DARKGOLD;
					else if (value.equalsIgnoreCase("NUTBROWN"))       result = NUTBROWN;
					else if (value.equalsIgnoreCase("LIGHTBROWN"))     result = LIGHTBROWN;
					else if (value.equalsIgnoreCase("REDBROWN"))       result = REDBROWN;
					else if (value.equalsIgnoreCase("REDORANGE"))      result = REDORANGE;
					else if (value.equalsIgnoreCase("GREENBROWN"))     result = GREENBROWN;
					else if (value.equalsIgnoreCase("SLATEGREY"))      result = SLATEGREY;
					else result = BLACK;
				} // end try/catch
			} // end if else
		} // if value
		return result;
	} // method

	// Method: beep
	/** Tweeks the fweeper. */
	public static void beep () { Toolkit.getDefaultToolkit().beep(); }

	// Image Utilities
	// ----------------------------------------------------------------------
	// Method: getImage, getImageIcon
	/** Loads images. */
	public static Image getImage (URL location) {
		return(Toolkit.getDefaultToolkit().getImage(location));
	} // method

	/** @see #getImage(URL) */
	public static Image getImage (String filename) {
		Image result = null;
		URL url = null;
		if ((filename != null) && (filename.length() > 0)) {
			if (isApplet()) {
				try {
					url = new URL(filename);
					result = getImage(url);
				} catch(Exception e) { }
			} else result = Toolkit.getDefaultToolkit().getImage(filename);
		} // end if filename
		return(result);
	} // method

	/** @see #getImage(URL) */
	public static ImageIcon getImageIcon (URL location) {
		return(new ImageIcon(location));
	} // method

	/** @see #getImage(URL) */
	public static ImageIcon getImageIcon (String filename) {
		ImageIcon result = null;
		URL url = null;
		if (isApplet()) {
			try {
				url = new URL(filename);
				result = getImageIcon(url);
			} catch(Exception e) { }
		} else result = new ImageIcon(filename);
		return(result);
	} // method

	// Debugging Utilities.
	// -----------------------------------------------------------------------
	// Debugging Properties
	/** Stash for debugging log file.
	* Default: DEFAULT_DEBUG_FILE;
	* * Do not access directly. USE THE ACCESSORS.
	* @see QRGconstants#DEFAULT_DEBUG_FILE
	* @see #debugfile()
	* @see #debugfile(String)
	*/
	protected static String debugfile = DEFAULT_DEBUG_FILE;
	/** @see QRGconstants#DEFAULT_DEBUG_FILE
	* @see #debugfile
	*/
	public static String debugfile () {
		if (debugfile == null) debugfile = DEFAULT_DEBUG_FILE;
		return(debugfile);
	} // reader
	/** @see #debugfile */
	public static String debugfile (String df) { return(debugfile = df); }

	/** Maximum number of entries in the debugging log.
	* Default: DEFAULT_DEBUG_LIMIT;
	* * Do not access directly. USE THE ACCESSORS.
	* @see QRGconstants#DEFAULT_DEBUG_LIMIT
	* @see #debuglimit()
	* @see #debuglimit(int)
	*/
	protected static int debuglimit = DEFAULT_DEBUG_LIMIT;
	/** @see #debuglimit */
	public static int debuglimit () { return(debuglimit); }
	/** @see #debuglimit */
	public static int debuglimit (int l) { return(debuglimit = l); }

	/** Is debugging on / off?
	* Default: DEFAULT_DEBUG_FLAG   (false)
	* * Do not access directly. USE THE ACCESSORS.
	* @see QRGconstants#DEFAULT_DEBUG_FLAG
	* @see #debugflag()
	* @see #debugflag(boolean)
	*/
	protected static boolean debugflag = DEFAULT_DEBUG_FLAG;
	/** @see #debugflag */
	public static boolean debugflag () { return(debugflag); }
	/** @see #debugflag */
	public static boolean debugflag (boolean b) { return(debugflag = b); }

	/** Stash for the debugging file writer stream.
	* * Do not access directly. USE THE ACCESSORS.
	* @see #debugfw()
	* @see #debugfw(FileWriter)
	*/
	protected static FileWriter debugfw = null;
	/** @see #debugfw */
	public static FileWriter debugfw () { return(debugfw); }
	/** @see #debugfw */
	public static FileWriter debugfw (FileWriter fw) { return(debugfw = fw); }

	/** Stash for the debugging print writer stream.
	* * Do not access directly. USE THE ACCESSORS.
	* @see #debugpw()
	* @see #debugpw(PrintWriter)
	*/
	protected static PrintWriter debugpw = null;
	/** @see #debugpw */
	public static PrintWriter debugpw () { return(debugpw); }
	/** @see #debugpw */
	public static PrintWriter debugpw (PrintWriter pw) { return(debugpw = pw); }

	/** Returns true if debugging is turned on. */
	public static boolean debug () { return(debugflag()); }

	/** Starts/Stops debugging to a file with a file size limit. */
	public static boolean debug (boolean flag, String filename, int limit) {
		debuglimit(limit);
		return(debug(flag, filename));
	} // method

	/** Starts/Stops debugging to a file. */
	public static boolean debug (boolean flag, String filename) {
		if (debugflag() && flag) debug(false);
		debugfile(filename);
		return(debug(flag));
	} // method

	/** Starts/Stops debugging. */
	public static boolean debug (boolean flag) {
		if (flag != debugflag()) {
			if (debugflag() && !flag) debug("Stop Debugging", "flag=" + flag + " old=" + debugflag + " --------------------------------");
			debugflag(flag);
			if (debugflag()) {
				if ((!isApplet()) && (debugfw() == null) && (debugpw() == null)) {
					try {
						boolean appendflag = false;
						File foo = new File(debugfile());
						if (foo.exists() && (foo.length() < debuglimit())) appendflag=true;
						debugfw(new FileWriter(debugfile(), appendflag));
						debugpw(new PrintWriter(debugfw()));
						debug("Start Debugging", "--------------------------------");
					} catch (Exception e) {
						System.out.println("Debug Error: \n" + e);
						e.printStackTrace();
						debugflag(false);
					} // catch
				} // end if
			} else {
				try {
					if (!isApplet()) {
						debugpw().close();
						debugfw().close();
						debugpw(null);
						debugfw(null);
					} // end if
				} catch (Exception e) {
					System.out.println("debug Error: \n" + e);
					e.printStackTrace();
					debugflag(false);
				} // catch
			} // end if else
		} // end if
		return(debugflag);
	} // method

	/** Print debugging message. */
	public static boolean debug(String info) {
		Date currentTime = null;
		String datetime = "";
		if (debugflag()) {
			try {
				if (debugpw() == null) debug(true);
				currentTime = new Date();
				datetime = dateformat().format(currentTime);
				if (TESTRUN) System.out.println(datetime + info);
				if (!isApplet()) {
					debugpw().println(datetime + info);
					debugpw().flush();
				} // end if
			} catch (Exception e) {
				System.out.println("DEBUGGER Error: \n" + e);
				e.printStackTrace();
				debug(false);
			} // catch
		} // end if
		return(debugflag());
	} // method

	/** @see #debug(String) */
	public static boolean debug(String id, Object info) {
		return(debug(id + " " + info));
	} // method

	/** @see #debug(String) */
	public static boolean debug(String klass, String method, Object info) {
		return(debug(klass + " " + method + " " + info));
	} // method

	/** Print debugging stack trace. */
	public static void debugStackTrace(Exception e) {
		e.printStackTrace(debugpw());
		e.printStackTrace();
	} // method

	public static void debugStackTrace(Throwable e) {
		e.printStackTrace(debugpw());
		e.printStackTrace();
	} // method

	public static void debugStackTrace(String msg, Exception e) {
		debug(msg);
		e.printStackTrace(debugpw());
		e.printStackTrace();
	} // method

	/** Print debugging message & pop-up warning dialog. */
	public static void warn (String id, Object info) {
		if (!debugflag()) debug(true);
		debug(id, info);
		JOptionPane.showMessageDialog(null,
												"WARNING " + id + ": " + info,
												"WARNING", JOptionPane.ERROR_MESSAGE);
	} // method

	/** Print debugging message & pop-up error dialog. */
	public static void error (String id, Object info) {
		if (!debugflag()) debug(true);
		debug(id, info);
		JOptionPane.showMessageDialog(null,
												"ERROR " + id + ": " + info,
												"ERROR", JOptionPane.ERROR_MESSAGE);
	} // method

	/** Print debugging message, pop-up error dialog, & force exit. */
	public static void fatalError (String id, Object info) {
		if (!debugflag()) debug(true);
		debug(id, info);
		debug(false);
		JOptionPane.showMessageDialog(null,
												"FATAL ERROR " + id + ": " + info,
												"FATAL ERROR", JOptionPane.ERROR_MESSAGE);
		System.exit(-1);
	} // method

} // class QRGutils

// ---------------------------------------------------------------------------
// End of File: QRGutils.java                    2003 Northwestern University