package edu.qrg.utils;

import java.io.DataInputStream;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.PrintWriter;
import java.lang.Class;
import java.lang.ClassNotFoundException;
import java.lang.Exception;
import java.lang.IllegalAccessException;
import java.lang.InstantiationException;
import java.lang.Object;
import java.lang.String;
import java.lang.System;
import java.net.URL;
import java.util.Vector;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/utils
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: XMLio.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: XMLio
* Purpose: Quick & Dirty Load and Save objects in the XML format.
* ----------------------------------------------------------------------------
* NOTE: The following constructor must be in any class implementing
*       the XMLio.XMLable:
*
*       public Constructor CLASSNAME (Vector attributes, Vector elements)
*
*       Java, of course, does not allow programmers to specify constructor
*       arguments in an interface.  Silly me for wanting to.
* ----------------------------------------------------------------------------
* NOTE: THIS IS NOT A COMPLETE XML PARSER/FORMATTER - SIMPLY Q&D.
* ----------------------------------------------------------------------------
* </PRE> */
public class XMLio {
	// Interface
	//-------------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Interface: XMLio.XMLable
	* Purpose: Objects implementing this interface may be saved/loaded by the
	*          XMLio class methods.  This interface insures the implementing
	*          objects contain their own stringification methods.
	* ------------------------------------------------------------------------
	* </PRE>
	*/
	public interface XMLable {
		/** Stringifies a XML definition for the implementing class. */
		public String xmlDef ();

		/** Stringifies the implementing class into XML format. */
		public String xmlify ();

		/** Initializes slots after object is created by XML loader. */
		public void XMLioObjectInit (Vector attr, Vector elmnt);

		/** Save the calling object to filename. */
		public void exportXML (String filename);
	} // interface

	// Properties & Accessors
	// -----------------------------------------------------------------------
	/** Stash for KeyValuePairs while parsing file.
	* * This property is static so that objects may be loaded statically.
	* * Do not read/write directly. Use the accessors.
	*/
	protected static Vector attributes = new Vector();
	/** @see #attributes */
	public static Vector attributes () { return(attributes); }
	/** @see #attributes */
	public static Vector attributes (Vector v) { return(attributes = v); }

	/** Stash for result of readElement while parsing file.
	* * This property is static so that objects may be loaded statically.
	* * Do not read/write directly. Use the accessors.
	*/
	protected static XMLio.XMLable readElementResult = null;
	/** @see #readElementResult */
	public static XMLio.XMLable readElementResult () { return(readElementResult); }
	/** @see #readElementResult */
	public static XMLio.XMLable readElementResult (XMLio.XMLable x) {
		return(readElementResult = x);
	} // writer readElementResult

	/** Stash for filename.  Used to print error messages.
	* Because there is no way to get filename from stream.
	* * This property is static so that objects may be loaded statically.
	* * Do not read/write directly. Use the accessors.
	*/
	protected static String fname = null;
	/** @see #fname */
	public static String fname () { return(fname); }
	/** @see #fname */
	public static String fname (String s) { return(fname = s); }

	// HACK!
	public static String packagePrefix = "edu.qrg.vmodel.";
	public static String packagePrefix () { return(packagePrefix); }
	public static String packagePrefix (String s) { return(packagePrefix = s); }

	// Methods
	// -----------------------------------------------------------------------
	/** Quick & Dirty XML comment wrapper.
	* * This method is static so that objects may be encoded statically.
	* * Does not check for illegal characters.
	* * Does not format text.
	*/
	public static String commentate (String text) {
		return("<!-- " + text + " -->");
	} // method

	// Loading.
	// -----------------------------------------------------------------------
	/** Reads XML key-value pairs from input stream.
	* * This method is static so that objects may be loaded statically.
	* ---------------------------------------------------------------------
	*/
	public static int readAttributes(InputStream inp) {
		int c = 0;
		String attributeTag = "", attributeValue = "";
		while((c != -1) && (c != 0x2F)) {
			attributeTag = "";
			attributeValue="";
			c = QRGutils.gobbleWhiteSpace(inp);
			while (legalChar(c)) {
				attributeTag += (char) c;
				try {c = inp.read();}
				catch (Exception e) {c = -1;}
			} // while
			if ((c == 0x3E) || (c == -1) || (c == 0x2F)) return(c);
			if ((c != -1) && (c != 0x3D)) c = QRGutils.gobbleWhiteSpace(inp);
			if ((c != 0X3D)) return(c); // error
			c = QRGutils.gobbleWhiteSpace(inp);
			if ((c != 0x22)) return(c); //error
			try {c = inp.read();}
			catch (Exception e) {c = -1;}
			while ((c != -1) && (c != 0x22)) {
				attributeValue = attributeValue + (char) c;
				try {c = inp.read();}
				catch (Exception e) {c = -1;}
			} // end while
			attributes.addElement(new KeyValuePair(attributeTag, attributeValue));
		} // end while
		return c;
	} // method readAttributes

	/** Gobbles whitespace until the beginning of an attribute tag.
	* * This method is static so that objects may be loaded statically.
	*/
	public static int gobbleUntilTag (InputStream inp) {
		int c = 0;
		try {while ((c != -1) && (c != 0x3C)) c = inp.read();}
		catch (Exception e) {c = -1;}
		return(c);
	} // method gobbleUntilTag

	/** Ensures a character is a legal XML character.
	* * This method is static so that objects may be loaded statically.
	*/
	public static boolean legalChar (int c) {
		return (((c >= 0x30) && (c <=0x39))||
			     ((c >= 0x41) && (c <=0x5A))||
			     ((c >= 0x61) && (c <= 0x7A)) ||
			     (c == 0x5F) || (c == 0x2D)); // Why is this 2D?
	} // method legalChar

	/** Ensures a character is valid first character
	*   in an XML identifier.
	* * This method is static so that objects may be loaded statically.
	*/
	public static boolean legalFirstChar (int c) {
		return (((c >= 0x41) && (c <=0x5A)) ||
		        ((c >= 0x61) && (c <= 0x7A)) ||
		         (c == 0x5F) || (c == 2F)); // and this is 2F?
	} // method legalFirstChar

	/** Gobbles whitespace until the first legal character
	*   of an attribute tag.
	* * This method is static so that objects may be loaded statically.
	* ------------------------------------------------------------------------
	*/
	public static int gobbleUntilLegalFirstChar (InputStream inp) {
		int c = 0;
		boolean flag = true;
		try {
			while((c!=-1) && flag && (c != 2F)) {
				c = gobbleUntilTag(inp);
				if (c != -1) {
				try {c = inp.read();}
				catch (Exception e) {c = -1;}
				flag = (!legalFirstChar(c));
				if (c == 0x2F) flag = false;
		   		} // end if c != 1
			} // end while
		} catch (Exception e) {c = -1;}
		return(c);
	} // method gobbleUntilLegalFirstChar

	/** Creates an instance of a class specified by the tag.
	* * This method is static so that objects may be loaded statically.
	*/
	public static XMLio.XMLable makeInstance(String classname, Vector attributes, Vector subElements) {
		XMLio.XMLable result = null;
		Class tagClass = null;
		// Guard against null pointer errors.  Could be done in accessor?
		if (attributes == null) attributes = new Vector();
		if (subElements == null) subElements = new Vector();
		// 1. Find the class.
		try {
        tagClass = Class.forName(classname);
      } catch (ClassNotFoundException e) {
		try {
				// need to make a default class.... PList?
	        	tagClass = Class.forName(packagePrefix() + classname);
	      } catch (ClassNotFoundException e1) {
			try {
					// need to make a default class.... PList?
		        	tagClass = Class.forName("edu.qrg.utils." + classname);
		      } catch (ClassNotFoundException ex) {
		      	QRGutils.warn("XMLio.makeInstance", "Class Not Found Error.\r\n" +
		      						"File=" + fname() +
		      						".\r\nClass=" + classname + ".");
				try {
						// need to make a default class.... PList?
			        	tagClass = Class.forName("PList");
			      } catch (ClassNotFoundException e2) {
						QRGutils.debugStackTrace(e2);
			        	System.exit(-1);
			      } // try - catch
		      } // try - catch
	      } // try - catch
      } // try - catch
		// 2. Call the constructor
		// Note: PList had better not be the default class, since it doesn't
		// implement XMLable, so we get a class cast exception. (TRH)
		try {
        result = (XMLio.XMLable) tagClass.newInstance();
        result.XMLioObjectInit(attributes, subElements);
      } catch (IllegalAccessException e3) {
			QRGutils.debugStackTrace(e3);
        System.exit(-3);
      } catch (InstantiationException e6) {
			QRGutils.debugStackTrace(e6);
        System.exit(-6);
      } // try - catch
      return(result);
	} // method

	/** Reads a single xml object and objects contained therein.
	* * This method is static so that objects may be loaded statically.
	*/
	public static int readElement(InputStream inp) throws Exception {
		int c = 0;
		String elementTag = "";
		Object result = null;
		Vector attributeCache = null;
		Vector subElements = new Vector();
		if (inp != null) {
			c = gobbleUntilLegalFirstChar(inp);
			if (c != 0x2F) {
				if ((c != 0x2F) && (c != -1)) {
					elementTag = "";
					while (((c >= 0x30) && (c <=0x39))||
					       ((c >= 0x41) && (c <=0x5A))||
					       ((c >= 0x61) && (c <= 0x7A)) ||
					        (c == 0x5F) || (c == '.')) {
						elementTag += (char) c;
						try {c = inp.read();}
						catch (Exception e) {c = -1;}
					} // while
					attributes(new Vector());
					if (c != 0x3E) c = readAttributes(inp);
					attributeCache = attributes();
					while((c != -1) && (c != 0x2F)) {
						readElementResult(null);
						c = readElement(inp);
						if (readElementResult() != null) {
							subElements.addElement(readElementResult());
						} // end if
					} // while
				} // end if
				if (c == 0x2F) c = 0x20;
				readElementResult(makeInstance(elementTag, attributeCache, subElements));
			} // end if
		} else QRGutils.debug("XMLio.readElement", "inp=null");
		return c;
	} // method readElement

	/** Load from a stream.
	* * This method is static so that objects may be loaded statically.
	*/
	public static XMLio.XMLable load (InputStream inp) {
		XMLio.XMLable result = null;
		try {
			readElement(inp);
			result = readElementResult();
		} catch (Exception e) {
			// Perhaps we shouldn't warn here because this isn't always fatal.
		   QRGutils.warn("XMLio.load(InputStream)","Load Error: \n" + e);
		   QRGutils.debugStackTrace(e);
		} // try - catch
		return(result);
	} // method load

	/** Load from a file specified by filename.
	* * This method is static so that objects may be loaded statically.
	*/
	public static XMLio.XMLable load (String filename) {
		XMLio.XMLable result = null;
		InputStream inp = null;
		URL url = null;
		fname(filename);
		try {
			if (QRGutils.isApplet()) {
   			url = new URL(filename);
				if (url != null) inp = url.openStream();
				if (inp != null) inp = new DataInputStream(inp);
			} else inp = new FileInputStream(filename);
			if (inp != null) {
				//readElement(inp);
				//result = lastElementRead;
				result = load(inp);
				inp.close();
			} else QRGutils.debug( "XMLio.load", "FileInputStream is null." );
		} catch (Exception e) {
		   QRGutils.warn("XMLio.load(String)","Load Error: \n" + e);
		   QRGutils.debugStackTrace(e);
		} // catch
		fname(null);
		return(result);
	} // method load

	// Saving.
	// -----------------------------------------------------------------------
	/** Save an object in XML format to the specified file.
	* * This method is static so that objects may be saved statically.
	*/
	public static void save (String filename, XMLio.XMLable thing) {
		String header = "<!DOCTYPE VMODEL SYSTEM \"vmodel.dtd\">\r\n";
		save(filename, header, thing);
	} // method save

	/** Save an object in XML format to the specified file.
	* * This method is static so that objects may be saved statically.
	* * Header is used for doctype but seems arcane.
	*/
	public static XMLio.XMLable save (String filename, String header, XMLio.XMLable thing) {
		XMLio.XMLable result = null;
		try {
			FileWriter fw = new FileWriter(filename);
			PrintWriter pw = new PrintWriter(fw);
			pw.println(
			"<?xml version = \"1.0\" encoding = \"UTF-8\" standalone = \"yes\" ?>\r\n"
			+ "<?" + QRGutils.projectName() + " version = \"" + QRGutils.projectVersion() + "\" ?>\r\n");
			pw.println(thing.xmlDef() + "\r\n");
			pw.print(thing.xmlify());
			pw.close();
			fw.close();
			result = thing;
		} catch (Exception e) {
			QRGutils.debug("Save XML", " Error: \n" + e);
			QRGutils.debugStackTrace(e);
		   //  Potential leak here.
		} // try catch
		return(result);
	} // method save

	/** Save an object in XML format to the specified stream.
	* * This method is static so that objects may be saved statically.
	*/
	public static XMLio.XMLable save (FileOutputStream fout, XMLio.XMLable thing) {
		XMLio.XMLable result = null;
		try { result = save(fout.getFD(), thing);}
		catch (Exception e) {
		   QRGutils.debug("Save XML", " Error: \n" + e);
			QRGutils.debugStackTrace(e);
		} // try catch
		return(result);
	} // method save

	/** Save an object in XML format to the specified file descripter.
	* * This method is static so that objects may be saved statically.
	*/
	public static XMLio.XMLable save (FileDescriptor fout, XMLio.XMLable thing) {
		XMLio.XMLable result = null;
		try {
			FileWriter fw = new FileWriter(fout);
			PrintWriter pw = new PrintWriter(fw);
			pw.print(thing.xmlify());
			pw.close();
			fw.close();
			result = thing;
		} catch (Exception e) {
		   QRGutils.debug("Save XML", " Error: \n" + e);
			QRGutils.debugStackTrace(e);
		   //  Potential leak here.
		} // try catch
		return(result);
	} // method save
} // class XMLio

// ---------------------------------------------------------------------------
// End of File: XMLio.java                       2003 Northwestern University